# -*- coding: utf-8 -*-
from osv import osv, fields
import decimal_precision as dp
import netsvc
import pooler
from tools.sql import drop_view_if_exists

class res_partner(osv.osv):
    _inherit = 'res.partner'
    
    def check_vat_xx(self, vat):
        return True
    
res_partner()

class account_journal(osv.osv):
    _inherit = "account.journal"
    _columns = {
        'code': fields.char('Code', size=64, required=True, \
         help="The code will be used to generate the numbers of the journal entries of this journal."),
    }

account_journal()


class account_aged_trial_balance(osv.osv_memory):
    _inherit = 'account.aged.trial.balance'
    
    _defaults = {
        'result_selection': lambda *a: 'supplier',
    }

account_aged_trial_balance()

class account_invoice_refund(osv.osv_memory):

    """Refunds invoice"""
    
    """Para sobre escribir el wizard de facturas rectificativas y poder usar 
    siempre por defecto 'Facturas rectificativas'.
    
    Tambien se hereda para colocar en 0.0 el Total al generar una factura
    rectificativa."""

    _inherit = "account.invoice.refund"
    _description = "Sobreescribir Factura Rectificativas"
    
    _defaults = {
        'filter_refund': 'refund',
    }

    def compute_refund(self, cr, uid, ids, mode='refund', context=None):
        """
        Super de compute_refund para colocar las facturas rectificativas con el
        monto check_total en 0
        """
        result = super(account_invoice_refund, self).compute_refund(cr, uid, ids, mode, context)
        inv_obj = self.pool.get('account.invoice')
        if result.get('domain', False):
            for res in result['domain']:
                if res[0] == 'id':
                    inv_obj.write(cr, uid, res[2], {'check_total':0.0}, context)
                    
        return result

account_invoice_refund()

class account_invoice(osv.osv):
    """Se hereda account.invoice para modificar el campo wh_iva_rate, creado 
    por l10n_ve_withholding_iva y colocarlo siempre en readonly, esto porque 
    ese campo se debe llenar por defecto segun el proveedor/cliente"""
    _inherit = "account.invoice"
    
    
    _columns = {
        'wh_iva_rate': fields.float('Wh rate', digits_compute= dp.get_precision('Withhold'), readonly=True, help="Withholding vat rate"),
    }
    
    
    def copy(self, cr, uid, id, default=None, context=None):
        if default is None:
            default = {}
        default = default.copy()
        default.update({'wh_src_id': False, 'wh_src_rate': 0.00})
        return super(account_invoice, self).copy(cr, uid, id, default, context)
    
    
    def validar_button_compute(self, cr, uid, ids):
        #Verificar lo registrado con el boton calcular
        if isinstance(ids,(int, long)):
            inv_id= [ids]
        else:
            inv_id= ids
        self.button_compute(cr, uid, inv_id, set_total=False)
        ai_brw = self.browse(cr, uid, ids)
        for inv in ai_brw:
            if inv.type in ('in_invoice', 'in_refund') and abs(inv.check_total - inv.amount_total) >= (inv.currency_id.rounding/2.0):
                raise osv.except_osv('Total errado !', 'Por favor verifique si ha presionado el boton calcular antes de validar la factura. Los totales pueden ser diferentes.')

        return True


    def validar_campos_obligatorios(self, cr, uid, ids):
        """Funcion para validar que la factura tenga el campo nro_ctrl y
        reference con alguna informacion, en caso de que por vista no se 
        active la obligatoriedad de los campos"""
        
        ai_brw = self.browse(cr, uid, ids)
        for ai in ai_brw:
            if (ai.type in ('in_invoice','in_refund')) and (not ai.nro_ctrl or not ai.reference):
                raise osv.except_osv('Error',"Falta por registrar campo 'Numero de Control' o 'Referencia' en la factura")
        return True

    def validar_account_invoice_refund(self, cr, uid, ids):
        """Funcion para validar que la factura rectificativa de proveedor no se le apruebe si el monto total
        de la factura rectificativa de proveedor es mayor a la factura origen"""
        ai_brw = self.browse(cr, uid, ids)
        for ai in ai_brw:
            if ai.parent_id and ai.type == 'in_refund':
                ai_rect_list = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',ai.parent_id.id),('type','=','in_refund')])
                ai_rect_brw = self.browse(cr, uid, ai_rect_list)
                monto_total = 0
                for fact_rec in ai_rect_brw:
                    monto_total += fact_rec.amount_total
                #~ Busca las notas de debito 
                ai_rect_list = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',ai.parent_id.id),('type','=','in_invoice')])
                ai_rect_brw = self.browse(cr, uid, ai_rect_list)
                monto_db_total = 0
                for fact_rec in ai_rect_brw:
                    monto_db_total += fact_rec.amount_total
                #~ Se calcula el verdadero monto total
                monto_total = monto_total - monto_db_total
                #~ Se validad el resultado
                if (monto_total - ai.parent_id.amount_total) > 0.0001:
                    raise osv.except_osv('Error con Factura Rectificativa de Proveedor',"El monto total de la factura origen '%s', es menor que la factura rectificativa que intenta aprobar."%ai.parent_id.number)
                    
            elif ai.parent_id and ai.type == 'out_refund':
                ai_rect_list = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',ai.parent_id.id),('type','=','out_refund')])
                ai_rect_brw = self.browse(cr, uid, ai_rect_list)
                monto_total = 0
                for fact_rec in ai_rect_brw:
                    monto_total += fact_rec.amount_total
                #~ Busca las notas de debito 
                ai_rect_list = self.pool.get('account.invoice').search(cr, uid, [('parent_id','=',ai.parent_id.id),('type','=','out_invoice')])
                ai_rect_brw = self.browse(cr, uid, ai_rect_list)
                monto_db_total = 0
                for fact_rec in ai_rect_brw:
                    monto_db_total += fact_rec.amount_total
                #~ Se calcula el verdadero monto total
                monto_total = monto_total - monto_db_total
                #~ Se validad el resultado
                if (monto_total - ai.parent_id.amount_total) > 0.0001:
                    raise osv.except_osv('Error con Factura Rectificativa de Cliente',"El monto total de la factura origen '%s', es menor que la factura rectificativa que intenta aprobar."%ai.parent_id.number)
            
        return True
    
    def validar_account_invoice_line(self, cr, uid, ids):
        """Funcion para validar que la factura no se le apruebe si unas de las
        lineas de la factura no tiene impuesto asociado o el precio unitario es
        menor o igual a 0"""
        ai_brw = self.browse(cr, uid, ids)
        for ail in ai_brw[0].invoice_line:
            if (ail.price_unit < 0) and (ai_brw[0].type not in ['in_invoice']):
                raise osv.except_osv('Error con Monto',"En la linea '%s' tiene un monto negativo, que no se permite en este tipo de facturas."%ail.name)
            elif ail.invoice_line_tax_id == [] and ail.price_unit >= 0:
                raise osv.except_osv('Error con Impuestos',"Se debe asignar impuesto a '%s', asi este sea impuesto exento"%ail.name)
            elif (ail.invoice_line_tax_id != []) and (ail.price_unit < 0) and (ai_brw[0].type in ['in_invoice']):
                raise osv.except_osv('Error con Impuestos',"No se debe asignar impuesto a '%s', para evitar error en libro de compra/retencion"%ail.name)
            elif ail.concept_id.withholdable and (ail.price_unit < 0) and (ai_brw[0].type in ['in_invoice']):
                raise osv.except_osv('Error con islr',"Se debe asignar concepto de retencion 'NO APLICA RETENCION' a la linea '%s', por tener un monto negativo."%ail.name)
        return True
        

    def action_date_assign(self, cr, uid, ids, *args):
        #Se hereda la función action_date_assign para llamar funcion de validacion
        self.validar_button_compute(cr, uid, ids)
        self.validar_campos_obligatorios(cr, uid, ids)
        self.validar_account_invoice_refund(cr, uid, ids)
        self.validar_account_invoice_line(cr, uid, ids)
        return super(account_invoice, self).action_date_assign(cr, uid, ids, args)

account_invoice()

class txt_iva(osv.osv):
    _inherit = "txt.iva"
    
    def action_generate_lines_txt(self,cr,uid,ids,context={}):
        """Sobreescritura de la funcion action_generate_lines_txt del modulo
        l10n_ve_withholding_iva para modificar forma en que se genera las lineas
        de retencion.""" 
        voucher_obj = self.pool.get('account.wh.iva')
        txt_iva_obj = self.pool.get('txt.iva.line')
        voucher_ids=''
        txt_brw= self.browse(cr,uid,ids[0])
        txt_ids = txt_iva_obj.search(cr,uid,[('txt_id','=',txt_brw.id)])
        if txt_ids:
            txt_iva_obj.unlink(cr,uid,txt_ids)
        
        if txt_brw.type:
            voucher_ids = voucher_obj.search(cr,uid,[('date_ret','>=',txt_brw.date_start),('date_ret','<=',txt_brw.date_end),('period_id','=',txt_brw.period_id.id),('state','=','done'),('type','in',['in_invoice','in_refund'])])
        else:
            voucher_ids = voucher_obj.search(cr,uid,[('date_ret','>=',txt_brw.date_start),('date_ret','<=',txt_brw.date_end),('period_id','=',txt_brw.period_id.id),('state','=','done'),('type','in',['out_invoice','out_refund'])])
        
        for voucher in voucher_obj.browse(cr,uid,voucher_ids):
            
            for voucher_lines in voucher.wh_lines:
                
                if voucher_lines.invoice_id.state in ['open','paid']:
                    
                    for voucher_tax_line in voucher_lines.tax_line:
                        txt_iva_obj.create(cr,uid,
                        {'partner_id':voucher.partner_id.id,
                        'voucher_id':voucher.id,
                        'invoice_id':voucher_lines.invoice_id.id,
                        'txt_id': txt_brw.id,
                        'untaxed': voucher_tax_line.base,
                        'amount_withheld': voucher_tax_line.amount_ret,
                        'tax_wh_iva_id': voucher_tax_line.id,
                        })
        return True
        
    def get_alicuota_max(self, cr, uid, txt_line):
        """Obtener maxima alicuota por factura"""
        list = []
        for tax_line in txt_line.invoice_id.tax_line:
            list.append(int(tax_line.tax_id.amount * 100))
        return max(list)
        
    
    def get_amount_line(self, cr, uid, txt_line, amount_exempt):
        """Funcion para calcular el monto total"""
        ali_max = self.get_alicuota_max(cr, uid, txt_line)
        exento = 0
        
        if ali_max == int(txt_line.tax_wh_iva_id.tax_id.amount * 100):
            exento = amount_exempt
        total = txt_line.tax_wh_iva_id.base + txt_line.tax_wh_iva_id.amount + exento
        return total, exento
    
    
    def get_alicuota(self,cr,uid,txt_line):
         return int(txt_line.tax_wh_iva_id.tax_id.amount * 100)
         
    def get_type_document(self,cr,uid,txt_line):
        type= '03'
        if txt_line.invoice_id.type in ['out_invoice','in_invoice'] and txt_line.invoice_id.parent_id:
            type= '02'
        elif txt_line.invoice_id.type in ['out_invoice','in_invoice']:
            type= '01'
        return type
         
    
    def generate_txt(self,cr,uid,ids,context={}):
        txt_string = ''
        for txt in self.browse(cr,uid,ids,context):
            vat = txt.company_id.partner_id.vat[2:]
            for txt_line in txt.txt_ids:
                
                vendor,buyer=self.get_buyer_vendor(cr,uid,txt,txt_line)
                period = txt.period_id.name.split('/')
                period2 = period[1]+period[0]
                
                operation_type = 'V' if txt_line.invoice_id.type in ['out_invoice','out_refund'] else 'C'
                document_type  = self.get_type_document(cr,uid,txt_line)
                document_number=self.get_document_number(cr,uid,ids,txt_line,'inv_number')
                control_number = self.get_number(cr,uid,txt_line.invoice_id.nro_ctrl,'inv_ctrl',20)
                document_affected= self.get_document_affected(cr,uid,txt_line)
                voucher_number = self.get_number(cr,uid,txt_line.voucher_id.number,'vou_number',14)
                amount_exempt,amount_untaxed = self.get_amount_exempt_document(cr,uid,txt_line)
                alicuota = self.get_alicuota(cr,uid,txt_line)
                amount_total,amount_exempt = self.get_amount_line(cr, uid, txt_line, amount_exempt)

                txt_string= txt_string + buyer +'\t'+period2.strip()+'\t'\
                 +txt_line.invoice_id.date_invoice+'\t'+operation_type+'\t'+document_type+'\t'+vendor+'\t'\
                 +document_number+'\t'+control_number+'\t'+str(round(amount_total,2))+'\t'\
                 +str(round(txt_line.untaxed,2))+'\t'\
                 +str(round(txt_line.amount_withheld,2))+'\t'+document_affected+'\t'+voucher_number+'\t'\
                 +str(round(amount_exempt,2))+'\t'+str(alicuota)+'\t'+'0'\
                 +'\n'
        return txt_string
    
txt_iva()


class txt_iva_line(osv.osv):
    _inherit = "txt.iva.line"
    
    _columns = {
        'tax_wh_iva_id':fields.many2one('account.wh.iva.line.tax','Tax Wh Iva Line'),
    }
 
txt_iva_line()


class fiscal_reports_purchase(osv.osv):
    '''
    Modifying the object fiscal.reports.purchase
    '''
    _inherit = "fiscal.reports.purchase"
    
    def init(self, cr):
        '''
        Create or replace view fiscal_reports_purchase
        '''
        drop_view_if_exists(cr, 'fiscal_reports_purchase')
        cr.execute("""
            create or replace view fiscal_reports_purchase as (
                SELECT
                     ai."date_document" AS ai_date_document,
                     ai."date_invoice" AS ai_date_invoice,
                     rp."vat" AS rp_vat,
                     rp."id" AS rp_id,
                     ai."nro_ctrl" AS ai_nro_ctrl,
                     ai."reference" AS ai_reference,
                     ar."date_ret" AS ar_date_document,
                case when ai.type='in_refund'
                then
                    (sum(ait.base)+sum(ait.amount))*(-1)
                else
                    sum(ait.base)+sum(ait.amount)
                end as ai_amount_total,
                case when ai.type='in_refund'
                then
                    ai."amount_untaxed"*(-1)
                else
                    ai."amount_untaxed" 
                end as ai_amount_untaxed,
                case when ai.type='in_refund'
                then
                    ai."amount_tax"*(-1)
                else
                    ai."amount_tax" 
                end as ai_amount_tax,
                     ai."type" AS ai_type,
                     rp."wh_iva_rate" AS rp_retention,
                     ai."id" AS id,
                     ai."id" AS ai_id,
                     ar_line."id" AS ar_line_id,
                     ar_line."retention_id" AS ar_id
                FROM
                     "res_partner" rp INNER JOIN "account_invoice" ai ON rp."id" = ai."partner_id"
                     INNER JOIN "account_invoice_tax" ait ON ait."invoice_id" = ai."id"
                     LEFT JOIN "account_wh_iva_line" ar_line ON ar_line."invoice_id" = ai."id"
                     LEFT JOIN "account_wh_iva" ar ON ar_line."retention_id" = ar."id"                                                     
                WHERE
                     (ai.type = 'in_refund'
                  OR ai.type = 'in_invoice')
                 AND (ai.state = 'open'
                  OR ai.state = 'paid'
                  OR ai.state = 'done')
                GROUP BY ai."date_document",
                    ai."date_invoice",
                    rp."vat",
                    rp."id",
                    ai."nro_ctrl",
                    ai."reference",
                    ar."date_ret",
                    ai."amount_total",
                    ai."amount_untaxed", 
                    ai."amount_tax",
                    ai."type",
                    rp."wh_iva_rate",
                    ai."id",
                    ar_line."id",
                    ar_line."retention_id",
                    ar."number"
                ORDER BY
                     ai_date_invoice ASC,
                     ar."number" ASC
                 )
        """)
fiscal_reports_purchase() 
