# -*- encoding: utf-8 -*-
###########################################################################
#    Module Writen to OpenERP, Open Source Management Solution
#    Copyright (C) OpenERP Venezuela (<http://openerp.com.ve>).
#    All Rights Reserved
###############Credits######################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
##############################################################################

import xml
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
from tools import config


class account_balance_analityc(report_sxw.rml_parse):


    def __init__(self, cr, uid, name, context):
        super(account_balance_analityc, self).__init__(cr, uid, name, context)
        self.sum_debit = 0.00
        self.sum_credit = 0.00
        self.sum_balance = 0.00
        self.sum_debit_fy = 0.00
        self.sum_credit_fy = 0.00
        self.sum_balance_fy = 0.00
        self.date_lst = []
        self.date_lst_string = ''
        self.localcontext.update({
            'time': time,
            'lines': self.lines,
            'get_fiscalyear_text': self.get_fiscalyear_text,
            'get_periods_and_date_text': self.get_periods_and_date_text,
            'get_inf_text': self.get_informe_text,            
        })
        self.context = context


    def get_fiscalyear_text(self, form):
        """
        Returns the fiscal year text used on the report.
        """
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        fiscalyear = None
        if form.get('fiscalyear'):
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, form['fiscalyear'])
            return fiscalyear.name or fiscalyear.code
        else:
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, fiscalyear_obj.find(self.cr, self.uid))
            return "%s*" % (fiscalyear.name or fiscalyear.code)

    def get_informe_text(self, form):
        """
        Returns the header text used on the report.
        """
        inf_type = {
            'banal' : 'Balance Comprobacion por Cuentas Analiticas',
        }
        return inf_type[form['inf_type']]

    def get_periods_and_date_text(self, form):
        """
        Returns the text with the periods/dates used on the report.
        """
        period_obj = self.pool.get('account.period')
        periods_str = None
        fiscalyear_id = form['fiscalyear'] or fiscalyear_obj.find(self.cr, self.uid)
        period_ids = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear_id),('special','=',False)])
        if form['filter'] in ['byperiod', 'all']:
            period_ids = form['periods']
            periods_str = ', '.join([period.name or period.code for period in period_obj.browse(self.cr, self.uid, period_ids)])

        dates_str = None
        if form['filter'] in ['bydate', 'all']:
            dates_str = self.formatLang(form['date_from'], date=True) + ' - ' + self.formatLang(form['date_to'], date=True) + ' '
        return {'periods':periods_str, 'date':dates_str}
    
    
    def _get_lineas_analiticas(self, brw_accounts, context=None):
        """
        Funcion para extraer el saldo por cuenta analitica de cada
        cuenta contable.
        """
        if context is None:
            return {}

        account_ids = [x.id for x in brw_accounts if x.type != 'view']
        
        res = {}
        if context['filter'] in ('bydate','none'):
            #Para filtrar por 'fechas' o 'sin filtro', este porque se envia el año fiscal por fecha.
            where = """and aml.date between '%s' and '%s'"""%(context['date_from'],context['date_to'])
        elif context['filter'] == 'byperiod':
            #Para filtrar por periodos
            periodos = str(context['periods']).replace("[","(").replace("]",")")
            where = """and aml.period_id in %s"""%(periodos)
        else:
            #Para filtrar por periodos y fechas
            periodos = str(context['periods']).replace("[","(").replace("]",")")
            where = """and aml.period_id in %s and aml.date between '%s' and '%s'"""%(periodos,context['date_from'],context['date_to'])
        if context.get('state'):
            estado = " and am.state = '%s'"%context['state']
            where = where + estado
        
        where = where + " and aa.id in %s"%(str(account_ids).replace("[","(").replace("]",")"))
        
        sql_detalle = """select aa.id as account_id,
                aaa.code as code,
                aaa.name as name,
                sum(aml.debit) as debit,
                sum(aml.credit) as credit
            from account_move_line aml
                inner join account_account aa on aml.account_id = aa.id
                inner join account_move am on aml.move_id = am.id
                left outer join account_analytic_account aaa on aml.analytic_account_id = aaa.id
                where aml.state <> 'draft'""" + where + \
            """ group by aa.id, aaa.code, aaa.name"""
            
        self.cr.execute(sql_detalle)
        resultat = self.cr.dictfetchall()
        for analityc in resultat:
            if res.get(analityc['account_id'], False):
                res[analityc['account_id']][analityc['code'] or ''] = {
                    'name': analityc['name'] or 'Sin Cta. Analitica',
                    'debit': analityc['debit'] or 0.0,
                    'credit': analityc['credit'] or 0.0
                }
            else:
                detail = {}
                detail[analityc['code'] or ''] = {
                    'name': analityc['name'] or 'Sin Cta. Analitica',
                    'debit': analityc['debit'] or 0.00,
                    'credit': analityc['credit'] or 0.00
                }
                res[analityc['account_id']] = detail
        return res
    
    
    def lines(self, form, ids={}, done=None, level=0):
        """
        Returns all the data needed for the report lines
        (account info plus debit/credit/balance in the selected period
        and the full year)
        """
        tot_bin = 0.0
        tot_deb = 0.0
        tot_crd = 0.0
        tot_eje = 0.0
        if not ids:
            ids = self.ids
        if not ids:
            return []
        if not done:
            done = {}
    
        if form.has_key('account_list') and form['account_list']:
            account_ids = form['account_list']
            del form['account_list']
        res = {}
        result_acc = []
        accounts_levels = {}
        account_obj = self.pool.get('account.account')
        period_obj = self.pool.get('account.period')
        fiscalyear_obj = self.pool.get('account.fiscalyear')

        # Get the fiscal year
        fiscalyear = None
        if form.get('fiscalyear'):
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, form['fiscalyear'])
        else:
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, fiscalyear_obj.find(self.cr, self.uid))

        #
        # Get the accounts
        #
        def _get_children_and_consol(cr, uid, ids, level, context={}):
            aa_obj = self.pool.get('account.account')
            ids2=[]
            temp=[]
            read_data= aa_obj.read(cr, uid, ids,['id','child_id','level','type'], context)
            for data in read_data:
                #~ cr.execute("select left(create_date::varchar,10) from account_account where id = %s"%(data['id']))
                #~ if cr.fetchall()[0][0] < fiscalyear.date_stop:
                if data['child_id'] and data['level'] < level and data['type']!='consolidation':
                    #ids2.append([data['id'],'Label', 'Total'])
                    ids2.append([data['id'],True, False])
                    temp=[]
                    for x in data['child_id']:
                        temp.append(x)
                    ids2 += _get_children_and_consol(cr, uid, temp, level, context)
                    ids2.append([data['id'],False,True])
                else:
                    ids2.append([data['id'],True,True])
            return ids2

        child_ids = _get_children_and_consol(self.cr, self.uid, account_ids, form['display_account_level'] and form['display_account_level'] or 100,self.context)
        if child_ids:
            account_ids = child_ids
        
        account_obj = self.pool.get('account.account')
        period_obj = self.pool.get('account.period')
        fiscalyear_obj = self.pool.get('account.fiscalyear')

        #############################################################################
        # Calculate the period Debit/Credit                                         #
        # (from the selected period or all the non special periods in the fy)       #
        #############################################################################
        ctx = self.context.copy()
        ctx['filter'] = form.get('filter','all')
        ctx['fiscalyear'] = fiscalyear.id
        
        if form['filter'] in ['byperiod', 'all']:
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('id','in',form['periods'] or ctx['periods'])])
        
        if form['filter'] in ['bydate','all','none']:
            ctx['date_from'] = form['date_from']
            ctx['date_to'] = form['date_to']
        
        if form.get('state'):
            ctx['state'] = form['state']
            
        accounts=[]
        
        val = account_obj.browse(self.cr, self.uid, [aa_id[0] for aa_id in account_ids], ctx)
        analityc_lines = self._get_lineas_analiticas(val, ctx)
        c = 0
        for aa_id in account_ids:
            new_acc = {
            'id'        :val[c].id, 
            'type'      :val[c].type,
            'code'      :val[c].code,
            'name'      :val[c].name,
            'debit'     :val[c].debit,
            'credit'    :val[c].credit,
            'parent_id' :val[c].parent_id and val[c].parent_id.id,
            'level'     :val[c].level,
            'label'     :aa_id[1],
            'total'     :aa_id[2],
            #~ 'lines'     :analityc_lines.get(val[c].id, {}),
            }
            c += 1
            accounts.append(new_acc)
        
        def missing_period():
            ctx['fiscalyear'] = fiscalyear_obj.search(self.cr, self.uid, [('date_stop','<',fiscalyear.date_start)],order='date_stop') and \
                                fiscalyear_obj.search(self.cr, self.uid, [('date_stop','<',fiscalyear.date_start)],order='date_stop')[-1] or []
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',ctx['fiscalyear']),('date_stop','<',fiscalyear.date_start)])
        
        #############################################################################
        # Calculate the period initial Balance                                      #
        # (fy balance minus the balance from the start of the selected period       #
        #  to the end of the year)                                                  #
        #############################################################################
        
        ctx = self.context.copy()
        acum = False
        ctx['filter'] = form.get('filter','all')
        ctx['fiscalyear'] = fiscalyear.id

        if form['filter'] in ['byperiod', 'all']:
            ctx['periods'] = form['periods']
            date_start = min([period.date_start for period in period_obj.browse(self.cr, self.uid, ctx['periods'])])
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_stop','<=',date_start)])
            ctx['periods'] = list(set(ctx['periods'])-set(form['periods']))

        elif form['filter'] in ['bydate']:
            ctx['date_from'] = fiscalyear.date_start
            ctx['date_to'] = form['date_from']
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_stop','<=',ctx['date_to'])])
            
        elif form['filter'] == 'none':
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('special','=',True)])
            date_start = min([period.date_start for period in period_obj.browse(self.cr, self.uid, ctx['periods'])])
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_start','<=',date_start),('special','=',True)])
        
        del ctx['fiscalyear']
        fiscal_year_ant = fiscalyear_obj.search(self.cr, self.uid, [('state', '=', 'draft'),('date_stop', '<', fiscalyear.date_start)])
        period_ant = period_obj.search(self.cr, self.uid, [('fiscalyear_id', 'in', fiscal_year_ant)])
        ctx['periods'] = ctx['periods'] + period_ant

        period_balanceinit = {}
        listado_cuentas = list(set([x[0] for x in account_ids]))
        analityc_lines_init = self._get_lineas_analiticas(val, ctx)
        for acc in account_obj.browse(self.cr, self.uid, listado_cuentas, ctx):
            period_balanceinit[acc['id']] = acc.balance
            
        
        #~ if ((form['filter'] in ['byperiod', 'all']) and acum):
            #~ ctx['fiscalyear'] = fiscalyear_before
            #~ ctx['periods'] = period_before
            #~ for acc in account_obj.browse(self.cr, self.uid, listado_cuentas, ctx):
                #~ period_balanceinit[acc['id']] += acc.balance
            #~ 
            #~ analityc_lines_faltante = self._get_lineas_analiticas(val, ctx)
            #~ for cuenta in analityc_lines_faltante:
                #~ if analityc_lines_init.get(cuenta, False):
                    #~ for analitica in analityc_lines_faltante.get(cuenta, {}):
                        #~ if analityc_lines_init[cuenta].get(analitica, False):
                            #~ analityc_lines_init[cuenta][analitica]['debit'] += analityc_lines_faltante[cuenta][analitica]['debit']
                            #~ analityc_lines_init[cuenta][analitica]['credit'] += analityc_lines_faltante[cuenta][analitica]['credit']
                        #~ else:
                            #~ aux_analitico = {
                                #~ 'name': analityc_lines_faltante[cuenta][analitica]['name'],
                                #~ 'debit': analityc_lines_faltante[cuenta][analitica]['debit'],
                                #~ 'credit': analityc_lines_faltante[cuenta][analitica]['credit'],
                            #~ }
                            #~ analityc_lines_init[cuenta][analitica] = aux_analitico
                #~ else:
                    #~ analityc_lines_init[cuenta] = analityc_lines_faltante[cuenta]
        
        """
        Armar las lineas analiticas con saldo inicial y balance
        """
        analityc_account_obj = self.pool.get('account.analytic.account')
        analityc_final = {}
        cuentas_detalle = [x.id for x in val if x.type != 'view']
        cuentas_analiticas = analityc_account_obj.search(self.cr, self.uid, [('type','!=','view'), ('state','=','open'), ('company_id','=',form['company_id'])])
        cuentas_analiticas = [a.code for a in analityc_account_obj.browse(self.cr, self.uid, cuentas_analiticas)]
        cuentas_analiticas.append('')
        for cuenta in cuentas_detalle:
            if analityc_lines.get(cuenta,False) or analityc_lines_init.get(cuenta,False):
                list_analitica = []
                for analitica in cuentas_analiticas:
                    if analityc_lines.get(cuenta,{}).get(analitica,False) or analityc_lines_init.get(cuenta,{}).get(analitica,False):
                        line = {
                            'code': analitica,
                            'name': analityc_lines.get(cuenta,{}).get(analitica, {}).get('name', analityc_lines_init.get(cuenta,{}).get(analitica, {}).get('name','Sin Cta. Analitica')),
                            'balanceinit': analityc_lines_init.get(cuenta,{}).get(analitica,{}).get('debit', 0.00) - analityc_lines_init.get(cuenta,{}).get(analitica,{}).get('credit', 0.00),
                            'debit': analityc_lines.get(cuenta,{}).get(analitica,{}).get('debit', 0.00),
                            'credit': analityc_lines.get(cuenta,{}).get(analitica,{}).get('credit', 0.00),
                        }
                        line.update({'balance': line['balanceinit']+line['debit']-line['credit']})
                        list_analitica.append(line)
                analityc_final[cuenta] = list_analitica
        ########################################################################
        
        #
        # Generate the report lines (checking each account)
        #
        tot = {}        
        for account in accounts:
            account_id = account['id']

            if account_id in done:
                pass

            done[account_id] = 1

            accounts_levels[account_id] = account['level']

            #
            # Check if we need to include this level
            #
            if not form['display_account_level'] or account['level'] <= form['display_account_level']:
                #
                # Copy the account values
                #
                res = {
                        'id' : account_id,
                        'type' : account['type'],
                        'code': account['code'],
                        'name': (account['total'] and not account['label']) and 'TOTAL %s'%(account['name'].upper()) or account['name'],
                        'level': account['level'],
                        'balanceinit': period_balanceinit[account_id],
                        'debit': account['debit'],
                        'credit': account['credit'],
                        'balance': period_balanceinit[account_id]+account['debit']-account['credit'],
                        'parent_id': account['parent_id'],
                        'bal_type': '',
                        'label': account['label'],
                        'total': account['total'],
                        'lines': analityc_final.get(account_id, []),
                    }
                #
                # Round the values to zero if needed (-0.000001 ~= 0)
                #
                if abs(res['balance']) < 0.5 * 10**-4:
                    res['balance'] = 0.0

                #
                # Check whether we must include this line in the report or not
                #
                if form['display_account'] == 'con_movimiento' and account['parent_id']:
                    # Include accounts with movements
                    if abs(res['debit']) >= 0.5 * 10**-int(2) or abs(res['credit']) >= 0.5 * 10**-int(2):
                        result_acc.append(res)
                elif form['display_account'] == 'con_balance' and account['parent_id']:
                    # Include accounts with balance
                    if abs(res['balance']) >= 0.5 * 10**-4:
                        result_acc.append(res)
                else:
                    # Include all accounts
                    result_acc.append(res)
                if form['tot_check'] and res['type'] == 'view' and res['level'] == 1 and (res['id'] not in tot):
                    tot[res['id']] = True
                    tot_bin += res['balanceinit']
                    tot_deb += res['debit']
                    tot_crd += res['credit']
                    tot_eje += res['balance']
        if form['tot_check']:
            str_label = form['lab_str']
            res2 = {
                    'type' : 'view',
                    'name': 'TOTAL %s'%(str_label),
                    'balanceinit': tot_bin,
                    'debit': tot_deb,
                    'credit': tot_crd,
                    'balance': tot_eje,
                    'label': False,
                    'total': True,
            }
            result_acc.append(res2)
            
        return result_acc

report_sxw.report_sxw('report.wizard.reporte.balance.detail.analityc', 
                      'wizard.reporte.comprobacion', 
                      'l10n_ve_account_financial_report/report/account_balance_detail_analityc.rml',
                       parser=account_balance_analityc, 
                       header=False)
