# -*- encoding: utf-8 -*-
###########################################################################
#    Module Writen to OpenERP, Open Source Management Solution
#    Copyright (C) OpenERP Venezuela (<http://openerp.com.ve>).
#    All Rights Reserved
###############Credits######################################################
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
##############################################################################

import xml
import xlwt
import copy
from operator import itemgetter
import time
import datetime
from report import report_sxw
from tools import config
import csv


class account_mayor_analitico(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(account_mayor_analitico, self).__init__(cr, uid, name, context)
        self.sum_debit = 0.00
        self.sum_credit = 0.00
        self.sum_balance = 0.00
        self.sum_debit_fy = 0.00
        self.sum_credit_fy = 0.00
        self.sum_balance_fy = 0.00
        self.date_lst = []
        self.date_lst_string = ''
        self.localcontext.update({
            'time': time,
            'lines': self.lines,
            'get_fiscalyear_text': self.get_fiscalyear_text,
            'get_periods_and_date_text': self.get_periods_and_date_text,
            'get_inf_text': self.get_informe_text,
            'set_fecha': self.set_fecha,
            'get_partner': self.get_partner,
        })
        self.context = context
    
    
    def get_partner(self, form):
        """
        Returns the fiscal year text used on the report.
        """
        partner_obj = self.pool.get('res.partner')
        if form.get('partner_id'):
            partner_brw = partner_obj.browse(self.cr, self.uid, form['partner_id'])
            if partner_brw.ref:
                ref = "[%s] "%(partner_brw.ref)
            else:
                ref = ''
            name = ref + partner_brw.name
            return name
        else:
            return False
    
    
    def set_fecha(self, fecha):
        f = fecha.split('-')
        date = datetime.date(int(f[0]),int(f[1]),int(f[2]))
        return str(date.strftime("%d/%m/%Y"))
    
    def _get_mayor_detalle(self, account, form):
        res = []

        if account['type'] != 'view':
            if form['filter'] in ('bydate','none'):
                #Para filtrar por 'fechas' o 'sin filtro', este porque se envia el año fiscal por fecha.
                where = """where am.date between '%s' and '%s' and aa.id = %s and aml.state <> 'draft'"""%(form['date_from'],form['date_to'],account['id'])
            elif form['filter'] == 'byperiod':
                #Para filtrar por periodos
                periodos = str(form['periods']).replace("[","(").replace("]",")")
                where = """where am.period_id in %s and aa.id = %s and aml.state <> 'draft'"""%(periodos,account['id'])
            else:
                #Para filtrar por periodos y fechas
                periodos = str(form['periods']).replace("[","(").replace("]",")")
                where = """where am.period_id in %s and am.date between '%s' and '%s' and aa.id = %s and aml.state <> 'draft'"""%(periodos,form['date_from'],form['date_to'],account['id'])
            if form.get('state'):
                estado = " and am.state = '%s'"%form['state']
                where = where + estado
            
            if form.get('partner_id'):
                partner = """ and aml.partner_id = %s"""%(form['partner_id'])
                where = where + partner
                
            sql_detalle = """select aml.id as id, aj.name as diario, aa.name as descripcion,
                (select name from res_partner where aml.partner_id = id) as partner,
                aa.code as cuenta, aml.name || ' -- Ref: ' || aml.ref as ref,
                case when aml.debit is null then 0.00 else aml.debit end as debit, 
                case when aml.credit is null then 0.00 else aml.credit end as credit,
                (select code from account_analytic_account where  aml.analytic_account_id = id) as analitica,
                am.date as fecha, ap.name as periodo
                from account_move_line aml
                inner join account_journal aj on aj.id = aml.journal_id
                inner join account_account aa on aa.id = aml.account_id
                inner join account_period ap on ap.id = aml.period_id
                inner join account_move am on am.id = aml.move_id """ + where +\
                """ order by fecha"""

            self.cr.execute(sql_detalle)
            resultat = self.cr.dictfetchall()
            balance = account['balanceinit']

            for det in resultat:
                balance += det['debit'] - det['credit']
                res.append({
                    'id': det['id'],
                    'date': det['fecha'],
                    'journal':det['diario'],
                    'ref': det['ref'],
                    'debit': det['debit'],
                    'credit': det['credit'],
                    'analytic': det['analitica'],
                    'period': det['periodo'],
                    'balance': balance,
                })
        return res

    def get_fiscalyear_text(self, form):
        """
        Returns the fiscal year text used on the report.
        """
        fiscalyear_obj = self.pool.get('account.fiscalyear')
        fiscalyear = None
        if form.get('fiscalyear'):
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, form['fiscalyear'])
            return fiscalyear.name or fiscalyear.code
        else:
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, fiscalyear_obj.find(self.cr, self.uid))
            return "%s*" % (fiscalyear.name or fiscalyear.code)

    def get_informe_text(self, form):
        """
        Returns the header text used on the report.
        """
        inf_type = {
            'bgen' : '               Balance General',
            'bcom' : '       Balance de Comprobacion',            
            'edogp': 'Estado de Ganancias y Perdidas',
            'bml':   '               Mayor Analitico'
        }
        return inf_type[form['inf_type']]

    def get_periods_and_date_text(self, form):
        """
        Returns the text with the periods/dates used on the report.
        """
        period_obj = self.pool.get('account.period')
        periods_str = None
        fiscalyear_id = form['fiscalyear'] or fiscalyear_obj.find(self.cr, self.uid)
        period_ids = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear_id),('special','=',False)])
        if form['filter'] in ['byperiod', 'all']:
            period_ids = form['periods']
            periods_str = ', '.join([period.name or period.code for period in period_obj.browse(self.cr, self.uid, period_ids)])

        dates_str = None
        if form['filter'] in ['bydate', 'all']:
            dates_str = self.formatLang(form['date_from'], date=True) + ' - ' + self.formatLang(form['date_to'], date=True) + ' '
        return {'periods':periods_str, 'date':dates_str}
    
    
    def calcular_saldos_inicio_fin(self, cr, uid, account_ids, form, context):
        move_line_obj = self.pool.get('account.move.line')
        
        accounts = str(account_ids).replace('[','(').replace(']',')')
        where_q = move_line_obj._query_get(cr, uid, obj='aml', context=context)
        
        if form.get('partner_id'):
            partner = """ AND aml.partner_id = %s """%(form['partner_id'])
            where_q = where_q + partner
        
        where_q = where_q + """ AND aml.account_id in """ + accounts
        
        query = """
                SELECT aa.id, aa.type as type, aa.code as code, aa.name as name,
                    aa.parent_id as parent_id, aa.level as level, 'TRUE'::boolean as label, 'TRUE'::boolean as total,
                    case when tbl.debit is null then 0 else tbl.debit end,
                    case when tbl.credit is null then 0 else tbl.credit end,
                    case when tbl.debit is null and tbl.credit is null then 0 else tbl.debit-tbl.credit end as balance
                FROM account_account aa
                LEFT OUTER JOIN (
                    SELECT aml.account_id, sum(COALESCE(aml.debit, 0.00)) as debit, sum(COALESCE(aml.credit, 0.00)) as credit
                    FROM account_move_line aml
                    WHERE %s
                    GROUP BY aml.account_id) tbl on aa.id = tbl.account_id
                WHERE aa.id IN %s
                ORDER BY aa.code
                """%(where_q, accounts)

        self.cr.execute(query)
        resultat = self.cr.dictfetchall()
        
        return resultat
        
    
    def lines(self, form, ids={}, done=None, level=0):
        """
        Returns all the data needed for the report lines
        (account info plus debit/credit/balance in the selected period
        and the full year)
        """
        tot_bin = 0.0
        tot_deb = 0.0
        tot_crd = 0.0
        tot_eje = 0.0
        if not ids:
            ids = self.ids
        if not ids:
            return []
        if not done:
            done = {}
    
        if form.has_key('account_list') and form['account_list']:
            account_ids = form['account_list']
            del form['account_list']
        res = {}
        result_acc = []
        accounts_levels = {}
        account_obj = self.pool.get('account.account')
        period_obj = self.pool.get('account.period')
        fiscalyear_obj = self.pool.get('account.fiscalyear')

        # Get the fiscal year
        fiscalyear = None
        if form.get('fiscalyear'):
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, form['fiscalyear'])
        else:
            fiscalyear = fiscalyear_obj.browse(self.cr, self.uid, fiscalyear_obj.find(self.cr, self.uid))

        #
        # Get the accounts
        #
        def _get_children_and_consol(cr, uid, ids, level, context={}):
            aa_obj = self.pool.get('account.account')
            ids2=[]
            temp=[]
            read_data= aa_obj.read(cr, uid, ids,['id','child_id','level','type'], context)
            for data in read_data:
                #~ cr.execute("select left(create_date::varchar,10) from account_account where id = %s"%(data['id']))
                #~ if cr.fetchall()[0][0] < fiscalyear.date_stop:
                if data['child_id'] and data['level'] < level and data['type']!='consolidation':
                    #ids2.append([data['id'],'Label', 'Total'])
                    ids2.append([data['id'],True, False])
                    temp=[]
                    for x in data['child_id']:
                        temp.append(x)
                    ids2 += _get_children_and_consol(cr, uid, temp, level, context)
                    ids2.append([data['id'],False,True])
                else:
                    ids2.append([data['id'],True,True])
            return ids2

        child_ids = _get_children_and_consol(self.cr, self.uid, account_ids, form['display_account_level'] and form['display_account_level'] or 100,self.context)
        if child_ids:
            account_ids = child_ids

        #############################################################################
        # Calculate the period Debit/Credit                                         #
        # (from the selected period or all the non special periods in the fy)       #
        #############################################################################
        ctx = self.context.copy()
        ctx['filter'] = form.get('filter','all')
        ctx['fiscalyear'] = fiscalyear.id
        
        if form['filter'] in ['byperiod', 'all']:
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('id','in',form['periods'] or ctx['periods'])])
        
        if form['filter'] in ['bydate','all','none']:
            ctx['date_from'] = form['date_from']
            ctx['date_to'] = form['date_to']
        
        if form.get('state'):
            ctx['state'] = form['state']
        
        accounts=[]
        
        lista_act = [aa_id[0] for aa_id in account_ids]
        val = self.calcular_saldos_inicio_fin(self.cr, self.uid, lista_act, form, ctx)
        
        c = 0
        for aa_id in val:
            new_acc = {
            'id'        :aa_id['id'], 
            'type'      :aa_id['type'],
            'code'      :aa_id['code'],
            'name'      :aa_id['name'],
            'debit'     :aa_id['debit'],
            'credit'    :aa_id['credit'],
            'parent_id' :aa_id['parent_id'],
            'level'     :aa_id['level'],
            'label'     :True,
            'total'     :True,
            }
            c += 1
            accounts.append(new_acc)
        
        def missing_period():
            ctx['fiscalyear'] = fiscalyear_obj.search(self.cr, self.uid, [('date_stop','<',fiscalyear.date_start)],order='date_stop') and \
                                fiscalyear_obj.search(self.cr, self.uid, [('date_stop','<',fiscalyear.date_start)],order='date_stop')[-1] or []
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',ctx['fiscalyear']),('date_stop','<',fiscalyear.date_start)])
        
        #############################################################################
        # Calculate the period initial Balance                                      #
        # (fy balance minus the balance from the start of the selected period       #
        #  to the end of the year)                                                  #
        #############################################################################
        
        ctx = self.context.copy()
        acum = False
        ctx['filter'] = form.get('filter','all')
        ctx['fiscalyear'] = fiscalyear.id

        if form['filter'] in ['byperiod', 'all']:
            ctx['periods'] = form['periods']
            date_start = min([period.date_start for period in period_obj.browse(self.cr, self.uid, ctx['periods'])])
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_stop','<=',date_start)])
            ctx['periods'] = list(set(ctx['periods'])-set(form['periods']))

        elif form['filter'] in ['bydate']:
            ctx['date_from'] = fiscalyear.date_start
            ctx['date_to'] = form['date_from']
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_stop','<=',ctx['date_to'])])
            
        elif form['filter'] == 'none':
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('special','=',True)])
            date_start = min([period.date_start for period in period_obj.browse(self.cr, self.uid, ctx['periods'])])
            ctx['periods'] = period_obj.search(self.cr, self.uid, [('fiscalyear_id','=',fiscalyear.id),('date_start','<=',date_start),('special','=',True)])

        del ctx['fiscalyear']
        fiscal_year_ant = fiscalyear_obj.search(self.cr, self.uid, [('state', '=', 'draft'),('date_stop', '<', fiscalyear.date_start)])
        period_ant = period_obj.search(self.cr, self.uid, [('fiscalyear_id', 'in', fiscal_year_ant)])
        ctx['periods'] = ctx['periods'] + period_ant
        
        period_balanceinit = {}
        for acc in self.calcular_saldos_inicio_fin(self.cr, self.uid, lista_act, form, ctx):
            period_balanceinit[acc['id']] = acc['balance']
        
        #
        # Generate the report lines (checking each account)
        #
        tot = {}        
        for account in accounts:
            account_id = account['id']

            if account_id in done:
                pass

            done[account_id] = 1

            accounts_levels[account_id] = account['level']

            #
            # Check if we need to include this level
            #
            if not form['display_account_level'] or account['level'] <= form['display_account_level']:
                #
                # Copy the account values
                #
                res = {
                        'id' : account_id,
                        'type' : account['type'],
                        'code': account['code'],
                        'name': (account['total'] and not account['label']) and 'TOTAL %s'%(account['name'].upper()) or account['name'],
                        'level': account['level'],
                        'balanceinit': period_balanceinit[account_id],
                        'debit': account['debit'],
                        'credit': account['credit'],
                        'balance': period_balanceinit[account_id]+account['debit']-account['credit'],
                        'parent_id': account['parent_id'],
                        'bal_type': '',
                        'label': account['label'],
                        'total': account['total'],
                    }
                #
                # Round the values to zero if needed (-0.000001 ~= 0)
                #
                if abs(res['balance']) < 0.5 * 10**-4:
                    res['balance'] = 0.0

                #
                # Check whether we must include this line in the report or not
                #
                if form['display_account'] == 'con_movimiento' and account['parent_id']:
                    # Include accounts with movements
                    if abs(res['debit']) >= 0.5 * 10**-int(2) or abs(res['credit']) >= 0.5 * 10**-int(2):
                        if res['type'] != 'view' and res['code']!='0':
                            res['mayor'] = self._get_mayor_detalle(res,form)
                            result_acc.append(res)
                elif form['display_account'] == 'con_balance' and account['parent_id']:
                    # Include accounts with balance
                    if abs(res['balance']) >= 0.5 * 10**-4:
                        if res['type'] != 'view' and res['code']!='0':
                            res['mayor'] = self._get_mayor_detalle(res,form)
                            result_acc.append(res)
                else:
                    # Include all accounts
                    if res['type'] != 'view' and res['code']!='0':
                        res['mayor'] = self._get_mayor_detalle(res,form)
                        result_acc.append(res)
                if form['tot_check'] and res['type'] == 'view' and res['level'] == 1 and (res['id'] not in tot):
                    tot[res['id']] = True
                    tot_bin += res['balanceinit']
                    tot_deb += res['debit']
                    tot_crd += res['credit']
                    tot_eje += res['balance']
        if form['tot_check']:
            str_label = form['lab_str']
            res2 = {
                    'type' : 'view',
                    'name': 'TOTAL %s'%(str_label),
                    'balanceinit': tot_bin,
                    'debit': tot_deb,
                    'credit': tot_crd,
                    'balance': tot_eje,
                    'label': False,
                    'total': True,
            }
            result_acc.append(res2)
            
        return result_acc

#~ report_sxw.report_sxw('report.wizard.reporte.comprobacion.mayor.analitico', 'wizard.reporte.comprobacion', 'l10n_ve_account_financial_report/report/mayor_analitico.rml', parser=account_mayor_analitico, header=False)

report_sxw = report_sxw.report_sxw

class TxtParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """
    def __init__(self, name, table, rml=False, parser=False,
        header=True, store=False):
        self.parser_instance = False
        self.localcontext={}
        report_sxw.__init__(self, name, table, rml, parser,
            header, store)
   
   
    def crear_xls(self, mayor_anatilico, data, ids):
        import StringIO
        import base64
        try:
            import xlwt
        except:
            raise osv.except_osv('Warning !','Please download python xlwt module from\nhttp://pypi.python.org/packages/source/x/xlwt/xlwt-0.7.2.tar.gz\nand install it')
        
        """Parsing data as string """
        wbk = xlwt.Workbook()
        style1 = xlwt.easyxf('font: bold on, height 240;' 'align: horiz center;''borders: left thin, right thin, top thin')
        style2 = xlwt.easyxf('font: bold on, height 240;' 'align: horiz center;')
        numbers = xlwt.easyxf('font: name Arial, colour black, height 200;', num_format_str='#,##0.00')
        s1=0
        """adding a worksheet along with name"""
        sheet1 = wbk.add_sheet('Mayor Analitico')
        sheet1.col(0).width = 4700
        sheet1.col(1).width = 4700
        sheet1.col(2).width = 13800
        sheet1.col(3).width = 4800
        sheet1.col(4).width = 4800
        sheet1.col(5).width = 4800
        sheet1.col(6).width = 4800
        lines = mayor_anatilico.lines(data['form'], ids=ids)
        sheet1.write(s1,2,"MAYOR ANALITICO",style2)
        s1 += 1
        if lines:
            fiscalyear = mayor_anatilico.get_fiscalyear_text(data['form'])
            if fiscalyear:
                sheet1.write(s1,0,"Ejercicio Fiscal:",style1)
                sheet1.write(s1,1,fiscalyear)
                s1 += 1
            
            periods = mayor_anatilico.get_periods_and_date_text(data['form'])
            if periods['periods']:
                sheet1.write(s1,0,"Periodos:",style1)
                sheet1.write(s1,1,periods['periods'])
                s1 += 1
            
            
            date = mayor_anatilico.get_periods_and_date_text(data['form'])
            if date['date']:
                sheet1.write(s1,0,"Fecha Intervalo:",style1)
                sheet1.write(s1,1,date['date'])
                s1 += 1
            
            partner = mayor_anatilico.get_partner(data['form'])
            if partner:
                sheet1.write(s1,0,"Empresa:",style1)
                sheet1.write(s1,1,partner)
                s1 += 1
            s1 += 1
            for linea_cuenta in lines:
                """ writing field headings """
                sheet1.write(s1,1,"Cuenta",style1)
                sheet1.write(s1,2,"Descripcion",style1)
                sheet1.write(s1,3,"Saldo Inicial",style1)
                sheet1.write(s1,4,"Debe",style1)
                sheet1.write(s1,5,"Haber",style1)
                sheet1.write(s1,6,"Saldo Final",style1)
                s1 += 1
                sheet1.write(s1,1,linea_cuenta['code'])
                sheet1.write(s1,2,linea_cuenta['name'])
                sheet1.write(s1,3,linea_cuenta['balanceinit'],numbers)
                sheet1.write(s1,4,linea_cuenta['debit'],numbers)
                sheet1.write(s1,5,linea_cuenta['credit'],numbers)
                sheet1.write(s1,6,linea_cuenta['balance'],numbers)
                s1 += 1
                if linea_cuenta['mayor']:
                    sheet1.write(s1,0,"Fecha",style1)
                    sheet1.write(s1,1,"Periodo",style1)
                    sheet1.write(s1,2,"Referencia",style1)
                    sheet1.write(s1,3,"Cuenta Analitica",style1)
                    sheet1.write(s1,4,"Debe",style1)
                    sheet1.write(s1,5,"Haber",style1)
                    sheet1.write(s1,6,"Balance",style1)
                    s1 += 1
                    """writing data according to query and filteration in worksheet"""
                    for mayor_line in linea_cuenta['mayor']:
                        sheet1.write(s1,0,str(mayor_anatilico.set_fecha(mayor_line['date'])))
                        sheet1.write(s1,1,str(mayor_line['period']))
                        sheet1.write(s1,2,mayor_line['ref'])
                        sheet1.write(s1,3,mayor_line['analytic'])
                        sheet1.write(s1,4,mayor_line['debit'],numbers)
                        sheet1.write(s1,5,mayor_line['credit'],numbers)
                        sheet1.write(s1,6,mayor_line['balance'],numbers)
                        s1 += 1
                s1 += 1
            
        else:
            sheet1.write(s1,1,"Cuenta",style1)
            sheet1.write(s1,2,"Descripcion",style1)
            sheet1.write(s1,3,"Saldo Inicial",style1)
            sheet1.write(s1,4,"Debe",style1)
            sheet1.write(s1,5,"Haber",style1)
            sheet1.write(s1,6,"Saldo Final",style1)
            s1 += 1
            sheet1.write(s1,0,"Sin Informacion para mostrar")
            
        file_data=StringIO.StringIO()
        o=wbk.save(file_data)
        """string encode of data in wksheet"""
        return file_data.getvalue()
    
    
    def create_single_pdf(self, cursor, uid, ids, data, report_xml, context):
        """generate the PDF"""
      
        if context is None:
            context={}
        
        if  data['form']['en_xls']:
            a = account_mayor_analitico(cursor, uid, report_xml.report_name, context)
            xls_report = self.crear_xls(a, data, ids)
            return (xls_report, 'ods')
            
        else:
            return super(TxtParser,self).create_single_pdf(cursor, uid, ids, data, report_xml, context=context)


TxtParser('report.wizard.reporte.comprobacion.mayor.analitico', 'wizard.reporte.comprobacion', 'l10n_ve_account_financial_report/report/mayor_analitico.rml', parser=account_mayor_analitico, header=False)
