from osv import osv, fields

class wizard_estado_rendimiento_financiero(osv.osv_memory):
    """
    Este wizard provee el estado de rendimiento financiero a una fecha o periodo dado.
    """
    _name = "wizard.estado.rendimiento.financiero"
    _description = "Estado de Rendimiento Financiero"
    _inherit = "account.pl.report"
    
    _columns = {
        'display_account_level': fields.integer('Up to level', help='Display accounts up to this level (0 to show all)', required=True),
     }
    
    def _print_report(self, cr, uid, ids, data, context=None):
        if context is None:
            context = {}
        data = self.pre_print_report(cr, uid, ids, data, context=context)
        data['form'].update(self.read(cr, uid, ids, ['display_type'])[0])
        data['form'].update(self.read(cr, uid, ids, ['display_account_level'])[0])

        if data['form']['period_from'] > data['form']['period_to']:
            raise osv.except_osv('ERROR', "El periodo inicial debe ser de una fecha menor con relacion al periodo final")         

        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'estado.rendimiento.financiero',
            'datas': data,
        }

    def get_period(self, cr, uid, context=None, fiscalyear_id=False):
        res = {}   
        start_period = end_period = False
        if  fiscalyear_id:
            cr.execute('''
                    SELECT * FROM (SELECT p.id
                        FROM account_period p
                        LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                        WHERE f.id = %s
                        ORDER BY p.date_start ASC
                    LIMIT 1) AS period_start
                UNION
                    SELECT * FROM (SELECT p.id
                        FROM account_period p
                        LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                        WHERE f.id = %s
                        AND p.date_start < NOW()
                        ORDER BY p.date_stop DESC
                    LIMIT 1) AS period_stop''', (fiscalyear_id, fiscalyear_id))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 1:
                start_period = periods[0]
                end_period = periods[1]
        res['value'] = {'period_from': start_period, 'period_to': end_period}
        return res
        
    def _get_period_start(self, cr, uid, context=None):
        start_period = fiscalyear_id = False
        fiscalyear_id = self._get_fiscalyear(cr, uid, context)
        if  fiscalyear_id:
            cr.execute('''
                    SELECT * FROM (SELECT p.id
                        FROM account_period p
                        LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                        WHERE f.id = %s
                        ORDER BY p.date_start ASC
                    LIMIT 1) AS period_start''', (fiscalyear_id, ))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 0:
                start_period = periods[0]
        return start_period
        
    def _get_period_end(self, cr, uid, context=None):
        end_period = fiscalyear_id = False
        fiscalyear_id = self._get_fiscalyear(cr, uid, context)
        if  fiscalyear_id:
            cr.execute('''
                    SELECT * FROM (SELECT p.id
                        FROM account_period p
                        LEFT JOIN account_fiscalyear f ON (p.fiscalyear_id = f.id)
                        WHERE f.id = %s
                        AND p.date_start < NOW()
                        ORDER BY p.date_stop DESC
                    LIMIT 1) AS period_stop''', (fiscalyear_id, ))
            periods =  [i[0] for i in cr.fetchall()]
            if periods and len(periods) > 0:
                end_period = periods[0]
        return end_period        

    _defaults = {
        'display_account_level': lambda *a: 0,
        'period_from': _get_period_start,
        'period_to': _get_period_end,
    }

wizard_estado_rendimiento_financiero()
