#!/usr/bin/env python
# -*- coding: utf-8 -*--

from osv import fields, osv
import datetime

class horas_extras_parametros_turnos(osv.osv):
        
    _name = 'horas.extras.parametros.turnos'
    _description = 'parametros de horas extras'
    _columns = {
    'name': fields.char('Nombre', size=60),
    'concept_id': fields.many2one('hr.concept', 'Concepto', required=True),
    'feriado_id': fields.many2one('hr.concept', 'Concepto feriado'),
    'hr_hours_per_day_id': fields.many2one('hr.hours.per.day', 'Turno', required=True),
    'hour_start': fields.time('hora inicial', required=True),
    'hour_end': fields.time('hora final', required=True),
    'day_start': fields.selection([(1, 'Lunes'),(2, 'Martes'),
    (3, 'Miercoles'),(4, 'Jueves'),(5, 'Viernes'),(6, 'Sabado'),(7, 'Domingo')], 
    'Dia desde', required=True), 
    'day_end': fields.selection([(1, 'Lunes'),(2, 'Martes'),
    (3, 'Miercoles'),(4, 'Jueves'),(5, 'Viernes'),(6, 'Sabado'),(7, 'Domingo')], 
    'Dia hasta', required=True), 
    }

    def diff(self, hr, ha):
        criterio = 3600
        r1 =  datetime.datetime.strptime(hr, '%H:%M:%S')
        r2 =  datetime.datetime.strptime(ha, '%H:%M:%S')
        dif = r2 - r1 
        ab = abs(dif)
        time =  float(ab.seconds / float(criterio))
        time = round(time, 2)
        return time

    def search_parameters_overtime(self,cr, uid, hr_hours_per_day_id, dia, hour_start, hour_end, feriado=None):
        sql = """
        select id FROM horas_extras_parametros_turnos
        WHERE %s BETWEEN day_start and day_end 
        and (hour_start >= '%s' and hour_end <= '%s' OR  
        (hour_start <= '%s' and hour_end >= '%s') OR
        (hour_start <= '%s' and hour_end >= '%s')) 
        and hr_hours_per_day_id = %s
        """%(dia, hour_start, hour_end, hour_start, hour_start, hour_end, hour_end, hr_hours_per_day_id)
        cr.execute(sql)
        turnos_ids = cr.fetchall()
        turnos_ids = [t for i in turnos_ids for t in i]
        parametro_read = ['hour_start', 'hour_end', 'concept_id']
        if feriado:
		   parametro_read.append('feriado_id')
        turnos_read = self.read(cr, uid, turnos_ids, parametro_read)
        return turnos_read
    
    def conditionals(self, e1, e2, p1, p2, c, val):
        diff = self.diff
        param = diff(p1,p2)
        extra = diff(e1,e2)
        if extra > param:
            if e1 >= p1 and e1 <= p2 and e2 > p1 and e2 > p2: 
                val[c] += diff(e1,p2)
            elif e1 <= p1 and e1 <= p2 and e2 > p1 and e2 > p2:
                val[c] += diff(p1,p2)
            elif e1 <= p1 and e1 <= p2 and e2 > p1 and e2 < p2:
                val[c] += diff(p1,e2)
            else:
                val[c] += diff(p1,e2) 
        else:
            if e1 > e2:
                val[c] += diff(p1, e2)
            else:
                if e1 >= p1 and e2 <= p2:
                   val[c] += diff(e2, e1)
                elif e1 <= p1 and e2 <= p2:
                     val[c] += diff(p1, e2)
                else:
                    val[c] += diff(e1, p2)
        return val 
    
    def calculte_overtime(self, cr, uid, hr_hours_per_day_id, dia, hour_start, hour_end, feriado=None): 
        parameters  = self.search_parameters_overtime(cr, uid, hr_hours_per_day_id, dia, hour_start, hour_end, feriado=feriado)
        diff = self.diff
        val = {} 
        for i in parameters:
            if feriado:
                if not i['feriado_id']:
                   val[i['concept_id'][0]] = 0.00
                else:
                    val[i['feriado_id'][0]] = 0.00
            else:
                val[i['concept_id'][0]] = 0.00
        e1 = hour_start
        e2 = hour_end
        for p in parameters:
            p1 = p['hour_start']
            p2 = p['hour_end']
            c = p['feriado_id'][0] if feriado and p['feriado_id'] else p['concept_id'][0]
            #Turno normal, 1er grupo, 2do grupo, 3er grupo, 4to grupo
            val = self.conditionals(e1, e2, p1, p2, c, val)
        return val
    
    def validate_parameters_day_hour(self, cr, day_start,day_end, hour_start,hour_end, hr_hours_per_day_id):
        sql = """
        select id FROM horas_extras_parametros_turnos
        WHERE (day_start >= %s and day_end <= %s 
        or (day_start <= %s and day_end >= %s) 
        or (day_start <= %s and day_end >= %s))   
        and (hour_start >= '%s' and hour_end <= '%s' OR  
        (hour_start <= '%s' and hour_end >= '%s') OR
        (hour_start <= '%s' and hour_end >= '%s')) 
        and hr_hours_per_day_id = %s
        """%(day_start, day_end, day_start,day_start, day_end, day_end, hour_start, hour_end, hour_start, hour_start, hour_end, hour_end, hr_hours_per_day_id)
        cr.execute(sql)
        query  = cr.fetchall()
        query = [t for i in query for t in i]
        return query
    
    def validate_ranges_hours(self, hour_start,hour_end):
        if datetime.datetime.strptime(hour_end, '%H:%M:%S') <  datetime.datetime.strptime(hour_start, '%H:%M:%S'):
           raise osv.except_osv('ERROR', "La hora final es debe ser mayor a la inicial")
        return True  
         

    def create(self, cr, uid, values, context=None):
        day_start = values['day_start']
        day_end = values['day_end']
        hour_start = values['hour_start']
        hour_end = values['hour_end']
        hr_hours_per_day_id = values['hr_hours_per_day_id'] 
        self.validate_ranges_hours(hour_start,hour_end)
        validar  = self.validate_parameters_day_hour(cr, day_start, day_end, hour_start,hour_end, hr_hours_per_day_id)
        if validar:
           raise osv.except_osv('ERROR', "No se puede crear un nuevo parametro si los rangos de horas y fechas coinciden con las anteriores")
        else:              
            super(horas_extras_parametros_turnos, self).create(cr, uid, values, context=None) 
        return True
    
    #~ def write(self, cr, uid, ids, values, context=None):
        #~ parameters_brw = self.browse(cr, uid, ids[0])
        #~ day_start = values['day_start']
        #~ day_end = values['day_end']
        #~ hour_start = values['hour_start']
        #~ hour_end = values['hour_end']
        #~ hr_hours_per_day_id = parameters_brw.hr_hours_per_day_id.id 
        #~ self.validate_ranges_hours(hour_start,hour_end)
        #~ validar  = self.validate_parameters_day_hour(cr, day_start, day_end, hour_start,hour_end, hr_hours_per_day_id)
        #~ if validar:
           #~ raise osv.except_osv('ERROR', "No se puede actualizar los parametros si los rangos de horas y fechas coinciden con las anteriores")
        #~ else:              
            #~ super(horas_extras_parametros_turnos, self).create(cr, uid, values, context=None) 
        #~ return True
        
        


horas_extras_parametros_turnos()

class hr_hours_per_day(osv.osv):

    _inherit = 'hr.hours.per.day'
    _description = 'Horario'

    _columns = {
        'reglas_horas_extras_ids': fields.one2many('horas.extras.parametros.turnos', 'hr_hours_per_day_id', 'Reglas de sobretiempo del turno'),

    }
    
hr_hours_per_day()   
