#!/usr/bin/env python
# -*- coding: utf-8 -*-

from osv import fields, osv
import datetime
import time 
import logging
from tools.translate import _

class horas_extras_wizard_concept_variable(osv.osv_memory):
    _name = 'horas.extras.wizard.concept.variable'
    _description = 'formato de horas extras'
    _logger = logging.getLogger('horas.extras.planificacion')
    _columns = {
        'hr_payroll_id': fields.many2one('hr.payroll','Nomina'),
        'hr_sede_ids': fields.many2many('hr.sede','sisb_reposos_medicos_conceptos_variables_sede_rel','sisb_reposos_medicos_conceptos_variables_id','hr_sede_id'),
        'ejecutado':fields.boolean('Ejecutado'),
    }
    
    def _get_procesos_horas_extras(self, cr, uid, hr_payroll, horas_extra_lines, context=None):
        proceso_obj = self.pool.get('hr.process')
        proceso_ids = proceso_obj.search(cr, uid, [('type','=','nomina'),('state','in',['load', 'review'])], context=None)
        procesos = proceso_obj.browse(cr, uid, proceso_ids, context)
        procesos_horas_extra = {}
        for proceso in procesos:
            if len(procesos_horas_extra) == len(horas_extra_lines):
                break;
            if hr_payroll in proceso.hr_payroll_ids:
                for horas_extra in horas_extra_lines:
                    if (not proceso.hr_sede_ids or horas_extra.empleado_id.hr_sede_id in proceso.hr_sede_ids) and (not proceso.hr_employee_ids or horas_extra.empleado_id in proceso.hr_employee_ids):
                        procesos_horas_extra[horas_extra.horas_extras_planificacion_id.id] = proceso.id

        if not procesos_horas_extra:
            raise osv.except_osv(_('Error'), _('No existe un proceso en el cual cargar horas extras para los trabajadores de la nomina %s') % hr_payroll.name)
        return procesos_horas_extra 
		
    def _get_conceptos_variables_procesos(self, cr, uid, procesos_ids, context=None):
        """
        Devuelve un diccionario con clave el id del proceso y valor el id de los
        conceptos variables en los cuales se cargaran las lineas asociadas a los reposos.
        """
        conceptos_variables_obj =  self.pool.get('hr.variable.concepts')
        conceptos_variables_reposos = {}

        for proceso_id in procesos_ids:
            conceptos_variables_id = conceptos_variables_obj.search(cr, uid, 
                                 [('hr_process_id','=',proceso_id),
                                 ('tipo_de_proceso','=','sobretiempo'),
                                 ('state','in',['draft','load','review'])], context=None)
            
            if not conceptos_variables_id:
                values = {
                    'tipo_de_proceso': 'sobretiempo',
                    'hr_process_id': proceso_id,
                }
                self._logger.info("Creando conceptos variables de horas extras para el proceso %s",proceso_id )
                conceptos_variables_id = conceptos_variables_obj.create(cr, uid, values, context)
                conceptos_variables_reposos[proceso_id] = conceptos_variables_id
            else:
                conceptos_variables_reposos[proceso_id] = conceptos_variables_id[0]
			
        return conceptos_variables_reposos
   
    def get_shitf_week(self, cr, uid, ids, date_start, date_end, emp_id, context=None):
        sql = """
            select max(shit.id) from hr_shift_week as shit 
            join hr_period as period ON shit.period_id = period.id 
            join hr_employee as emp ON shit.employee_id = emp.id 
            join hr_shift as sh ON shit.shift_id = sh.id 
            where emp.id = '%s' 
            and period.date_start <= '%s'
            """%(emp_id, date_end)
        cr.execute(sql)
        shitf_week_obj = self.pool.get('hr.shift.week')
        shitf_week_id = cr.fetchall()[0][0]
        shitf_week_brw = None
        if shitf_week_id:
            shitf_week_brw = shitf_week_obj.browse(cr, uid, shitf_week_id)
        return shitf_week_brw
        
    def get_hours_shift(self, fecha, turno, dia=None):
        dia_id = {1:'monday', 2:'tuesday', 3:'wednesday', 4:'thursday', 5:'friday', 6:'saturday', 7:'sunday'}
        if not dia:
           dia = datetime.datetime.strptime(fecha, '%Y-%m-%d').isoweekday()
        func = getattr(turno, dia_id[dia]+'_hours_day_id')
        func = getattr(func, dia_id[dia]+'_hours_id')
        l = [func.hour_start, func.hour_end]
        return l

    def subtract_hours(self,  hr, ha, criterio):
        r1 =  datetime.datetime.strptime(hr, '%H:%M:%S')
        r2 =  datetime.datetime.strptime(ha, '%H:%M:%S')
        dif = r2 - r1 
        ab = abs(dif)
        time =  float(ab.seconds / float(criterio))
        time = round(time, 2)
        return time 
	
    def diff_days(self, start, end):
        d_start = self.convert_datetime(start, tipo='object')
        d_end = self.convert_datetime(end, tipo='object')
        day = abs(d_end - d_start)
        day = day.days
        return day
	
    def day_week(self, fecha):
        dia = datetime.datetime.strptime(fecha, '%Y-%m-%d').isoweekday()
        return dia
    
    def get_hours_per_day_id(self, fecha, turno, dia=None):
        dia = self.day_week(fecha)
        dia_id = {1:'monday', 2:'tuesday', 3:'wednesday', 4:'thursday', 5:'friday', 6:'saturday', 7:'sunday'}
        func = getattr(turno, dia_id[dia]+'_hours_day_id')
        return func.id
    
    def convert_datetime(self, value_date, tipo=None, format_inp=None):
        d = {}
        if not format_inp:
           format_inp = "%Y-%m-%d %H:%M:%S"
        value_date = datetime.datetime.strptime(value_date, format_inp)
        d = {'date':value_date.strftime("%Y-%m-%d"),
            'time':value_date.strftime("%H:%M:%S"), 
            'object': value_date}
        if  tipo:
            return d[tipo] 
        return d
        
    def calcular_sobretiempos_aux(self, cr, uid, hr_hours_per_day_id, dia, hora_entrada, hora_salida, feriado=[None,None]):
        sobretiempos = {}
        aux_end = '23:59:59'
        aux_start = '00:00:00'
        parametros_turnos_obj = self.pool.get('horas.extras.parametros.turnos')
        turno = parametros_turnos_obj.search_parameters_overtime(cr, uid, hr_hours_per_day_id, dia, hora_entrada, hora_salida) 
        if turno: 
            val1 = parametros_turnos_obj.calculte_overtime(cr, uid, hr_hours_per_day_id, dia, hora_entrada, aux_end, feriado=feriado[0])  
            val2 = parametros_turnos_obj.calculte_overtime(cr, uid, hr_hours_per_day_id, dia+1 if dia < 7 else 1, aux_start, hora_salida, feriado=feriado[1]) 
            total_acum = [val1, val2]
            for t in total_acum:
                for k in t: 
                    if not k in sobretiempos:
                        sobretiempos[k] = t[k]
                    else:
                        sobretiempos[k] += t[k]
        return sobretiempos   

    def buscar_feriados(self, cr, uid, ids, fecha):
		calendario_obj = self.pool.get('hr.holidays.calendar')
		calendario_id = calendario_obj.search(cr, uid, [('date','=', fecha)]) 
		if calendario_id:
		   return True
		return False 	
    
    def calcular_sobretiempo(self, cr, uid, ids,  date_start, date_end, emp_id, line_id, shift_id=None):
        parametros_turnos_obj = self.pool.get('horas.extras.parametros.turnos')
        day = self.diff_days(date_start, date_end)
        date_convert_inp = self.convert_datetime(date_start)
        date_convert_exit = self.convert_datetime(date_end)
        hora_entrada = date_convert_inp['time']
        hora_salida = date_convert_exit['time']
        date_start = date_convert_inp['date'] 
        date_end = date_convert_exit['date']
        sobretiempos = {}
        dia1_feriado = self.buscar_feriados(cr, uid, ids, date_start)
        dia2_feriado = self.buscar_feriados(cr, uid, ids, date_end)
        if not shift_id:
           shitf_week = self.get_shitf_week(cr, uid, ids, date_start, date_end, emp_id)
           shift_id = shitf_week.shift_id
        hr_hours_per_day_id = self.get_hours_per_day_id(date_start, shift_id)
        if date_start == date_end:
           dia = self.day_week(date_end)
           sobretiempos = parametros_turnos_obj.calculte_overtime(cr, uid, hr_hours_per_day_id, dia, hora_entrada, hora_salida, feriado=dia1_feriado)
        else:
            dia = self.day_week(date_start)
            if hora_entrada >= hora_salida:
               sobretiempos = self.calcular_sobretiempos_aux(cr, uid, hr_hours_per_day_id, dia, hora_entrada, hora_salida, feriado=[dia1_feriado, dia2_feriado])
            else:
                dia_1 = self.day_week(date_start)
                dia_2 = self.day_week(date_end)
                dates = range(dia_1, dia_2+1)
                calculos = []
                for dia in dates:
                    val = parametros_turnos_obj.calculte_overtime(cr, uid, hr_hours_per_day_id, dia, hora_entrada, hora_salida)
                    calculos.append(val)
                for d in calculos:
                    for k in d:
                        if not k in sobretiempos:
                           sobretiempos[k] = d[k]
                        else:
                            sobretiempos[k] += d[k]
        return sobretiempos 
    
    def buscar_hojas_porcesados(self, cr, uid, ids, nomina, sedes, actualizar_hojas=False ):
        horas_extras_obj = self.pool.get('horas.extras.planificacion')
        horas_extras_lines_obj = self.pool.get('horas.extras.planificacion.lineas')
        if actualizar_hojas:
           horas_extras_obj.write(cr, uid, actualizar_hojas, {'state':'processing'})
           horas_extras_obj.write_state_lines(cr ,uid, actualizar_hojas, 'processing')
        horas_extras_id = horas_extras_obj.search(cr, uid, [('state','=','processing')]) 
        horas_extras_line = []
        if not horas_extras_id:
           raise osv.except_osv('ERROR', "No hay hojas de horas extras en estado procesado") 
        horas_extras_brw = horas_extras_obj.browse(cr, uid, horas_extras_id)
        horas_extras_line= [line for i in horas_extras_brw for line in i.planificacion_lineas_ids 
                            if line.empleado_id.hr_sede_id.id in [s.id for s in sedes] and line.empleado_id.hr_payroll_id.id == nomina.id]
        validar_excepciones = horas_extras_lines_obj.search(cr, uid, [('horas_extras_planificacion_id', 'in', horas_extras_id), 
                                                                     ('state', '=', 'exception')])
        if validar_excepciones:
           raise osv.except_osv('ERROR', "No puedes cargar lineas con estado de excepcion")   
        return horas_extras_line
        
    
    def verificar_conceptos_variables(self, cr, uid, ids, variable_id, lineas, nomina, sedes):
        horas_extras_line = []
        conceptos_variables_obj =  self.pool.get('hr.variable.concepts')
        hr_variable_concept_line_obj = self.pool.get('hr.variable.concept.line')
        horas_extras_obj = self.pool.get('horas.extras.planificacion')
        horas_extras_lines_obj = self.pool.get('horas.extras.planificacion.lineas')
        horas_extras_calculos_obj = self.pool.get('horas.extras.planificacion.calculos')
        variable_id = conceptos_variables_obj.search(cr, uid, [('id','=',variable_id)])
        if not variable_id:
           return False  
        horas_extras_id = horas_extras_obj.search(cr, uid, [('variable_concept_id','in', variable_id), ('state','=','load')])
        lineas_ids = horas_extras_lines_obj.search(cr, uid, [('horas_extras_planificacion_id','in', horas_extras_id), ('state','=','load') ])
        calculos_ids = horas_extras_calculos_obj.search(cr, uid, [('horas_extras_planificacion_id','in', horas_extras_id)])
        calculos_brw = horas_extras_calculos_obj.browse(cr, uid, calculos_ids)
        trabajadores_cargados = [i.trabajador_id.id for i in calculos_brw]
        trabajadores_procesados = [i.empleado_id.id for i in lineas]
        lineas_coincidentes = []
        if set(trabajadores_cargados).intersection(trabajadores_procesados):
           for i in calculos_brw:
               variable_ids = hr_variable_concept_line_obj.search(cr, uid, [('variable_concept_id','in',variable_id), 
                                                       ('employee_id','=', i.trabajador_id.id), 
                                                       ('concept_id','=',i.concept_id.id)] )
               lineas_coincidentes.append(variable_ids)
        if lineas_coincidentes:
           conceptos_variables_obj.unlink(cr, 1, variable_id)
           return calculos_brw
        return False 
        
    def calcular(self, cr, uid, ids, horas_extras_line, context=None):
        horas_extras_execcion_obj = self.pool.get('horas.extras.planificacion.lineas.excepciones')
        horas_extras_lines_obj = self.pool.get('horas.extras.planificacion.lineas')
        dict_emp_values = {}  
        for i in horas_extras_line:
            dict_emp_values[i.empleado_id.id] = {}
        for i in horas_extras_line:
            if i.state != 'cancel':
                if i.excepciones:
                   horas_extras_execcion_id = horas_extras_execcion_obj.search(cr, uid, [('horas_extras_planificacion_lineas_id', 'in', [i.id])])
                   horas_extras_execcion_brw = horas_extras_execcion_obj.browse(cr, uid, horas_extras_execcion_id)
                   for exect in horas_extras_execcion_brw:
                       if not exect.concept_id.id in dict_emp_values[i.empleado_id.id]: 
                          dict_emp_values[i.empleado_id.id][exect.concept_id.id] = exect.value
                       else:
                           dict_emp_values[i.empleado_id.id][exect.concept_id.id] += exect.value
                   horas_extras_execcion_obj.unlink(cr, uid, horas_extras_execcion_id)
                   horas_extras_lines_obj.write(cr, uid, i.id, {'excepciones':False})
                else:
                    fecha_desde = "%s %s"%(i.fecha_desde, i.hora_desde)
                    fecha_hasta = "%s %s"%(i.fecha_hasta, i.hora_hasta)
                    valor = self.calcular_sobretiempo(cr, uid, ids, fecha_desde, fecha_hasta, i.empleado_id.id, i.id)
                    for concepto in valor: 
                        if not concepto in dict_emp_values[i.empleado_id.id]:
                           dict_emp_values[i.empleado_id.id][concepto] = valor[concepto]
                        else:
                            dict_emp_values[i.empleado_id.id][concepto] += valor[concepto]
        return  dict_emp_values   
    
    def sumar_diccionarios(self, d1, d2):
        aux = {} 
        tan1 = len(d1) 
        tan2 = len(d2)
        if tan1 > tan2:
            aux = d1 
            d1 = d2 
            d2 = aux
        for i in d2:
            if i in d1:
               for c in d2[i]:
                   if c in d1[i]:
                      d1[i][c] += d2[i][c]
                   else:
                       d1[i][c] = d2[i][c]
            else:
                d1[i] = d2[i]
        return d1
    
    def recalcular(self, cr, uid, ids, lineas, horas_extras_line, context=None):
        lineas_omitidas = [l.id for i in lineas for l in i.horas_extras_planificacion_id.planificacion_lineas_ids]
        horas_extras_line = [l for l in horas_extras_line if not l.id in lineas_omitidas]
        val1 = {} 
        val2 = {} 
        total = {} 
        for i in lineas:
            val1[i.trabajador_id.id] = {} 
        for i in lineas:
            val1[i.trabajador_id.id].update({i.concept_id.id:i.value})
        val2 = self.calcular(cr, uid, ids, horas_extras_line)
        total = self.sumar_diccionarios(val1,val2)
        return total 
     
    def crear_lineas_conceptos_variables(self, cr, uid, ids,  context = None):
        horas_extras_execcion_obj = self.pool.get('horas.extras.planificacion.lineas.excepciones')
        horas_extras_obj = self.pool.get('horas.extras.planificacion')
        horas_extras_lines_obj = self.pool.get('horas.extras.planificacion.lineas')
        hr_variable_concept_line_obj = self.pool.get('hr.variable.concept.line')
        idsw = [context['active_id']]
        nomina = self.browse(cr, uid, ids[0], context).hr_payroll_id
        sedes = self.browse(cr, uid, ids[0], context).hr_sede_ids
        horas_extras_line = self.buscar_hojas_porcesados(cr, uid, ids, nomina, sedes)
        process_horas_extras = self._get_procesos_horas_extras(cr, uid, nomina, horas_extras_line, context)
        procesos_ids =  list(set(process_horas_extras.values()))
        conceptos_variables_procesos = self._get_conceptos_variables_procesos(cr, uid, procesos_ids, context)
        variable_ids = [conceptos_variables_procesos[i] for i in conceptos_variables_procesos] 
        recalcular = False 
        
        verificar_conceptos_variables = self.verificar_conceptos_variables(cr, uid, ids, variable_ids, horas_extras_line, nomina, sedes)          
        if verificar_conceptos_variables:
           actualizar_hojas = [i.horas_extras_planificacion_id.id for i in verificar_conceptos_variables] 
           horas_extras_line = self.buscar_hojas_porcesados(cr, uid, ids, nomina, sedes, actualizar_hojas=actualizar_hojas)
           process_horas_extras = self._get_procesos_horas_extras(cr, uid, nomina, horas_extras_line, context)
           procesos_ids =  list(set(process_horas_extras.values()))
           conceptos_variables_procesos = self._get_conceptos_variables_procesos(cr, uid, procesos_ids, context)
           recalcular = True
        
        dict_emp_values = {}
        variable_create = {}
        if recalcular:
           dict_emp_values = self.recalcular(cr, uid, ids, verificar_conceptos_variables, horas_extras_line)
        else:
            dict_emp_values = self.calcular(cr, uid, ids, horas_extras_line)	
        
        for i in dict_emp_values:
            variable_create[i] = {}
        
        for e in dict_emp_values:
            for c in dict_emp_values[e]:
                variable_create[e].update({c:{'variable_concept_id': None,
                    'employee_id': e,
                    'concept_id': c,
                    'type': 'qty',
                    'value': dict_emp_values[e][c]
                     }})
        
        horas_extras_update = []
        lines_update = []
        
        for i in horas_extras_line:
            if i.horas_extras_planificacion_id.id in process_horas_extras:
                horas_extras_update.append(i.horas_extras_planificacion_id.id )
                process_id = process_horas_extras[i.horas_extras_planificacion_id.id]
                variable_id = conceptos_variables_procesos[process_id]
                if i.state != 'cancel':
                   lines_update.append(i.id)
                   for e in variable_create:
                        for c in variable_create[e]:
                            variable_create[e][c]['variable_concept_id'] = variable_id
                   horas_extras_obj.write(cr, uid, i.horas_extras_planificacion_id.id, {'variable_concept_id':variable_id})        
        
        for i in variable_create:
            for c in variable_create[i]:
                if variable_create[i][c]['value'] > 0.00: 
                   hr_variable_concept_line_obj.create(cr, uid, variable_create[i][c], context)             
         
        horas_extras_obj.write(cr, uid, horas_extras_update, {'state':'load'})
        horas_extras_obj.write_state_lines(cr ,uid, horas_extras_update, 'load', lines=lines_update)
        
        return {
            'type': 'ir.actions.act_window_close',
            'active_id': ids
         }





horas_extras_wizard_concept_variable()
