#!/usr/bin/python
# -*- encoding: utf-8 -*-

from time import strftime
from datetime import *
import datetime
from dateutil import relativedelta
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import logging
import base64
from xlwt import Workbook
from xlwt import Font
from xlwt import XFStyle
from xlwt import Borders
import os
import math

class hr_sede(osv.osv):
    _inherit = 'hr.sede'
    
    _columns = {
        'forma_pago_bono_alimentacion': fields.selection([
            ('sodexo', 'Sodexo'),
            ('bicentenario', 'Bicentenario'),
        ], 'Forma pago bono alimentacion', select=True),
        'serial_bicentenario':fields.char('Serial', size=64),
    }
    
hr_sede()

class hr_process(osv.osv):
    _inherit = 'hr.process'
    
    def generar_archivo_bono_alimentacion(self, cr, uid, ids, context):
        process_brw = self.browse(cr, uid, ids[0], context)
        list_sodexo = [x for x in process_brw.hr_payslip_ids \
          if x.employee_id.hr_sede_id.forma_pago_bono_alimentacion == 'sodexo']
        list_bicentenario = [x for x in process_brw.hr_payslip_ids \
          if x.employee_id.hr_sede_id.forma_pago_bono_alimentacion == 'bicentenario']
          
        if list_sodexo:
            self.generar_xls_sodexo(cr, uid, list_sodexo, context)
        if list_bicentenario:
            self.generar_txt_bicentenario(cr, uid, list_bicentenario, context)
          
        return True
        
    def generar_txt_bicentenario(self, cr, uid, payslip_brw, context):
        send_txt, total = self.txt_tbicentenario(cr, uid, payslip_brw, context)
        
        fecha = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        id_att = self._write_attachment_balimentacion(cr, uid, payslip_brw[0].hr_process_id.id, 
          send_txt, 'AlimentacionBBU-'+str(fecha)+'-Bs.'+str(total)+'.txt', context)
        
        if id_att:
            self._logger.info("%s: Se genero el txt de pago." % (payslip_brw[0].hr_process_id.name,))
        else:
            raise osv.except_osv(_('Error!'),_('El archivo no se genero por estar vacío. \
              Favor revisar datos de cuantas bancarias de trabajadores'))
        return True


    def txt_tbicentenario(self, cr, uid, payslip_brw, context):
        """Metodo Ticket Banco Bicentenario"""
        txt_string = ''
        total = 0.0
        for payslip in payslip_brw:
            employee = []
            if len(payslip.employee_id.identification_id) > 9:
                raise osv.except_osv(_('Error!'),_('El número de cédula del \
                  empleado %s tiene mas dígitos de los debidos')%(payslip.employee_id.name))
            txt_string = txt_string + payslip.employee_id.hr_sede_id.serial_bicentenario
            nac = 'V' if payslip.employee_id.country_id.code == 'VE' else 'E'
            while len(nac)+len(payslip.employee_id.identification_id) < 11:
                nac = nac+'0'
            cedula = nac+payslip.employee_id.identification_id
            txt_string =  txt_string + str(cedula)
            amount_allocation = sum([line.total for line in payslip.line_ids \
              if line.type_calc == 'allocation'])
            amount_deduction = sum([line.total for line in payslip.line_ids \
              if line.type_calc == 'deduction'])
            monto = str(round(amount_allocation - amount_deduction, 2))
            # Verificar si el monto no tiene coma o punto y colocarselo 
            # es caso de que no tenga
            if monto.find(',') < 0 and monto.find('.') < 0: 
                monto = monto + '.'
            # Verificar que tenga dos decimales, si tienen menos de dos 
            # se agregan ceros al final para completar
            while ((monto.find(',') > 0 and (len(monto) - monto.find(',')) < 3) \
              or (monto.find('.') > 0 and (len(monto) - monto.find('.')) < 3)):
                monto = monto + '0'
            monto = monto.replace(',','')
            monto = monto.replace('.','')
            while len(monto) < 15:
                monto = '0'+monto
            txt_string =  txt_string + str(monto) +'\n'
            total = total + round(amount_allocation - amount_deduction, 2)

        return txt_string, total
        
    def generar_xls_sodexo(self, cr, uid, payslip_brw, context):
        '''
        payslip_brw: brw de los payslip
        '''
        #~ payslip_brw = self.browse(cr, uid, ids, context)
        tarjetas = []
        tickets = []
        total_tarjetas = 0.0
        for payslip in payslip_brw:
            if payslip.employee_id.type_bono_alimen == 'tarjeta':
                total_tarjetas = total_tarjetas + payslip.total
                tarjetas.append(payslip)
            elif payslip.employee_id.type_bono_alimen == 'tickets':
                tickets.append(payslip)
        if tarjetas:
            send_xls, name = self.xls_tarjetas(cr, uid, payslip_brw[0].hr_process_id.id, 
              tarjetas, total_tarjetas, context)
            id_att = self._write_attachment_balimentacion(cr, uid, 
              payslip_brw[0].hr_process_id.id, 
              send_xls.read(), name, context)
            send_xls.close()
            os.remove(send_xls.name)
        if tickets:
            send_xls, name = self.xls_tickets(cr, uid, payslip_brw[0].hr_process_id.id, 
              tickets, context)
            id_att = self._write_attachment_balimentacion(cr, uid, 
              payslip_brw[0].hr_process_id.id, 
              send_xls.read(), name, context)
            send_xls.close()
            os.remove(send_xls.name)
        
        return True

    def _write_attachment_balimentacion(self, cr, uid, ids, root, name, context):
        '''
        Encrypt xls, save it to the db and view it on the client as an attachment
        '''
        fecha = datetime.datetime.now().strftime('%Y-%m-%d')
        id_att = self.pool.get('ir.attachment').create(cr, uid, {
            'name': name,
            'voucher_support': name,
            'res_name': name,
            'datas': base64.encodestring(root),
            'datas_fname': name,
            'res_model': 'hr.process',
            'res_id': str(ids),
            }, context=context)
        return id_att
        
    def xls_tarjetas(self, cr, uid, ids, tarjetas, total_tarjetas, context): 
        codigo_cliente = tarjetas[0].employee_id.company_id.cod_bono_alim \
          if tarjetas[0].employee_id.company_id.cod_bono_alim else '---'
        header_style = XFStyle()
        borders = Borders()
        borders.left = 1
        borders.right = 1
        borders.top = 1
        borders.bottom = 1
        header_style.borders = borders
        first_book = Workbook()
        ws1 = first_book.add_sheet('first_sheet', cell_overwrite_ok=True)
        ws1.write_merge(0, 0, 0, 4,)
        ws1.row(0).height = 1500
        ws1.write(0, 0, 'Bienvenido a la planilla de carga de pedidos de \
          Sodexo, por favor completar solo los espacios en blanco.', header_style)
        ws1.write(1, 0, 'Codigo cliente:')
        ws1.write(1, 1, codigo_cliente)
        ws1.write(1, 2, 'Producto:')
        ws1.write_merge(1, 1, 3, 4,)
        ws1.write(1, 3, 'TARJETA ALIMENTACION PASS')
        ws1.write(2, 0, 'Observaciones:')
        ws1.write_merge(2, 2, 1, 4,)
        ws1.write_merge(3, 3, 0, 1,)
        ws1.write(3, 0, 'Monto total del pedido:')
        ws1.write(3, 2, total_tarjetas)
        ws1.write_merge(3, 3, 3, 4,)
        ws1.write(4, 0, 'Cedula')
        ws1.write(4, 1, 'Apellido')
        ws1.write(4, 2, 'Nombre')
        ws1.write(4, 3, 'Monto de la Tarjeta')
        ws1.write(4, 4, 'Cod. Pto. Entrega')
        
        for i in range(len(tarjetas)):
            payslip = tarjetas[i]
            second_surname = ' ' + payslip.employee_id.second_surname if \
              payslip.employee_id.second_surname else ' '
            second_name = ' ' + payslip.employee_id.second_name if \
              payslip.employee_id.second_name else ' '
            ws1.write(i+5, 0, payslip.employee_id.identification_id)
            ws1.write(i+5, 1, payslip.employee_id.first_surname + second_surname)
            ws1.write(i+5, 2, payslip.employee_id.first_name + second_name)
            ws1.write(i+5, 3, payslip.total)
            ws1.write(i+5, 4, payslip.employee_id.hr_sede_id.cod_bono_alim)
        
        fecha = datetime.datetime.now().strftime('%Y-%m-%d')
        name = 'bono_alimentacion_tarjetas-'+str(fecha)+'.xls'
        first_book.save('/tmp/'+name)
        f = open('/tmp/'+name)
        return f,name
        
    def xls_tickets(self, cr, uid, ids, tickets, context):
        fecha = datetime.datetime.now().strftime('%Y-%m-%d')
        codigo_cliente = tickets[0].employee_id.company_id.cod_bono_alim \
          if tickets[0].employee_id.company_id.cod_bono_alim else '---'
        
        total_tickets = 0.0
        header_style = XFStyle()
        borders = Borders()
        borders.left = 1
        borders.right = 1
        borders.top = 1
        borders.bottom = 1
        header_style.borders = borders
        first_book = Workbook()
        ws1 = first_book.add_sheet('first_sheet', cell_overwrite_ok=True)
        ws1.write_merge(0, 0, 0, 7,)
        ws1.row(0).height = 1500
        ws1.write(0, 0, 'Bienvenido a la planilla de carga de pedidos de \
          Sodexo, por favor completar solo los espacios en blanco.', header_style)
        ws1.write(1, 0, 'Codigo cliente:')
        ws1.write(1, 1, codigo_cliente)
        ws1.write(1, 2, 'Producto:')
        ws1.write_merge(1, 1, 3, 7,)
        ws1.write(1, 3, 'TARJETA ALIMENTACION PASS')
        ws1.write(2, 0, 'Observaciones:')
        ws1.write_merge(2, 2, 1, 7,)
        ws1.write_merge(3, 3, 0, 1,)
        ws1.write(3, 0, 'Monto total del pedido:')
        ws1.write_merge(3, 3, 3, 7,)
        ws1.write(4, 0, 'Cedula')
        ws1.write(4, 1, 'Apellido y nombre')
        ws1.write(4, 2, 'Cod. Pto. Entrega')
        ws1.write(4, 3, 'Cod. Especial')
        ws1.write(4, 4, 'Cantidad Cheques 1')
        ws1.write(4, 5, 'Valor Facial 1')
        ws1.write(4, 6, 'Cantidad Cheques 2')
        ws1.write(4, 7, 'Valor Facial 2')
        
        # Busqueda del parametro que define la denominacion maxima y minima 
        # de los ticket de alimentacion
        parameter_obj = self.pool.get('hr.parameters')
        param_max_min_tick_alim = parameter_obj.search(cr, uid, [
          ('type', '=', 'max_min_tick_alim')])
        if not param_max_min_tick_alim:
            raise osv.except_osv(_('ERROR!'), _('No hay un parámetro definido \
              para Denominación máxima y mínima de los ticket de alimentación. Ir a \
              la pestaña Configuracion -> Configuraciones de nomina -> Parametros \
              de nomina y registrar un parametro de tipo Denominación máxima \
              y mínima de los ticket de alimentación'))
            
        maximo = parameter_obj.browse(cr, uid, param_max_min_tick_alim)[0].max_tickets_alim
        minimo = parameter_obj.browse(cr, uid, param_max_min_tick_alim)[0].min_tickets_alim
    
        for i in range(len(tickets)):
            payslip = tickets[i]

            num_payment_1 = payslip.employee_id.hr_sede_id.num_payment_1
            amount_payment_1 = payslip.employee_id.hr_sede_id.amount_payment_1
            num_payment_2 = payslip.employee_id.hr_sede_id.num_payment_2
            amount_payment_2 = payslip.employee_id.hr_sede_id.amount_payment_2
                    
            if payslip.employee_id.hr_sede_id.amount_bono_alim <> payslip.total:
                cant_1 = math.floor(payslip.total/payslip.employee_id.hr_sede_id.amount_payment_1)
                mont_1 = payslip.employee_id.hr_sede_id.amount_payment_1
                
                if (cant_1*mont_1) == payslip.total:
                    num_payment_1 = cant_1
                    amount_payment_1 = mont_1
                    num_payment_2 = 0
                    amount_payment_2 = 0.0
                    
                else:
                    num_payment_1 = cant_1
                    amount_payment_1 = mont_1

                    rest = (payslip.total-(cant_1*mont_1)) if payslip.total > \
                      (cant_1*mont_1) else ((cant_1*mont_1)-payslip.total)
                    if rest <= minimo:
                        num_payment_2 = 1
                        amount_payment_2 = minimo
                    else: 
                        num_payment_2 = 1
                        while rest > maximo:
                            rest = rest/num_payment_2
                            num_payment_2 = num_payment_2 + 1
                        
                        amount_payment_2 = math.ceil(rest)
                        
            ws1.write(i+5, 0, payslip.employee_id.identification_id)
            ws1.write(i+5, 1, payslip.employee_id.name)
            ws1.write(i+5, 2, payslip.employee_id.hr_sede_id.cod_bono_alim)
            ws1.write(i+5, 3, '')
            ws1.write(i+5, 4, num_payment_1)
            ws1.write(i+5, 5, amount_payment_1)
            ws1.write(i+5, 6, num_payment_2)
            ws1.write(i+5, 7, amount_payment_2)
            total_tickets = total_tickets + (num_payment_1*amount_payment_1 + \
              num_payment_2*amount_payment_2)
            
        ws1.write(3, 2, total_tickets)
        name = 'bono_alimentacion_tickets-'+str(fecha)+'.xls'
        first_book.save('/tmp/'+name)
        f = open('/tmp/'+name)
        return f,name
        
hr_process()

class res_company(osv.osv):
    _inherit = 'res.company'
    
    _columns = {
        'cod_bono_alim': fields.char('Código empresa bono alimentación', size=256),
    }

res_company()
