from report import report_sxw
from report.report_sxw import rml_parse
from operator import itemgetter
import time

class ProcessConceptsParser(report_sxw.rml_parse):
    def __init__(self, cr, uid, name, context):
        super(ProcessConceptsParser, self).__init__(cr, uid, name, 
          context)
        self.localcontext.update({
            'report_lines': self.get_report_lines,
            'total_asignaciones': self.get_total_asignaciones,
            'total_deducciones': self.get_total_deducciones,
            'company': self.get_company,
            'rif': self.get_rif,
            'time': time,
            'num_empleados': self.get_num_empleados,
            'emp_femenino': self.get_emp_femenino,
            'emp_masculino': self.get_emp_masculino,
        })
        self.context = context
        self.total_asignaciones = 0
        self.total_deducciones = 0
        self.result = []
        self.num_empleados = 0
        self.emp_femenino = 0
        self.emp_masculino = 0

    def get_company(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, 
          self.uid).company_id.name

    def get_rif(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, 
          self.uid).company_id.partner_id.vat

    def get_report_lines(self, process):
        self.total_asignaciones = 0
        self.total_deducciones = 0
        self.result = []
        cpt_obj = self.pool.get('hr.concept')
        #aportes patronales
        conceptos_aportes_patronales_ids = cpt_obj.search(self.cr,
          self.uid, [('type', '=', 'aporte_patronal')])
          
        codigos_aportes_patronales = []
        conceptos_aportes_patronales = []
        
        if conceptos_aportes_patronales_ids:
            conceptos_aportes_patronales = cpt_obj.browse(self.cr, 
              self.uid, conceptos_aportes_patronales_ids)
            codigos_aportes_patronales = [ap.code \
              for ap in conceptos_aportes_patronales]

        # Se buscan los payslip.lines que son visibles en los payslips y 
        # se cuentan los trabajadores totales, hombres y mujeres
        payslips = [payslip for payslip in process.hr_payslip_ids]
        num_emp = len(payslips)
        emp_femenino = 0
        emp_masculino = 0
        payslip_lines_visibles = []
        for payslip in payslips:
            payslip_lines_visibles.extend([payslip_line.id \
              for payslip_line in payslip.line_ids])
              
            emp_femenino = (emp_femenino + 1) if payslip.employee_id.gender == 'female' \
              else emp_femenino
            emp_masculino = (emp_masculino + 1) if payslip.employee_id.gender == 'male' \
              else emp_masculino
        
        self.num_empleados = num_emp
        self.emp_femenino = emp_femenino
        self.emp_masculino = emp_masculino
         
        payslip_line_obj = self.pool.get('hr.payslip.line')
        
        # Primero se obtienen las asignaciones
        payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
          [('process_id', '=', process.id),
          ('id', 'in', payslip_lines_visibles), 
          ('type_calc', '=', 'allocation')])
          
        payslip_lines = payslip_line_obj.browse(self.cr, self.uid, 
          payslip_lines_ids)
          
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            if not (code, concept) in result:
                result[(code, concept)] = [total, 1]
            else:
                result[(code, concept)][0] += total
                result[(code, concept)][1] += 1
            self.total_asignaciones += total
        
        self.result.extend([(code, concept, 'allocation', 
          result[(code, concept)]) for (code, concept) in result])
          
        # Luego se obtienen las deducciones
        payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
          [('process_id','=', process.id),
          ('id', 'in', payslip_lines_visibles),
          ('type_calc','=', 'deduction')])
        
        payslip_lines = payslip_line_obj.browse(self.cr, self.uid, 
          payslip_lines_ids)
          
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            if not (code, concept) in result:
                result[(code, concept)] = [total, 1]
            else:
                result[(code, concept)][0] += total
                result[(code, concept)][1] += 1
            self.total_deducciones += total
            
        self.result.extend([(code, concept, 'deduction', 
          result[(code, concept)]) for (code, concept) in result])

        payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
          [('process_id', '=', process.id),('code', 'in', 
          codigos_aportes_patronales)])
        
        payslip_lines = payslip_line_obj.browse(self.cr, self.uid, 
          payslip_lines_ids)
          
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            
            if not (code, concept) in result:
                result[(code, concept)] = [total, 1]
            else:
                result[(code, concept)][0] += total
                result[(code, concept)][1] += 1
                
        self.result.extend([(code, concept, 'allocation', 
          result[(code, concept)]) for (code, concept) in result])
        
        return self.result

    def get_total_asignaciones(self):
        return self.total_asignaciones

    def get_total_deducciones(self):
        return self.total_deducciones
        
    def get_num_empleados(self):
        return self.num_empleados

    def get_emp_femenino(self):
        return self.emp_femenino
        
    def get_emp_masculino(self):
        return self.emp_masculino
    
report_sxw.report_sxw(
                      'report.hr.process.concepts',
                      'hr.process',
                      'sisb_nomina/report/report_process_concepts.rml',
                      parser=ProcessConceptsParser, header=False
)
