from report import report_sxw
from report.report_sxw import rml_parse
from operator import itemgetter
import time

class reporte_mensual_conceptos(report_sxw.rml_parse):
    
    _name= 'reporte.mensual.conceptos'
    def __init__(self, cr, uid, name, context):
        super(reporte_mensual_conceptos, self).__init__(cr, uid, name, 
          context)
        self.localcontext.update({
            'report_lines': self.get_report_lines,
            'total_asignaciones': self.get_total_asignaciones,
            'total_deducciones': self.get_total_deducciones,
            'company': self.get_company,
            'rif': self.get_rif,
            'time': time,
            'get_mes':self.get_mes,
            'get_nomina':self.get_nomina,
            'get_tipo':self.get_type,
            'get_sedes':self.get_sedes,
        })
        self.context = context
        self.total_asignaciones = 0
        self.total_deducciones = 0
        self.result = []

    def get_company(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, 
          self.uid).company_id.name

    def get_rif(self):
        return self.pool.get('res.users').browse(self.cr, self.uid, 
          self.uid).company_id.partner_id.vat

    def get_report_lines(self, valor, process):
        monthr_brw = self.pool.get('hr.month').browse(self.cr, self.uid, valor['month_id'])
        period_in_month = [p.id for p in monthr_brw.hr_period_ids]

        self.total_asignaciones = 0
        self.total_deducciones = 0
        self.result = []
        cpt_obj = self.pool.get('hr.concept')
        #aportes patronales
        conceptos_aportes_patronales_ids = cpt_obj.search(self.cr,
          self.uid, [('type', '=', 'aporte_patronal')])
          
        codigos_aportes_patronales = []
        conceptos_aportes_patronales = []
        
        if conceptos_aportes_patronales_ids:
            conceptos_aportes_patronales = cpt_obj.browse(self.cr, 
              self.uid, conceptos_aportes_patronales_ids)
            codigos_aportes_patronales = [ap.code \
              for ap in conceptos_aportes_patronales]

        # Se buscan los payslip.lines que son visibles en los payslips
        process_month_ids = self.pool.get('hr.process').search(self.cr, self.uid, [('type', '=', valor['type']),
          ('hr_payroll_ids', '=', valor['payroll_id']),('hr_period_ids','=',period_in_month)])
        process_month_brw = self.pool.get('hr.process').browse(self.cr, self.uid, process_month_ids)
        
        payslips = []
        for process in process_month_brw:
            payslips.extend([payslip for payslip in process.hr_payslip_ids])
        
        payslip_line_obj = self.pool.get('hr.payslip.line')
        
        payslip_lines_total = []
        for payslip in payslips:
            if valor['sede_ids']:
                payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
                  [('slip_id', '=', payslip.id),('sede_id','in',valor['sede_ids'])])
            else:
                payslip_lines_ids = payslip_line_obj.search(self.cr, self.uid, 
                  [('slip_id', '=', payslip.id)])
              
            payslip_lines_total.extend([line for line in \
              payslip_line_obj.browse(self.cr, self.uid, payslip_lines_ids)])
        
        # Primero se obtienen las asignaciones
        payslip_lines = [line for line in payslip_lines_total \
          if ((line.type_calc == 'allocation') and (line.appears_on_payslip == True))]
        
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            if not (code, concept) in result:
                result[(code, concept)] = total
            else:
                result[(code, concept)] += total
            self.total_asignaciones += total
            
        self.result.extend([(code, concept, 'allocation', 
          result[(code, concept)]) for (code, concept) in result])
          
        # Se obtienen las deducciones
        payslip_lines = [line for line in payslip_lines_total \
          if ((line.type_calc == 'deduction') and (line.appears_on_payslip == True))]
        
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            if not (code, concept) in result:
                result[(code, concept)] = total
            else:
                result[(code, concept)] += total
            self.total_deducciones += total
            
        self.result.extend([(code, concept, 'deduction', 
          result[(code, concept)]) for (code, concept) in result])
        
        # Se obtienen los aportes patronales
        payslip_lines = [line for line in payslip_lines_total if line.code in codigos_aportes_patronales]
        
        result = {}
        for payslip_line in payslip_lines:
            code = payslip_line.code
            concept = payslip_line.concept_id.name
            total = payslip_line.total
            
            if not (code, concept) in result:
                result[(code, concept)] = total
            else:
                result[(code, concept)] += total
                
        self.result.extend([(code, concept, 'allocation', 
          result[(code, concept)]) for (code, concept) in result])
        return self.result

    def get_total_asignaciones(self):
        return self.total_asignaciones

    def get_total_deducciones(self):
        return self.total_deducciones
        
    def get_mes(self, valor):
        obj_mes = self.pool.get('hr.month')
        res = {}
        row_brw = obj_mes.browse(self.cr, self.uid, valor['month_id'])
        if not isinstance(row_brw, list):
            row_brw = [row_brw]
        for row in row_brw:
            res = {
            'name':row.name.upper(),
            }
        return res
        
    def get_nomina(self, valor):
        obj_mes = self.pool.get('hr.payroll')
        res = {}
        row_brw = obj_mes.browse(self.cr, self.uid, valor['payroll_id'])
        res = {
            'name':row_brw.name.upper(),
        }
        return res
        
    def get_type(self, valor):
        return valor['type'].upper()
        
    def get_sedes(self, valor):
        obj_mes = self.pool.get('hr.sede')
        res = {}
        if valor['sede_ids']:
            row_brw = obj_mes.browse(self.cr, self.uid, valor['sede_ids'])
            row_brw = [x.name for x in row_brw]
            name = ', '.join(row_brw)
            if not isinstance(row_brw, list):
                row_brw = [row_brw]
            for row in row_brw:
                res = {
                    'name':name,
                }
        else:
            res = {
                'name':'Todas las Sedes',
            }
        return res
        
report_sxw.report_sxw(
    'report.reporte.mensual.conceptos',
    'hr.process',
    parser=reporte_mensual_conceptos,
    rml='sisb_nomina/report/reporte_mensual_conceptos.rml',
                      
)
