# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time

from osv import fields, osv

class hr_payroll_year_salary(osv.osv_memory):
   _name = "hr.payroll.year.salary"
   
   def _select_types(self, cr, uid, context=None):
        if context is None:
            context = {}
        res = [('', '')]
        
        parameter_obj = self.pool.get('hr.parameters')
        concept_obj = self.pool.get('hr.concept')
        concept_id = concept_obj.search(cr, uid, [])
        
        param_conceptos_util = parameter_obj.search(cr, uid, [('type', '=', 'conceptos_util')])
        param_conceptos_deven = parameter_obj.search(cr, uid, [('type', '=', 'conceptos_deven')])
        
        if concept_id:
           res.append(( str(concept_id), 'Todos los conceptos'))
        
        
        for i in parameter_obj.browse(cr, uid, param_conceptos_util):
            l = [n.id for n in i.concept_util_ids]
            res.append(( str(l), 'Conceptos que inciden en Utilidades'))
        
        for i in parameter_obj.browse(cr, uid, param_conceptos_deven):
            l = [n.id for n in i.concept_deven_ids]
            res.append(( str(l), 'Conceptos que inciden en I.S.R.L. para Devengado'))

        return res
   
   _columns = {
        'employee_ids': fields.many2many('hr.employee', 'payroll_year_rel','payroll_year_id','emp_id', 'Employees',required=True),
        'date_from': fields.date('Start Date', required=True),
        'date_to': fields.date('End Date', required=True),
        'salary_on': fields.selection([('current_month','Current Month Date'),('next_month','Next Month Date')],'Salary On'),
        'param': fields.selection(_select_types, "Tipo de parametro", size=64, required=True),
        
       }


   _defaults = {
        'date_from': lambda *a: time.strftime('%Y-01-01'),
        'date_to': lambda *a: time.strftime('%Y-%m-%d'),
        'salary_on': 'current_month'
    }

   def print_report(self, cr, uid, ids, context=None):
        """
         To get the date and print the report
         @param self: The object pointer.
         @param cr: A database cursor
         @param uid: ID of the user currently logged in
         @param context: A standard dictionary
         @return: return report
        """
        if context is None:
            context = {}
        datas = {'ids': context.get('active_ids', [])}

        res = self.read(cr, uid, ids, ['employee_ids',  'date_from', 'date_to', 'salary_on', 'param'], context=context)
        res = res and res[0] or {}
        datas['form'] = res
        datas['ids'] = res.get('employee_ids',[])
        
        cod_number_employees = [x.cod_number for x in \
          self.pool.get('hr.employee').browse(cr, uid, res.get('employee_ids',[]))]
        for i,code in enumerate(cod_number_employees):
            message = 'Se consulto reporte Registro de Salario del Trabajador %s'%(code)
            self.log(cr, uid, res.get('employee_ids',[])[i], message)
            
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'year.salary',
            'datas': datas,
       }

hr_payroll_year_salary()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
