#!/usr/bin/env python
# -*- coding: utf-8 -*--

from time import strftime
from datetime import *
import datetime
from dateutil import relativedelta
import netsvc
from osv import fields, osv
import tools
from tools.translate import _
import logging
import base64
import os
import math
import calendar
import re 
import unicodedata




class wizard_generador_txt_FAOV(osv.osv_memory):
    _name = 'wizard.generador.txt.FAOV'
    _description = 'Wizar para generar TXT para sistema FAOV'
    _columns = {
    'month_id': fields.many2one('hr.month', 'Mes', required=True), 
    'sede_id': fields.many2one('hr.sede', 'Sede', required=True), 
    }
    
    def generar_txt_FAOV(self, cr, uid, ids, context=None):
        
        brw = self.browse(cr, uid, ids)[0]
        obj_period = self.pool.get('hr.period')
        obj_process = self.pool.get('hr.process') 
        obj_payslip_line = self.pool.get('hr.payslip.line') 
        sede_id = brw.sede_id.id
        obj_employee = self.pool.get('hr.employee') 
        year = brw.month_id.hr_year_id.year 
        month =  brw.month_id.month_year
        date_start = "%s-%s-01" %(year, month)
        date_end = "%s-%s-%s" %(year, month, calendar.monthrange(year, month)[1]) 
        fech=datetime.datetime.strptime(date_start, '%Y-%m-%d')
        parameter_obj = self.pool.get('hr.parameters')
        faov_id = parameter_obj.search(cr, uid, [('type', '=', 'concepto_faov'),('payroll_id', '=', None)])
        if not faov_id:
            raise osv.except_osv(_('ERROR!'), _('No hay un parametro definido para el concepto Retencion FAOV'))
        concept_id = parameter_obj.browse(cr, uid, faov_id)[0].concept_faov_id.id
        value = parameter_obj.browse(cr, uid, faov_id)[0].value
        if not value:
            raise osv.except_osv(_('ERROR!'), _('No se ha especificado el valor para calcular el porcentaje del concepto faov'))
        try:
            nombre_txt='N'+brw.sede_id.numero_cuenta_nomina_rel+fech.strftime('%m%Y') 
        except: 
          raise osv.except_osv(_('ERROR!'), _('No existe un numero de afiliacion de Nomina asociado a la Sede %s'%brw.sede_id.name))  
        
        period_id = obj_period.search(cr, uid, [('hr_month_id', '=', brw.month_id.id), ('state', '=', 'close')])
        process_id = obj_process.search(cr, uid, [('hr_period_ids', 'in', period_id)])
        process_brw = obj_process.browse(cr, uid, process_id)
        employee_id = obj_employee.search(cr, uid, [('hr_sede_id','=', sede_id)])
        obj_payroll = self.pool.get('hr.payroll')  
        payroll_id = obj_payroll.search(cr, uid, [])
        procesos_pago_list = ['nomina', 'anticipos', 'pago_especial', 
        'utilidades', 'vacaciones']
        for i in payroll_id:
            name = obj_payroll.browse(cr, uid, i).name  
            prestaciones_id = obj_process.search(cr, uid, [('hr_payroll_ids', '=', i), 
              ('type', '=', 'prestaciones'), ('hr_period_ids', 'in', period_id), ('state', '=', 'done')])
            procesos_pago = obj_process.search(cr, uid, [('hr_payroll_ids', '=', i), 
              ('type', 'in', procesos_pago_list), ('hr_period_ids', 'in', period_id), ('state', '=', 'done')])
            if procesos_pago and not prestaciones_id:
               name = obj_payroll.browse(cr, uid, i).name  
               raise osv.except_osv(_('ERROR!'), _('La %s para el mes de %s no \
                 posee proceso de Prestaciones Sociales aprobado'%(name, brw.month_id.name)))
        payslip_line_id = obj_payslip_line.search(cr, uid, [('process_id','in', process_id), ('concept_id', '=', concept_id), ('sede_id', '=', sede_id)])
        
        if not payslip_line_id:
           raise osv.except_osv(_('ERROR!'), _('Txt vacio o mal formado'))
        
        payslip_line_brw = obj_payslip_line.browse(cr, uid, payslip_line_id)
        salida = [('V', i.employee_id.identification_id, i.employee_id.first_name, i.employee_id.second_name, i.employee_id.first_surname, i.employee_id.second_surname, i.employee_id.start_rel_lab, i.employee_id.contract_id.date_end, round(i.total / value, 2)) for i in payslip_line_brw if i.total != 0.00]
        salida.sort()
        acum = {}
        
        for i in salida:
            acum[i[0:8]] = [0.00]
        
        for i in salida:
            if i[0:8] in acum:
               acum[i[0:8]].append(i[8])
        
        data = {'form':{'salida':[]}}
        ir_obj = self.pool.get('ir.actions.report.xml')
        ir_id = ir_obj.search(cr, 1, [('model','=',self._name)]) 
        ir_write= ir_obj.write(cr, 1, ir_id, {'name':nombre_txt})
        ir_browse= ir_obj.browse(cr, uid, ir_id)[0] 
        
        for n in acum:
            data['form']['salida'].append({'nac':n[0],'cedula':self.cvr_str(n[1]),'p_nombre':self.cvr_str(n[2]),'s_nombre':self.cvr_str(n[3]),'p_apellido':self.cvr_str(n[4]),'s_apellido':self.cvr_str(n[5]),'total':self.cvr_total(sum(acum[n])),'date_start':self.cvr_str(self.cvr_date(n[6])), 'date_end':self.cvr_str(self.cvr_date(self.date_end_rel(n[7], date_start, date_end)))})
      
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'txt.faov',
            'datas': data,
            }


    def cvr_str(self, string):

        if string:
           string = self.elimina_tildes(string)
        
        if  string:
            string = string.replace('.', '')
       
        if  string:
            string = string.replace(',', '')
        
        if  string:
            string = string.replace(' ', '')
            
        if  string:
            string = string.upper()
            
        return '' if not  string else string
    
    def elimina_tildes(self, s):
        return ''.join((c for c in unicodedata.normalize('NFD', unicode(s)) if unicodedata.category(c) != 'Mn'))
    
    def date_end_rel(self, fecha, date_start, date_end):
        if fecha:
            date_start = datetime.datetime.strptime(date_start, '%Y-%m-%d')
            date_end = datetime.datetime.strptime(date_end, '%Y-%m-%d')
            fin = datetime.datetime.strptime(fecha, '%Y-%m-%d')
            if fin >= date_start and fin <= date_end:
               return fecha
        else:
            return False    
    
    def cvr_date(self, fecha):
        if fecha:
            return  datetime.datetime.strptime(fecha, '%Y-%m-%d').strftime('%d%m%Y')                
        return False    
    
    def cvr_total(self, qty):
        if qty:
           qty = str(round(qty, 2))
           qty = qty.split('.')
           if len(qty[1]) < 2:
              qty = qty[0] + qty[1] + '0'
              return qty 
           else:
              return qty[0]+qty[1]
        return False     

wizard_generador_txt_FAOV()
