# -*- coding: utf-8 -*-


import time
from report import report_sxw
import datetime
from datetime import datetime


class analisis_facturas_report(report_sxw.rml_parse):
    
    _name = 'analisis.facturas.report'
    
    def __init__(self, cr, uid, name, context=None):
        super(analisis_facturas_report, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
        'get_empresa':self.get_empresa,
        'get_final_date':self.get_final_date,
        'cvr_date':self.cvr_date,
        'get_total':self.get_total,
        'get_subtotal':self.get_subtotal,
        'get_impuesto':self.get_impuesto,
        'get_equipo_ventas':self.get_equipo_ventas,
        'titulos_rml':self.titulos_rml,
        'sumador':self.sumador,
        'sumador_resumido':self.sumador_resumido,
        'sumador_resumido_product':self.sumador_resumido_product,
        'get_fecha':self.get_fecha,
        'generate_query':self.generate_query,})
        self.context = context
        self.cr = cr
        self.uid = uid
        self.sub_total = 0.0
        self.impuesto = 0.0
        self.total = 0.0
        self.salida_query = []
        self.equipo_ventas = None
        
    
    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
    
    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')    
    
    def cvr_date(self, fecha):
        d = datetime.datetime.strptime(fecha, '%Y-%m-%d').strftime('%Y/%m/%d')
        return d 
    
    def generate_query(self, value):
        """
        Funcion llamada en los rml para generar la consulta dependiendo
        del tipo de opcion que traiga la variable type
        parametros value: contiene el diccionario que carga 
        todas las variables necesarias utilizadas en las funciones   
        retorna una funcion dependiendo del tipo de reporte
        """
        period_id = value['form']['period_id']
        journal_id = value['form']['journal_id']
        is_periodo = value['form']['is_periodo']
        fecha_desde = value['form']['fecha_desde']
        fecha_hasta = value['form']['fecha_hasta']
        type = value['form']['type']
        all_sale = value['form']['all_sale']
        all_product = value['form']['all_product']
        user_id = value['form']['user_id']
        product_id = value['form']['product_id']
        partner_id = value['form']['partner_id']
        formato = value['form']['formato']
        is_refund = value['form']['is_refund']
        journal_access = [int(i) for i in value['form']['journal_access']]
        is_dadiva = value['form']['is_dadiva']
        if type == 'diario':
           return self.report_sale_diario(period_id, journal_id, fecha_desde, fecha_hasta, is_periodo, is_refund, is_dadiva)
      
        if type == 'vendedor':
           return self.report_sales_salesmen(period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, formato, is_refund, journal_access, is_dadiva)
        
        if type == 'cliente':
           return self.report_sales_client(period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, partner_id, is_refund, journal_access, is_dadiva)
        
        if type == 'product':
           return self.report_sale_product(period_id, journal_id, fecha_desde, fecha_hasta, all_product, product_id, is_periodo, formato, is_refund, journal_access, is_dadiva)
            
    
    
    def report_sales_client(self, period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, partner_id, is_refund, journal_access, is_dadiva):
        """
        Funcion que genera el reporte general por clientes, esta funcion
        ejecuta un query que es modelado dependiendo de las opciones
        seleccionadas
        @param period_id: variable que posee el id del periodo seleccionado 
        @param journal_id: variable que posee el id del diario seleccionado 
        @param user_id: variable que posee el id del usuario vendedor seleccionado 
        @param fecha_desde: variable de tipo date que posee la fecha seleccionado 
        @param fecha_hasta: variable de tipo date que posee la fecha seleccionado 
        @param all_sale: variable de tipo boleana que indica al query si 
        es necesario optener tosos los vendedores en la consulta
        @param is_periodo: variable de tipo boleana que indica al query si 
        va a consultar por rango de fechas o periodos
        @return {'cliente':{id_factura:{
                'vendedor':i[16], 
                'diario':i[17], 
                'cliente':i[1], 
                'n_fact':i[3],
                'sub_to':i[18], 
                'imp':i[19], 
                'total':i[20] }}} 
        """
        
        sql = """
            SELECT invoice.id, partner.ref|| ' ' ||partner.name as partner_name, invoice.reference, invoice.number, invoice.date_document,
            invoice.date_invoice, invoice_parent.number as number_origen, invoice_line.name, account.code,
            CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2)
            ELSE invoice_line.quantity
            END as cant_convertida,
            CASE WHEN t_unidad_deter.name_puom is null THEN
            unidad_deter.name ELSE t_unidad_deter.name_puom
            END as name_unidad,
            SUM(round(tax.amount * price_subtotal,2)) as tax,
            invoice_line.price_unit, invoice_line.discount,
            invoice_line.price_subtotal,
            CASE WHEN category.name is null THEN
            'N/A'
            ELSE category.name
            END as category, users.name, journal.name,
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice.amount_untaxed
            ELSE invoice.amount_untaxed* -1 END
            as subtotal,
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice.amount_tax
            ELSE invoice.amount_tax* -1 END
            as tax,
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice.amount_total
            ELSE invoice.amount_total* -1 END
            as total      
            FROM account_invoice as invoice
            INNER JOIN account_journal as journal ON (invoice.journal_id = journal.id)
            INNER JOIN res_partner as partner ON (invoice.partner_id = partner.id)
            INNER JOIN account_invoice_line as invoice_line ON (invoice_line.invoice_id = invoice.id)
            INNER JOIN account_invoice_line_tax as line_tax ON (invoice_line.id = line_tax.invoice_line_id)
            INNER JOIN account_tax as tax ON (line_tax.tax_id = tax.id)
            INNER JOIN account_account as account ON (invoice_line.account_id = account.id)
            LEFT JOIN account_invoice as invoice_parent ON (invoice.parent_id = invoice_parent.id)
            LEFT JOIN res_users as users ON (invoice.user_id=users.id)
            LEFT JOIN product_uom as unidad_venta ON (invoice_line.uos_id = unidad_venta.id)
            LEFT JOIN product_product as product ON (invoice_line.product_id = product.id)
            LEFT JOIN product_template as templatep ON (product.product_tmpl_id = templatep.id)
            LEFT JOIN product_uom as unidad_deter ON (templatep.uom_id = unidad_deter.id)
            LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
            FROM ir_translation it
            INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
            WHERE it.name = 'product.uom,name' ) as t_unidad_deter ON unidad_deter.id = t_unidad_deter.p_uom_id
            LEFT JOIN product_category as category ON (templatep.categ_id = category.id)
            LEFT JOIN sale_order_invoice_rel as sale_rel ON (invoice.id=sale_rel.invoice_id)
            LEFT JOIN sale_order as sale ON (sale_rel.order_id=sale.id)
            WHERE invoice.state IN ('open', 'paid') 
            and (invoice.nro_ctrl != 'saldo elite' or invoice.nro_ctrl is null)
            """
        
        journal_access_aux = journal_access
        journal_access = str(self.list_2_typle(journal_access))
        if is_refund: 
           sql = sql + "\n and invoice.type IN ('out_invoice', 'out_refund')"
           if journal_id:
               sql = sql +  '\nand invoice.journal_id IN (%s,(SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id = %s))  '%(journal_id, journal_id)
           else:
                sql = sql + '\nand invoice.journal_id = ANY((SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id IN %s) UNION SELECT unnest(array%s))'%(journal_access,journal_access_aux)
        else:
            sql = sql + "\nand invoice.type = 'out_invoice'"
            if journal_id:
               sql = sql + '\nand invoice.journal_id = %s'%journal_id
            else:
                sql = sql + '\nand invoice.journal_id IN %s'%journal_access
		
        
        if partner_id:
           sql = sql +  '\nand invoice.partner_id = %s'%partner_id
        
        if is_periodo:
           sql = sql + '\nand invoice.period_id = %s'%period_id
        else:
            sql = sql + "\nand invoice.date_invoice >= '%s' and invoice.date_invoice <= '%s'"%(fecha_desde, fecha_hasta)
        
      
        if not all_sale:
            if not user_id :
               sql = sql + '\nand invoice.user_id is null'
            
            if user_id:
               sql = sql + '\nand invoice.user_id = %s' %user_id
        else:
            pass
            
        if not is_dadiva:
               sql = sql + '\nand (sale.esdadiva is false or sale.esdadiva is null)'
               
        sql = sql +  "\nGROUP BY 1,partner.name,partner.ref,3,4,5,6,7,8,9,10,11,13,14,15,16,17,18,19,20,21  \nORDER BY partner.name, 1, 5;"
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        salida = []
        agrupador2 = {}
        self.sub_total = 0
        self.impuesto = 0
        
        for i in consulta:
            agrupador2[i[1]] = {}
        for i in consulta: 
            agrupador2[i[1]].update({i[0]:{}})
        for i in consulta:
           agrupador2[i[1]][i[0]].update({'vendedor':i[16], 
           'diario':i[17], 
           'cliente':i[1], 
           'n_fact':i[3],
           'sub_to':i[18], 
           'imp':i[19], 
           'total':i[20] })
        
        for i in agrupador2:
            for f in agrupador2[i]:
                self.sub_total += agrupador2[i][f]['sub_to']
                self.impuesto += agrupador2[i][f]['imp']
        self.total =   self.sub_total+self.impuesto
        return [agrupador2]
    
    def report_sale_product(self, period_id, journal_id, fecha_desde, fecha_hasta, all_product, product_id, is_periodo, formato, is_refund, journal_access, is_dadiva):
        """
        Funcion que genera el reporte detallado por productos, esta funcion
        ejecuta un query que es modelado dependiendo de las opciones
        seleccionadas
        @param period_id: variable que posee el id del periodo seleccionado 
        @param journal_id: variable que posee el id del diario seleccionado 
        @param product_id: variable que posee el id del producto seleccionado 
        @param fecha_desde: variable de tipo date que posee la fecha seleccionado 
        @param fecha_hasta: variable de tipo date que posee la fecha seleccionado 
        @param all_product: variable de tipo boleana que indica al query si 
        es necesario optener tosos los productos en la consulta
        @param is_periodo: variable de tipo boleana que indica al query si 
        va a consultar por rango de fechas o periodos
        @return [{
                'producto': i[0],
                'cantidad': i[1],
                'unidad':   i[2],
                'total_kg': i[3],
                'sub_total':i[4],
                }] : retorna una lista de diccionarios con los que 
                     indican los campos del rml  """ 
        sql = """
            SELECT product.default_code, btrim(invoice_line.name,' '),
            sum(CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            CASE WHEN invoice.type = 'out_invoice' THEN
               round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2)
            ELSE round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2) * -1
            END
            ELSE CASE WHEN invoice.type = 'out_invoice' THEN
                  invoice_line.quantity
            ELSE invoice_line.quantity * -1
            END
            END) as cant_convertida,
            CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            CASE WHEN t_unidad_deter.name_puom is null THEN
            unidad_deter.name
            ELSE t_unidad_deter.name_puom END
            ELSE CASE WHEN t_unidad_venta.name_puom is null THEN
            unidad_venta.name
            ELSE t_unidad_venta.name_puom END
            END as unidad_name,      
            sum(CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            CASE WHEN invoice.type = 'out_invoice' THEN    
            round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor * templatep.weight_net :: numeric ,2)
            ELSE
            round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor * templatep.weight_net :: numeric ,2) * -1
            END
            ELSE
            CASE WHEN invoice_line.uos_id is null THEN
            0
            ELSE
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice_line.quantity * templatep.weight_net
            ELSE
            invoice_line.quantity * templatep.weight_net * -1
            END
            END
            END) as cant_convertida_peso,
            CASE WHEN invoice.type = 'out_invoice' THEN  
            SUM (invoice_line.price_subtotal)
            ELSE SUM (invoice_line.price_subtotal) * -1 END as subtotal,
            CASE WHEN invoice.type = 'out_invoice' THEN
            SUM(round(tax.amount * price_subtotal,2))
            ELSE SUM(round(tax.amount * price_subtotal,2)) * -1 END as tax  
            FROM account_invoice as invoice
            INNER JOIN account_journal as journal ON (invoice.journal_id = journal.id)
            INNER JOIN res_partner as partner ON (invoice.partner_id = partner.id)
            INNER JOIN account_invoice_line as invoice_line ON (invoice_line.invoice_id = invoice.id)
            INNER JOIN account_invoice_line_tax as line_tax ON (invoice_line.id = line_tax.invoice_line_id)
            INNER JOIN account_tax as tax ON (line_tax.tax_id = tax.id)
            LEFT JOIN res_users as users ON (invoice.user_id=users.id)
            LEFT JOIN product_uom as unidad_venta ON (invoice_line.uos_id = unidad_venta.id)
            LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
            FROM ir_translation it
            INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
            WHERE it.name = 'product.uom,name' ) as t_unidad_venta ON unidad_venta.id = t_unidad_venta.p_uom_id
            LEFT JOIN product_product as product ON (invoice_line.product_id = product.id)
            LEFT JOIN product_template as templatep ON (product.product_tmpl_id = templatep.id)
            LEFT JOIN product_uom as unidad_deter ON (templatep.uom_id = unidad_deter.id)
            LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
            FROM ir_translation it
            INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
            WHERE it.name = 'product.uom,name' ) as t_unidad_deter ON unidad_deter.id = t_unidad_deter.p_uom_id 
            LEFT JOIN sale_order_invoice_rel as sale_rel ON (invoice.id=sale_rel.invoice_id)
            LEFT JOIN sale_order as sale ON (sale_rel.order_id=sale.id)   
            WHERE invoice.state IN ('open', 'paid')
            and (invoice.nro_ctrl != 'saldo elite' or invoice.nro_ctrl is null)
            """
        
        journal_access_aux = journal_access
        journal_access = str(self.list_2_typle(journal_access))
        if is_refund: 
           sql = sql + "\n and invoice.type IN ('out_invoice', 'out_refund')"
           if journal_id:
               sql = sql +  '\nand invoice.journal_id IN (%s,(SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id = %s))  '%(journal_id, journal_id)
           else:
                sql = sql + '\nand invoice.journal_id = ANY((SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id IN %s) UNION SELECT unnest(array%s))'%(journal_access,journal_access_aux)
        else:
            sql = sql + "\nand invoice.type = 'out_invoice'"
            if journal_id:
               sql = sql + '\nand invoice.journal_id = %s'%journal_id
            else:
                sql = sql + '\nand invoice.journal_id IN %s'%journal_access
		
        if is_periodo:
           sql = sql + '\nand invoice.period_id = %s'%period_id
        else:
            sql = sql + "\nand invoice.date_invoice >= '%s' and invoice.date_invoice <= '%s'"%(fecha_desde, fecha_hasta)
        
        if not all_product:
            if not product_id :
               sql = sql + '\nand invoice_line.product_id is null'
            
            if product_id:
               sql = sql + '\nand invoice_line.product_id = %s' %product_id
        else:
            pass 
            
        if not is_dadiva:
               sql = sql + '\nand (sale.esdadiva is false or sale.esdadiva is null)' 
                      
        sql = sql + '\nGROUP BY 1,2,4,invoice.type  \nORDER BY 2,1;'
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        agrupador = []
        for i in consulta:
            self.impuesto += i[6]
            self.sub_total += i[5]
            agrupador.append({
            'codigo':i[0],
            'producto':i[1],
            'cantidad':i[2],
            'unidad':i[3],
            'total_kg':0.0 if not i[4] else i[4],
            'sub_total':0.0 if not i[5] else i[5],
            'impuesto':0.0 if not i[6] else i[6],
            'total':i[5]+i[6],
            })
        self.total = self.impuesto + self.sub_total
        return agrupador
        
    
    def report_sale_diario(self, period_id, journal_id, fecha_desde, fecha_hasta, is_periodo, is_refund, is_dadiva):
        """
        Funcion que genera el reporte detallado de facturas por diario, 
        esta funcion ejecuta un query que es modelado dependiendo de las opciones
        seleccionadas
        @param period_id: variable que posee el id del periodo seleccionado 
        @param journal_id: variable que posee el id del diario seleccionado 
        @param fecha_desde: variable de tipo date que posee la fecha seleccionado 
        @param fecha_hasta: variable de tipo date que posee la fecha seleccionado 
        @param is_periodo: variable de tipo boleana que indica al query si 
        va a consultar por rango de fechas o periodos
        @return [{id_factura:{
                ('cliente','referencias','nro_documento',
                'fecha_de_documento','fecha_origen','origen'):
                 [('descripcion_producto','cuenta','cantidad',
                 'unidad','impuesto','precio_uni','descuento',                                          
                 'sub_total','categoria')]
                }}] : retorna una lista con un diccionario con los que 
                     indican los campos del rml  
        """
        sql = """
                SELECT invoice.id, partner.name, invoice.reference, invoice.number,
                invoice.date_document, invoice.date_invoice, invoice.origin,
                invoice_line.name, account.code,
                CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
                    CASE WHEN invoice.type = 'out_invoice' THEN
                       round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2)
                ELSE round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2) * -1
                END
                ELSE CASE WHEN invoice.type = 'out_invoice' THEN
                          invoice_line.quantity
                  ELSE invoice_line.quantity * -1
                END
                END as cant_convertida,
                CASE WHEN t_unidad_deter.name_puom is null THEN
                unidad_deter.name ELSE t_unidad_deter.name_puom
                END as name_unidad,
                CASE WHEN invoice.type = 'out_invoice' THEN
                SUM(round(tax.amount * price_subtotal,2))
                ELSE SUM(round(tax.amount * price_subtotal,2))*-1 END as tax,
                invoice_line.price_unit, invoice_line.discount,
                CASE WHEN invoice.type = 'out_invoice' THEN   
                SUM(invoice_line.price_subtotal)
                ELSE SUM(invoice_line.price_subtotal)* -1 END
                as price_subtotal,
                CASE WHEN category.name is null THEN
                'N/A' ELSE category.name
                END as category   
                FROM account_invoice as invoice
                INNER JOIN res_partner as partner ON (invoice.partner_id = partner.id)
                INNER JOIN account_invoice_line as invoice_line ON (invoice_line.invoice_id = invoice.id)
                INNER JOIN account_invoice_line_tax as line_tax ON (invoice_line.id = line_tax.invoice_line_id)
                INNER JOIN account_tax as tax ON (line_tax.tax_id = tax.id)
                INNER JOIN account_account as account ON (invoice_line.account_id = account.id)
                LEFT JOIN product_uom as unidad_venta ON (invoice_line.uos_id = unidad_venta.id)
                LEFT JOIN product_product as product ON (invoice_line.product_id = product.id)
                LEFT JOIN product_template as templatep ON (product.product_tmpl_id = templatep.id)
                LEFT JOIN product_uom as unidad_deter ON (templatep.uom_id = unidad_deter.id)
                LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
                FROM ir_translation it
                INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
                WHERE it.name = 'product.uom,name' ) as t_unidad_deter ON unidad_deter.id = t_unidad_deter.p_uom_id
                LEFT JOIN product_category as category ON (templatep.categ_id = category.id)
                LEFT JOIN sale_order_invoice_rel as sale_rel ON (invoice.id=sale_rel.invoice_id)
                LEFT JOIN sale_order as sale ON (sale_rel.order_id=sale.id)
                WHERE invoice.state IN ('open', 'paid')
                and (invoice.nro_ctrl != 'saldo elite' or invoice.nro_ctrl is null)
            """
        
        if is_refund:
		   sql = sql + "\n and invoice.type IN ('out_invoice', 'out_refund')"
		   sql = sql + "\n and invoice.journal_id IN (%s,(SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id = %s))"%(journal_id,journal_id)
        else:
			sql = sql + "\n and invoice.type = 'out_invoice' "   
			sql = sql +  '\nand invoice.journal_id = %s'%journal_id  
        
        if is_periodo:
           sql = sql + '\n and invoice.period_id = %s'%period_id
        else:
            sql = sql + "\nand invoice.date_invoice >= '%s' and invoice.date_invoice <= '%s'"%(fecha_desde, fecha_hasta)
        
        if not is_dadiva:
               sql = sql + '\nand (sale.esdadiva is false or sale.esdadiva is null)'       
        
        sql = sql + "\nGROUP BY 1,2,3,4,5,6,7,8,9,10,11,13,14,16\n ORDER BY 1, 5;" 
        agrupador = {}
        aux = {}
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        
        for i in consulta:
            agrupador[i[0]] = []
            aux[i[0]] = []

        for i in consulta:
            agrupador[i[0]].append(i[1:])

        for i in agrupador:
            for l in agrupador[i]:
                aux[i] = {l[0:6]:[]}
        
        for i in agrupador:
            for l in agrupador[i]:
                self.sub_total += l[13]
                self.impuesto += l[10]
                aux[i][l[0:6]].append(l[6:])
               
                
        self.total =  self.sub_total + self.impuesto
        return [aux]
    
    
    def report_sales_salesmen(self, period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, formato, is_refund, journal_access, is_dadiva):
        """
        Funcion que genera el reporte general por clientes, esta funcion
        ejecuta un query que es modelado dependiendo de las opciones
        seleccionadas
        @param period_id: variable que posee el id del periodo seleccionado 
        @param journal_id: variable que posee el id del diario seleccionado 
        @param user_id: variable que posee el id del usuario vendedor seleccionado 
        @param fecha_desde: variable de tipo date que posee la fecha seleccionado 
        @param fecha_hasta: variable de tipo date que posee la fecha seleccionado 
        @param formato: variable de tipo string que indica el tipo de funcion ha
        ejecutar para generar el reporte 
        @param all_sale: variable de tipo boleana que indica al query si 
        es necesario optener tosos los vendedores en la consulta
        @param is_periodo: variable de tipo boleana que indica al query si 
        va a consultar por rango de fechas o periodos
        @return funcion(): depende que valor tenga la variable formato retornara
        una funcion """
        sql = """
            SELECT invoice.id, partner.name, invoice.reference, invoice.number, invoice.date_document,
            invoice.date_invoice, invoice.origin, invoice_line.name, account.code,
            CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            CASE WHEN invoice.type = 'out_invoice' THEN
            round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2)
            ELSE round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2) * -1
            END
            ELSE CASE WHEN invoice.type = 'out_invoice' THEN
              invoice_line.quantity
            ELSE invoice_line.quantity * -1
            END
            END as cant_convertida,
            CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            CASE WHEN t_unidad_deter.name_puom is null THEN
            unidad_deter.name
            ELSE t_unidad_deter.name_puom END
            ELSE CASE WHEN t_unidad_deter.name_puom is null THEN
            unidad_venta.name
            ELSE t_unidad_deter.name_puom END
            END as unidad_name,
            CASE WHEN invoice.type = 'out_invoice' THEN      
            SUM(round(tax.amount * price_subtotal,2))
            ELSE SUM(round(tax.amount * price_subtotal,2)) * -1 END as tax,
            invoice_line.price_unit, invoice_line.discount,
            CASE WHEN invoice.type = 'out_invoice' THEN  
            SUM(invoice_line.price_subtotal)
            ELSE SUM(invoice_line.price_subtotal) * -1 END as price_subtotal,
            CASE WHEN category.name is null THEN
            'N/A'
            ELSE category.name
            END as category, users.name,
            CASE WHEN tt_sale.name_tsale is null THEN
            t_sale.name ELSE tt_sale.name_tsale
            END as name_team, journal.name, invoice.amount_untaxed, invoice.amount_tax, invoice.amount_total  
            FROM account_invoice as invoice
            INNER JOIN account_journal as journal ON (invoice.journal_id = journal.id)
            INNER JOIN res_partner as partner ON (invoice.partner_id = partner.id)
            INNER JOIN account_invoice_line as invoice_line ON (invoice_line.invoice_id = invoice.id)
            INNER JOIN account_invoice_line_tax as line_tax ON (invoice_line.id = line_tax.invoice_line_id)
            INNER JOIN account_tax as tax ON (line_tax.tax_id = tax.id)
            INNER JOIN account_account as account ON (invoice_line.account_id = account.id)
            LEFT JOIN res_users as users ON (invoice.user_id=users.id)
            LEFT JOIN crm_case_section as t_sale ON (users.context_section_id = t_sale.id)
            LEFT JOIN (SELECT t_sale_t.id as t_sale_id, it.value as name_tsale
            FROM ir_translation it
            INNER JOIN crm_case_section as t_sale_t ON t_sale_t.id = it.res_id
            WHERE it.name = 'crm.case.section,name' ) as tt_sale ON t_sale.id = tt_sale.t_sale_id
            LEFT JOIN product_uom as unidad_venta ON (invoice_line.uos_id = unidad_venta.id)
            LEFT JOIN product_product as product ON (invoice_line.product_id = product.id)
            LEFT JOIN product_template as templatep ON (product.product_tmpl_id = templatep.id)
            LEFT JOIN product_uom as unidad_deter ON (templatep.uom_id = unidad_deter.id)
            LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
            FROM ir_translation it
            INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
            WHERE it.name = 'product.uom,name' ) as t_unidad_deter ON unidad_deter.id = t_unidad_deter.p_uom_id
            LEFT JOIN product_category as category ON (templatep.categ_id = category.id)
            LEFT JOIN sale_order_invoice_rel as sale_rel ON (invoice.id=sale_rel.invoice_id)
            LEFT JOIN sale_order as sale ON (sale_rel.order_id=sale.id)
            WHERE invoice.state IN ('open', 'paid') 
            and (invoice.nro_ctrl != 'saldo elite' or invoice.nro_ctrl is null)
                """
        journal_access_aux = journal_access
        journal_access = str(self.list_2_typle(journal_access))
        if is_refund: 
           sql = sql + "\n and invoice.type IN ('out_invoice', 'out_refund')"
           if journal_id:
               sql = sql +  '\nand invoice.journal_id IN (%s,(SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id = %s))  '%(journal_id, journal_id)
           else:
                sql = sql + '\nand invoice.journal_id = ANY((SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id IN %s) UNION SELECT unnest(array%s))'%(journal_access,journal_access_aux)
        else:
            sql = sql + "\nand invoice.type = 'out_invoice'"
            if journal_id:
               sql = sql + '\nand invoice.journal_id = %s'%journal_id
            else:
                sql = sql + '\nand invoice.journal_id IN %s'%journal_access
         
        if is_periodo:
           sql = sql + '\nand invoice.period_id = %s'%period_id
        else:
            sql = sql + "\nand invoice.date_invoice >= '%s' and invoice.date_invoice <= '%s'"%(fecha_desde, fecha_hasta)
        
      
        if not all_sale:
            if not user_id :
               sql = sql + '\nand invoice.user_id is null'
            
            if user_id:
               sql = sql + '\nand invoice.user_id = %s' %user_id
        else:
            pass 
        
        if not is_dadiva:
               sql = sql + '\nand (sale.esdadiva is false or sale.esdadiva is null)'
        
        sql = sql +  "\nGROUP BY 1,2,3,4,5,6,7,8,9,10,11,13,14,16,17,18,19,20,21,22   \nORDER BY 17, 1, 5;"
        
        if formato=='resume_product':
           return self.report_resum_sale_product(period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, formato, is_refund, journal_access_aux, is_dadiva)
        
        if formato == 'resume_client':
           return self.report_resum_sale_client(period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, formato, is_refund, journal_access_aux, is_dadiva)     
        
        if formato=='detail':
            agrupador = {}
            aux = {}
            self.cr.execute(sql)
            consulta = self.cr.fetchall()
            for i in consulta:
                self.equipo_ventas = i[18]
                agrupador[i[16]] = {}
            for i in consulta:
                agrupador[i[16]][i[0]] = {i[1:7]:[]}
            for i in consulta:
                self.sub_total += i[14]
                self.impuesto += i[11]
                agrupador[i[16]][i[0]][i[1:7]].append(i[7:17])

            self.total =  self.sub_total + self.impuesto
            return [agrupador]
        
        return None

        
    def report_resum_sale_product(self, period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, formato, is_refund, journal_access, is_dadiva):
        sql = """
            SELECT users.name, journal.name, btrim(invoice_line.name,' '),
            SUM(CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            CASE WHEN invoice.type = 'out_invoice' THEN
            round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2)
            ELSE round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor :: numeric ,2) * -1
            END
            ELSE CASE WHEN invoice.type = 'out_invoice' THEN
              invoice_line.quantity
            ELSE invoice_line.quantity * -1
            END
            END) as cant_convertida,
            CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            unidad_deter.name
            ELSE unidad_venta.name
            END as unidad_name,      
            SUM(CASE WHEN invoice_line.uos_id != unidad_deter.id THEN
            CASE WHEN invoice.type = 'out_invoice' THEN    
            round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor * templatep.weight_net :: numeric ,2)
            ELSE
            round((invoice_line.quantity :: numeric/unidad_venta.factor :: numeric) * unidad_deter.factor * templatep.weight_net :: numeric ,2) * -1
            END
            ELSE
            CASE WHEN invoice_line.uos_id is null THEN
            0
            ELSE
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice_line.quantity * templatep.weight_net
            ELSE
            invoice_line.quantity * templatep.weight_net * -1
            END
            END
            END) as cant_convertida_peso,
            CASE WHEN invoice.type = 'out_invoice' THEN  
            SUM (invoice_line.price_subtotal)
            ELSE SUM (invoice_line.price_subtotal) * -1 END as subtotal,
            CASE WHEN invoice.type = 'out_invoice' THEN
            SUM(round(tax.amount * price_subtotal,2))
            ELSE SUM(round(tax.amount * price_subtotal,2)) * -1 END as tax,      
            CASE WHEN tt_sale.name_tsale is null THEN
            t_sale.name ELSE tt_sale.name_tsale
            END as name_team
            FROM account_invoice as invoice
            INNER JOIN account_journal as journal ON (invoice.journal_id = journal.id)
            INNER JOIN res_partner as partner ON (invoice.partner_id = partner.id)
            INNER JOIN account_invoice_line as invoice_line ON (invoice_line.invoice_id = invoice.id)
            INNER JOIN account_invoice_line_tax as line_tax ON (invoice_line.id = line_tax.invoice_line_id)
            INNER JOIN account_tax as tax ON (line_tax.tax_id = tax.id)
            LEFT JOIN res_users as users ON (invoice.user_id=users.id)
            LEFT JOIN crm_case_section as t_sale ON (users.context_section_id = t_sale.id)
            LEFT JOIN (SELECT t_sale_t.id as t_sale_id, it.value as name_tsale
            FROM ir_translation it
            INNER JOIN crm_case_section as t_sale_t ON t_sale_t.id = it.res_id
            WHERE it.name = 'crm.case.section,name' ) as tt_sale ON t_sale.id = tt_sale.t_sale_id
            LEFT JOIN product_uom as unidad_venta ON (invoice_line.uos_id = unidad_venta.id)
            LEFT JOIN product_product as product ON (invoice_line.product_id = product.id)
            LEFT JOIN product_template as templatep ON (product.product_tmpl_id = templatep.id)
            LEFT JOIN product_uom as unidad_deter ON (templatep.uom_id = unidad_deter.id)
            LEFT JOIN sale_order_invoice_rel as sale_rel ON (invoice.id=sale_rel.invoice_id)
            LEFT JOIN sale_order as sale ON (sale_rel.order_id=sale.id)
            WHERE invoice.state IN ('open', 'paid')
            and (invoice.nro_ctrl != 'saldo elite' or invoice.nro_ctrl is null)
            """
        journal_access_aux = journal_access
        journal_access = str(self.list_2_typle(journal_access))
        if is_refund: 
           sql = sql + "\n and invoice.type IN ('out_invoice', 'out_refund')"
           if journal_id:
               sql = sql +  '\nand invoice.journal_id IN (%s,(SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id = %s))  '%(journal_id, journal_id)
           else:
                sql = sql + '\nand invoice.journal_id = ANY((SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id IN %s) UNION SELECT unnest(array%s))'%(journal_access,journal_access_aux)
        else:
            sql = sql + "\nand invoice.type = 'out_invoice'"
            if journal_id:
               sql = sql + '\nand invoice.journal_id = %s'%journal_id
            else:
                sql = sql + '\nand invoice.journal_id IN %s'%journal_access
        
        if is_periodo:
           sql = sql + '\nand invoice.period_id = %s'%period_id
        else:
            sql = sql + "\nand invoice.date_invoice >= '%s' and invoice.date_invoice <= '%s'"%(fecha_desde, fecha_hasta)
        
        if not all_sale:
            if not user_id :
               sql = sql + '\nand invoice.user_id is null'
            if user_id:
               sql = sql + '\nand invoice.user_id = %s' %user_id
        else:
            pass 
        
        if not is_dadiva:
               sql = sql + '\nand (sale.esdadiva is false or sale.esdadiva is null)'
                       
        sql = sql +  "\nGROUP BY 1, 2, 3, 5, 9, invoice.type  \nORDER BY 1,3;"
        agrupador = {}
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        self.equipo_ventas = {'N/A':''}
        for i in consulta:
            self.equipo_ventas[i[0]]=i[8]
            self.impuesto += i[7]
            self.sub_total += i[6]
            agrupador[i[0]] = []
        for i in consulta:
            agrupador[i[0]].append({
            'diario':i[1],
            'producto':i[2],
            'cantidad':i[3],
            'unidad':i[4],
            'peso':i[5],
            'sub_to':i[6],
            'impuesto':i[7],
            'total':i[6]+i[7],
            })
        self.total = self.impuesto + self.sub_total
        return [agrupador]
        
    def report_resum_sale_client(self, period_id, journal_id, fecha_desde, fecha_hasta, all_sale, user_id, is_periodo, formato, is_refund, journal_access, is_dadiva):
        sql = """
            SELECT users.name, partner.ref|| ' ' ||partner.name as partner_name, journal.name, invoice.number,
            invoice_parent.number as number_origin,
            CASE WHEN tt_sale.name_tsale is null THEN
            t_sale.name ELSE tt_sale.name_tsale
            END as name_team,
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice.amount_untaxed
            ELSE invoice.amount_untaxed* -1 END
            as subtotal,
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice.amount_tax
            ELSE invoice.amount_tax* -1 END
            as tax,
            CASE WHEN invoice.type = 'out_invoice' THEN
            invoice.amount_total
            ELSE invoice.amount_total* -1 END
            as total
            FROM account_invoice as invoice
            INNER JOIN account_journal as journal ON (invoice.journal_id = journal.id)
            INNER JOIN res_partner as partner ON (invoice.partner_id = partner.id)
            LEFT JOIN account_invoice as invoice_parent ON (invoice.parent_id = invoice_parent.id)
            LEFT JOIN res_users as users ON (invoice.user_id=users.id)
            LEFT JOIN crm_case_section as t_sale ON (users.context_section_id = t_sale.id)
            LEFT JOIN (SELECT t_sale_t.id as t_sale_id, it.value as name_tsale
            FROM ir_translation it
            INNER JOIN crm_case_section as t_sale_t ON t_sale_t.id = it.res_id
            WHERE it.name = 'crm.case.section,name' ) as tt_sale ON t_sale.id = tt_sale.t_sale_id
            LEFT JOIN sale_order_invoice_rel as sale_rel ON (invoice.id=sale_rel.invoice_id)
            LEFT JOIN sale_order as sale ON (sale_rel.order_id=sale.id)
            WHERE invoice.state IN ('open', 'paid')
            and (invoice.nro_ctrl != 'saldo elite' or invoice.nro_ctrl is null)
            """
        journal_access_aux = journal_access
        journal_access = str(self.list_2_typle(journal_access))
        if is_refund: 
           sql = sql + "\n and invoice.type IN ('out_invoice', 'out_refund')"
           if journal_id:
               sql = sql +  '\nand invoice.journal_id IN (%s,(SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id = %s))  '%(journal_id, journal_id)
           else:
                sql = sql + '\nand invoice.journal_id = ANY((SELECT refund_journal_id FROM sale_shop as shop WHERE journal_id IN %s) UNION SELECT unnest(array%s))'%(journal_access,journal_access_aux)
        else:
            sql = sql + "\nand invoice.type = 'out_invoice'"
            if journal_id:
               sql = sql + '\nand invoice.journal_id = %s'%journal_id
            else:
                sql = sql + '\nand invoice.journal_id IN %s'%journal_access
        
        
        if is_periodo:
           sql = sql + '\nand invoice.period_id = %s'%period_id
        else:
            sql = sql + "\nand invoice.date_invoice >= '%s' and invoice.date_invoice <= '%s'"%(fecha_desde, fecha_hasta)
        
        if not all_sale:
            if not user_id :
               sql = sql + '\nand invoice.user_id is null'
            if user_id:
               sql = sql + '\nand invoice.user_id = %s' %user_id
        else:
            pass
            
        if not is_dadiva:
               sql = sql + '\nand (sale.esdadiva is false or sale.esdadiva is null)'        
        
        sql = sql +  "\nORDER BY 1, partner.name, 2, 4;"
        agrupador = {}
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        self.equipo_ventas = {'N/A':''}
        for i in consulta:
            self.equipo_ventas[i[0]] = i[5]
            self.impuesto += i[7]
            self.sub_total += i[6]
            agrupador[i[0]] = []
        
        for i in consulta:
            agrupador[i[0]].append({
            'diario':i[2],
            'cliente':i[1],
            'nro_fact':i[3],
            'fact_origen':i[4],
            'base':i[6],
            'impuesto':i[7],
            'total':i[8],
            
            })
          
        self.total = self.sub_total + self.impuesto
        return [agrupador]
    
    def get_total(self):
        return self.total
    
    def get_subtotal(self):
        return self.sub_total
        
    def get_impuesto(self):
        return self.impuesto
    
    def get_equipo_ventas(self, salesmen=None):
        if salesmen:
           return self.equipo_ventas[salesmen]
        return self.equipo_ventas
    
    def titulos_rml(self, data):
        d = {'detail':'VENTAS POR VENDEDOR DETALLADO',
            'resume_client':'VENTAS POR VENDEDOR RESUMIDO POR CLIENTES',
            'resume_product':'VENTAS POR VENDEDOR RESUMIDO POR PRODUCTOS',
            }
        return d[data['form']['formato']]
         
    def sumador(self, k, data):
        tipo = {'subtotal':0.0, 'impuesto':0.0, 'total':0.0}
        for i in data[k]:
            for n in data[k][i]:
                tipo['subtotal'] +=n[7]
                tipo['impuesto'] +=n[4]
                tipo['total'] +=n[4]+n[7]
        return tipo
    
    def sumador_resumido(self, data, param=None):
        tipo = {'subtotal':0.0, 'impuesto':0.0, 'total':0.0}
        for f in data:
            tipo['subtotal'] +=data[f]['sub_to']
            tipo['impuesto'] +=data[f]['imp']
            tipo['total'] +=data[f]['total']
        return tipo
    
    def sumador_resumido_product(self, data, param=False):
        tipo = {'subtotal':0.0, 'impuesto':0.0, 'total':0.0}
        if param:
           tipo[param] = 0.0
        for f in data:
            tipo[param] +=f[param]
        return tipo

    def cvr_fecha(self, f):
        d = datetime.strptime(f, '%Y-%m-%d').strftime('%d/%m/%Y')
        return d 
        
    def list_2_typle(self, list):
            list = [str(i) for i in list]    
            t = ()
            if len(list) == 1:
               t = "(%s)"%"".join(list)
            else:
                t = tuple(list)
            return t    

    def get_fecha(self, value):
        salida = ''
        if value['is_periodo']:
           salida = "Periodo: %s" %value['periodo']
        else:
            salida = 'Desde: %s Hasta: %s'%(self.cvr_fecha(value['fecha_desde']), self.cvr_fecha(value['fecha_hasta']))
        return salida 
    
report_sxw = report_sxw.report_sxw    
class RMLParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """
    def __init__(self, name, table, rml=False, parser=False, header=True, store=False):
        self.parser_instance = False
        self.localcontext={}
        report_sxw.__init__(self, name, table, rml, parser,
            header, store)
       
    
    
    def create_single_pdf(self, cursor, uid, ids, data, report_xml, context=None):
        """generate the PDF"""
        
        if context is None:
            context={}
        
        if report_xml.report_type == 'pdf':
            return super(RMLParser,self).create_single_pdf(cursor, uid, ids, data, report_xml, context=context)


RMLParser('report.analisis.facturas',
          'wizard.analisis.factura', 
          'sisb_invoice_report/report/analisis_factura_report.rml',
          parser=analisis_facturas_report)
