# -*- coding: utf-8 -*-


import time
from report import report_sxw
import datetime
from datetime import datetime


class analisis_inventario_report(report_sxw.rml_parse):
    
    _name = 'analisis.inventario.report'
    
    def __init__(self, cr, uid, name, context=None):
        super(analisis_inventario_report, self).__init__(cr, uid, name, context=context)
        self.localcontext.update({
        'generate_query':self.generate_query,
        'producto_detalle':self.producto_detalle,
        'get_empresa':self.get_empresa,
        'get_fecha':self.get_fecha,
        'titulos_rml':self.titulos_rml,
        'sumar':self.sumar,
        })
        self.context = context
        self.cr = cr
        self.uid = uid
        self.sub_total = 0.0
        self.impuesto = 0.0
        self.total = 0.0
        self.salida_query = []
        
    
    def get_empresa(self, partner_id):
        obj_addr = self.pool.get('res.partner.address')
        addr_id =obj_addr.search(self.cr, self.uid, [('type','=','invoice'),('partner_id','=',partner_id)])
        res = {}
        for row in obj_addr.browse(self.cr, self.uid, addr_id):
            res = {
            'street':row.street,
            'phone':row.phone,
            'fax':row.fax,
            'email':row.email,
            'city':row.city,
            'name':row.name,
            'country':row.country_id.name,
            } 
        return res
    
    def generate_query(self, value):
        value = value['form']
        location = value['location_id']
        is_periodo = value['is_periodo']
        period = value['period_id']
        fecha_desde = value['fecha_desde']
        fecha_hasta = value['fecha_hasta']
        product = value['product_id']
        all_product = value['all_product']
        formato = value['formato']
        existencias_productos = self.get_existencia_ubicacion(location, is_periodo, period, fecha_desde, fecha_hasta, product, all_product)
        self.producto = existencias_productos
        funciones_query = {'detail': self.detallado_productos,
                           'resume':self.resumen_producto,
                           'total':self.detallado_productos}
        salida = funciones_query[formato](location, is_periodo, period, fecha_desde, fecha_hasta, product, all_product, existencias_productos, formato)
        return [salida]
    
    
    
    def filter_sql(self, sql, is_periodo, period, fecha_desde, fecha_hasta, product, all_product):
        if not all_product:
            sql = sql + '\nAND  db_tbl.product_id = %s'%product
        if is_periodo:
            sql = sql + '\nAND db_tbl.date :: date <= (SELECT date_stop FROM account_period WHERE id < %s ORDER BY id desc LIMIT 1)'%period
        else:
            sql = sql + "\nAND db_tbl.date :: date < '%s' "%fecha_desde

        sql = sql + """\n group by db_tbl.product_id, db_tbl.default_code, db_tbl.name
                        \n order by db_tbl.name"""
        return sql 
   
    def filter_resumido_detallado_sql(self, sql, is_periodo, period, fecha_desde, fecha_hasta, product, all_product):
        if not all_product:
           sql = sql + '\nAND  db_tbl.product_id = %s'%product
        if is_periodo:
           sql = sql + """\nand db_tbl.date :: date >= (SELECT date_start FROM account_period WHERE id = %s) 
                            \nand db_tbl.date :: date <= (SELECT date_stop FROM account_period WHERE id = %s)"""%(period,period)
        else:
            sql = sql + "\n AND db_tbl.date :: date >= '%s' and db_tbl.date :: date <= '%s'" %(fecha_desde, fecha_hasta)
        return sql 
        
    def producto_detalle(self, k):
        producto_detail = self.producto[k]
        return producto_detail
    
    def get_existencia_ubicacion(self, location, is_periodo, period, fecha_desde, fecha_hasta, product, all_product):
        sql = """
        SELECT product_id, default_code, name, SUM(cant_convertida) as existencia_inicial from (
        SELECT
        min(m.id) AS id, m.date,
        m.location_id,
        m.product_id,
        pp.default_code,
        pt.name,
        m.state,
        CASE WHEN m.product_uom != pt.uom_id THEN
        COALESCE(sum(round((- m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
        ELSE COALESCE(sum(- m.product_qty), 0.0)
        END as cant_convertida
        FROM
        stock_move m
        LEFT JOIN stock_picking p ON m.picking_id = p.id
        LEFT JOIN product_product pp ON m.product_id = pp.id
        LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
        LEFT JOIN product_uom pu ON pt.uom_id = pu.id
        LEFT JOIN product_uom u ON m.product_uom = u.id
        LEFT JOIN stock_location l ON m.location_id = l.id
        GROUP BY
        m.id, m.product_id, pp.default_code, m.product_uom, pt.name, pt.categ_id, m.address_id, m.location_id, m.location_dest_id,
        m.prodlot_id, m.date, m.state, l.usage, m.company_id, pt.uom_id
        UNION ALL
        SELECT
        - m.id AS id, m.date,
        m.location_dest_id AS location_id,
        m.product_id,
        pp.default_code,
        pt.name,
        m.state,
        CASE WHEN m.product_uom != pt.uom_id THEN
        COALESCE(sum(round((m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
        ELSE COALESCE(sum(m.product_qty), 0.0)
        END as cant_convertida
        FROM
        stock_move m
        LEFT JOIN stock_picking p ON m.picking_id = p.id
        LEFT JOIN product_product pp ON m.product_id = pp.id
            LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
            LEFT JOIN product_uom pu ON pt.uom_id = pu.id
        LEFT JOIN product_uom u ON m.product_uom = u.id
        LEFT JOIN stock_location l ON m.location_dest_id = l.id
        GROUP BY
        m.id, m.product_id, pp.default_code, m.product_uom, pt.name, pt.categ_id, m.address_id, m.location_id, m.location_dest_id,
        m.prodlot_id, m.date, m.state, l.usage, m.company_id, pt.uom_id
        ) db_tbl
        where db_tbl.location_id = %s
        and db_tbl.state = 'done'
        """ %(location)
        
        sql = self.filter_sql(sql, is_periodo, period, fecha_desde, fecha_hasta, product, all_product)
        salida = self.execute_sql(sql)
        agrupar = {}
        for i in salida:
            agrupar[i[0]] = {'codigo':i[1], 'name':i[2], 'existencia':i[3]}
        
        self.producto = agrupar
        return agrupar 
    
    def detallado_productos(self, location, is_periodo, period, fecha_desde, fecha_hasta, product, all_product, existencias, formato):
        sql = """
			select *, CASE WHEN cant_convertida > 0 THEN  cant_convertida ELSE 0 END as entrada , CASE WHEN cant_convertida < 0 THEN  cant_convertida * -1 ELSE 0 END as salida from (
			SELECT
			min(m.id) AS id, m.date,
			m.location_id,
			m.product_id,
			pt.name,
			pp.default_code,
			p.name AS refer,
			CASE WHEN t_unidad_deter.name_puom is null THEN
			pu.name ELSE t_unidad_deter.name_puom
			END as name_unidad,
			m.state,
			CASE WHEN m.product_uom != pt.uom_id THEN
			COALESCE(sum(round((- m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
			ELSE COALESCE(sum(- m.product_qty), 0.0)
			END as cant_convertida,
			'A-->' AS mov,
			l2.name AS ubicacion
			FROM
			stock_move m
			LEFT JOIN stock_picking p ON m.picking_id = p.id
			LEFT JOIN product_product pp ON m.product_id = pp.id
			LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
			LEFT JOIN product_uom pu ON pt.uom_id = pu.id
			LEFT JOIN product_uom u ON m.product_uom = u.id
			LEFT JOIN stock_location l ON m.location_id = l.id
			LEFT JOIN stock_location l2 ON m.location_dest_id = l2.id
			LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
			FROM ir_translation it
			INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
			WHERE it.name = 'product.uom,name' ) as t_unidad_deter ON pu.id = t_unidad_deter.p_uom_id
			GROUP BY
			m.id, m.product_id, pt.name, p.name, pp.default_code, pu.name, t_unidad_deter.name_puom, m.product_uom, m.location_id, l2.name, m.location_dest_id,
			m.date, m.state, pt.uom_id
			UNION ALL
			SELECT
			m.id AS id, m.date,
			m.location_dest_id AS location_id,
			m.product_id,
			pt.name,
			pp.default_code,
			p.name AS refer,
			CASE WHEN t_unidad_deter.name_puom is null THEN
			pu.name ELSE t_unidad_deter.name_puom
			END as name_unidad,
			m.state,
			CASE WHEN m.product_uom != pt.uom_id THEN
			COALESCE(sum(round((m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
			ELSE COALESCE(sum(m.product_qty), 0.0)
			END as cant_convertida,
			'DE->' AS mov,
			l2.name AS ubicacion
			FROM
			stock_move m
			LEFT JOIN stock_picking p ON m.picking_id = p.id
			LEFT JOIN product_product pp ON m.product_id = pp.id
				LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
				LEFT JOIN product_uom pu ON pt.uom_id = pu.id
			LEFT JOIN product_uom u ON m.product_uom = u.id
			LEFT JOIN stock_location l ON m.location_dest_id = l.id
			LEFT JOIN stock_location l2 ON m.location_id = l2.id
			LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
			FROM ir_translation it
			INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
			WHERE it.name = 'product.uom,name') as t_unidad_deter ON pu.id = t_unidad_deter.p_uom_id             
			GROUP BY
			m.id, m.product_id, pt.name, p.name, pp.default_code, pu.name, t_unidad_deter.name_puom, m.product_uom,  m.location_id, l2.name, m.location_dest_id,
			m.date, m.state, pt.uom_id
			) db_tbl
			where db_tbl.location_id = %s
			and db_tbl.state = 'done' 
        """%(location)
        sql = self.filter_resumido_detallado_sql(sql, is_periodo, period, fecha_desde, fecha_hasta, product, all_product)
        sql = sql + '\n order by db_tbl.product_id, db_tbl.date' 
        salida = self.execute_sql(sql)
        agrupador = {}
        calculo = {}
        states = {'done':'Realizado'}
        
        for i in salida:
            if i[3] not in existencias:
               existencias.update({i[3]:{'name':i[4], 'codigo':i[5], 'existencia':0.00}})
        
        self.producto = existencias
        for i in salida:
            agrupador[i[3]] = []
            calculo[i[3]] =  existencias[i[3]]['existencia'] if i[3] in existencias else 0.00
       
        
        for i in salida:
            calculo[i[3]] +=i[12]- i[13]
            agrupador[i[3]].append({'fecha':i[1][0:10], 'entrada':i[12], 'ref':i[6],
                                    'udm':i[7], 'ubicacion': i[10]+' '+i[11], 'salida':i[13], 
                                    'existencia':calculo[i[3]]})
            if formato == 'total':
               existencias[i[3]]['existencia'] = calculo[i[3]]
        
        for i in existencias:
            if i not in agrupador:
               if existencias[i]['existencia'] != 0.00:
                   agrupador.update({i:[{'fecha':None, 'entrada':0.0, 'ref':None,
                                        'udm':None, 'ubicacion':None, 'salida':0.0, 
                                        'existencia':existencias[i]['existencia'],}]})
       
        return agrupador

    def resumen_producto(self, location, is_periodo, period, fecha_desde, fecha_hasta, product, all_product, existencias, formato):
       sql = """
            select db_tbl.location_id,  db_tbl.product_id, db_tbl.name, db_tbl.default_code, db_tbl.mov, db_tbl.ubicacion,  CASE WHEN SUM(db_tbl.cant_convertida) > 0 THEN  SUM(db_tbl.cant_convertida) ELSE 0 END as entrada , CASE WHEN SUM(db_tbl.cant_convertida) < 0 THEN  SUM(db_tbl.cant_convertida) * -1 ELSE 0 END as salida from (
            SELECT
            min(m.id) AS id, m.date,
            m.location_id,
            m.product_id,
            pt.name,
            pp.default_code,
            p.name AS refer,
            CASE WHEN t_unidad_deter.name_puom is null THEN
            pu.name ELSE t_unidad_deter.name_puom
            END as name_unidad,
            m.state,
            CASE WHEN m.product_uom != pt.uom_id THEN
            COALESCE(sum(round((- m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
            ELSE COALESCE(sum(- m.product_qty), 0.0)
            END as cant_convertida,
            'A-->' AS mov,
            l2.name AS ubicacion
            FROM
            stock_move m
            LEFT JOIN stock_picking p ON m.picking_id = p.id
            LEFT JOIN product_product pp ON m.product_id = pp.id
            LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
            LEFT JOIN product_uom pu ON pt.uom_id = pu.id
            LEFT JOIN product_uom u ON m.product_uom = u.id
            LEFT JOIN stock_location l ON m.location_id = l.id
            LEFT JOIN stock_location l2 ON m.location_dest_id = l2.id
            LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
            FROM ir_translation it
            INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
            WHERE it.name = 'product.uom,name' ) as t_unidad_deter ON pu.id = t_unidad_deter.p_uom_id
            GROUP BY
            m.id, m.product_id, pt.name, p.name, pp.default_code, pu.name, t_unidad_deter.name_puom, m.product_uom, m.location_id, l2.name, m.location_dest_id,
            m.date, m.state, pt.uom_id
            UNION ALL
            SELECT
            m.id AS id, m.date,
            m.location_dest_id AS location_id,
            m.product_id,
            pt.name,
            pp.default_code,
            p.name AS refer,
            CASE WHEN t_unidad_deter.name_puom is null THEN
            pu.name ELSE t_unidad_deter.name_puom
            END as name_unidad,
            m.state,
            CASE WHEN m.product_uom != pt.uom_id THEN
            COALESCE(sum(round((m.product_qty :: numeric/u.factor :: numeric) * pu.factor :: numeric ,3)), 0.0)
            ELSE COALESCE(sum(m.product_qty), 0.0)
            END as cant_convertida,
            'DE->' AS mov,
            l2.name AS ubicacion
            FROM
            stock_move m
            LEFT JOIN stock_picking p ON m.picking_id = p.id
            LEFT JOIN product_product pp ON m.product_id = pp.id
                LEFT JOIN product_template pt ON pp.product_tmpl_id = pt.id
                LEFT JOIN product_uom pu ON pt.uom_id = pu.id
            LEFT JOIN product_uom u ON m.product_uom = u.id
            LEFT JOIN stock_location l ON m.location_dest_id = l.id
            LEFT JOIN stock_location l2 ON m.location_id = l2.id
            LEFT JOIN (SELECT p_uom.id as p_uom_id, it.value as name_puom
            FROM ir_translation it
            INNER JOIN product_uom p_uom ON p_uom.id = it.res_id
            WHERE it.name = 'product.uom,name') as t_unidad_deter ON pu.id = t_unidad_deter.p_uom_id             
            GROUP BY
            m.id, m.product_id, pt.name, p.name, pp.default_code, pu.name, t_unidad_deter.name_puom, m.product_uom,  m.location_id, l2.name, m.location_dest_id,
            m.date, m.state, pt.uom_id
            ) db_tbl
            where db_tbl.location_id = %s
            and db_tbl.state = 'done'
       """%(location)
       sql = self.filter_resumido_detallado_sql(sql, is_periodo, period, fecha_desde, fecha_hasta, product, all_product) 
       sql = sql + """
                    \nGROUP BY
                    \ndb_tbl.location_id, db_tbl.product_id, db_tbl.name, 
                    \ndb_tbl.default_code, db_tbl.mov, db_tbl.ubicacion
                    \norder by db_tbl.mov desc
                    """
       salida = self.execute_sql(sql)
       agrupador = {}
       calculo = {}
       for i in salida:
            if i[1] not in existencias:
               existencias.update({i[1]:{'name':i[2], 'codigo':i[3], 'existencia':0.00}})
       for i in salida:
           agrupador[i[1]] = []
           calculo[i[1]] =  existencias[i[1]]['existencia'] if i[1] in existencias else 0.00
    
       for i in salida:
            calculo[i[1]] +=i[6]- i[7]
            agrupador[i[1]].append({'ubicacion': i[4]+' '+i[5], 'entrada':i[6], 
                                    'salida':i[7], 'existencia':calculo[i[1]]})
       for i in existencias:
           if i not in agrupador:
              if existencias[i]['existencia'] != 0.00:
                 agrupador.update({i:[{'ubicacion': None, 'entrada':0.00, 
                                    'salida':0.00, 'existencia':existencias[i]['existencia']}]})
       return  agrupador
             
    
    def execute_sql(self, sql):
        self.cr.execute(sql)
        consulta = self.cr.fetchall()
        return consulta
		
    def cvr_fecha(self, f):
        d = datetime.strptime(f, '%Y-%m-%d').strftime('%d/%m/%Y')
        return d 
        
    def list_2_typle(self, list):
            list = [str(i) for i in list]    
            t = ()
            if len(list) == 1:
               t = "(%s)"%"".join(list)
            else:
                t = tuple(list)
            return t   
    
    def get_final_date(self):
        return datetime.now().strftime('%d/%m/%Y %H:%M')    
    
    def cvr_date(self, fecha):
        d = datetime.datetime.strptime(fecha, '%Y-%m-%d').strftime('%Y/%m/%d')
        return d 
    
    def titulos_rml(self, data):
        d = {'detail':'DETALLADO',
            'resume':'RESUMIDO',
            'total':'TOTAL'
            }
        return d[data['form']['formato']]

    def get_fecha(self, value):
        salida = ''
        if value['is_periodo']:
           salida = "Periodo: %s" %value['periodo_name']
        else:
            salida = 'Desde: %s Hasta: %s'%(self.cvr_fecha(value['fecha_desde']), self.cvr_fecha(value['fecha_hasta']))
        return salida
    
    def sumar(self, k, d):
        calculo = {'salida':0.00, 'entrada':0.00}
        for i in d[k]:
            calculo['salida'] += i['salida']
            calculo['entrada'] += i['entrada']
        return calculo 
        

report_sxw = report_sxw.report_sxw    
class RMLParser(report_sxw):
    """Custom class that use webkit to render HTML reports
       Code partially taken from report openoffice. Thanks guys :)
    """
    def __init__(self, name, table, rml=False, parser=False, header=True, store=False):
        self.parser_instance = False
        self.localcontext={}
        report_sxw.__init__(self, name, table, rml, parser,
            header, store)
       
    
    
    def create_single_pdf(self, cursor, uid, ids, data, report_xml, context=None):
        """generate the PDF"""
        
        if context is None:
            context={}
        
        if report_xml.report_type == 'pdf':
            return super(RMLParser,self).create_single_pdf(cursor, uid, ids, data, report_xml, context=context)


RMLParser('report.analisis.inventario',
          'wizard.analisis.inventario', 
          'sisb_invoice_report/report/analisis_inventario.rml',
          parser=analisis_inventario_report)
