# /usr/bin/env python
# -*- encoding: utf-8 -*-

from osv import osv
from osv import fields
import datetime
from tools.translate import _

class wizard_analisis_factura(osv.osv_memory):
    
    _name = "wizard.analisis.factura"
    _description = "wizard para generar reporte de analisis de facturas"
    
    def lista_journal_permitidos(self, cr, uid):
            
        cr.execute("""select distinct aj.id from stock_location slp
            left outer join stock_location slh on slh.parent_left between slp.parent_left and slp.parent_right
            inner join diana_groups_location_rel dlr on dlr.location_id =slp.id
            inner join stock_warehouse sw on sw.lot_stock_id = slh.id
            inner join sale_shop ss on ss.warehouse_id = sw.id
            inner join account_journal aj on aj.sale_shop_id = ss.id 
            where dlr.groups_id in (SELECT gid FROM res_groups_users_rel where uid = %s)""", (uid,))
        ids = cr.fetchall()
        if ids:
            ids = [id[0] for id in ids]
        return ids
    
    
    def _select_types(self, cr, uid, context=None):
        if context is None:
            context = {}
        res = [('', '')]
        type_inv = context.get('type', 'out_invoice')
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        company_id = context.get('company_id', user.company_id.id)
        type2journal = {'out_invoice': 'sale', 'in_invoice': 'purchase', 'out_refund': 'sale_refund', 'in_refund': 'purchase_refund'}
        refund_journal = {'out_invoice': False, 'in_invoice': False, 'out_refund': True, 'in_refund': True}
        journal_obj = self.pool.get('account.journal')
        param_search =  [('type', '=', type2journal.get(type_inv, 'sale')),
                        ('company_id', '=', company_id),
                        ('refund_journal', '=', refund_journal.get(type_inv, False))]
        if uid  == 1:
           param_search = [('type', '=', 'sale')]
        jour_user = self.lista_journal_permitidos(cr, uid)
        list_res = journal_obj.search(cr, uid, param_search)
        journal_filter = list(set(jour_user) & set(list_res))
        if uid  == 1:
           journal_filter = list_res
        journal_brw = journal_obj.browse(cr, uid, journal_filter)
        for i in journal_brw:
            res.append(( str(i.id), i.name))
        
        if len(res) <= 1:
		   raise osv.except_osv(_('ERROR!'), _('No puede continuar no tiene acceso a ningun diario'))

        return res
    
    
    _columns = {
    'type': fields.selection([('diario','Ventas por Diarios'),
                             ('vendedor', 'Ventas por Vendedor'), 
                             ('cliente', 'Ventas por Cliente'), 
                             ('product', 'Ventas por Productos')], 
                             'Tipo', required=True), 
    'is_periodo': fields.boolean('Periodo'),
    'is_refund': fields.boolean('Nota de credito'),
    'is_dadiva': fields.boolean('Dadiva'),
    'fecha_desde': fields.date('Fecha desde'),
    'fecha_hasta': fields.date('Fecha Hasta'),
    'period_id': fields.many2one('account.period', 'Periodo'),
    'product_id': fields.many2one('product.product', 'Producto'),
    'user_id': fields.many2one('res.users', 'Vendedor'),
    'partner_id': fields.many2one('res.partner', 'Cliente'),
    'all_sale': fields.boolean('Todos los Vendedores'),
    'all_product': fields.boolean('Todos los Productos'),
    'formato':fields.selection([('detail','Detallado'),
        ('resume_client', 'Clientes'), 
        ('resume_product', 'Productos')], 'Resumido por', ),
    'journal_id': fields.selection(_select_types, "Diario", size=100,),
     }
    def _get_journal(self, cr, uid, context=None):

        if context is None:
            context = {}

        type_inv = context.get('type', 'out_invoice')
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        company_id = context.get('company_id', user.company_id.id)
        type2journal = {'out_invoice': 'sale', 'in_invoice': 'purchase', 'out_refund': 'sale_refund', 'in_refund': 'purchase_refund'}
        refund_journal = {'out_invoice': False, 'in_invoice': False, 'out_refund': True, 'in_refund': True}
        journal_obj = self.pool.get('account.journal')
        list_res = journal_obj.search(cr, uid, [('type', '=', type2journal.get(type_inv, 'sale')),
                                            ('company_id', '=', company_id),
                                            ('refund_journal', '=', refund_journal.get(type_inv, False))])
        jour_user = self.lista_journal_permitidos(cr, uid)
        res = list(set(jour_user) & set(list_res))
        res = res and res[0] or False
        if uid == 1:
           res = False 
        return res
        
    _defaults = {'type':'diario',
                'all_product':True,
                'all_sale':True,
                'is_refund':True,
                'journal_id':_get_journal,
                'formato':'resume_client'}
     
    

    def generate_report(self, cr, uid, ids, context=None):
        journal_obj = self.pool.get('account.journal')
        wizard_brw = self.browse(cr, uid, ids)[0]
        period_id = wizard_brw.period_id.id
        periodo = wizard_brw.period_id.name
        journal_id = int(wizard_brw.journal_id)
        journal = None 
        if journal_id:
			journal = journal_obj.browse(cr, uid, journal_id).name 
        is_periodo = wizard_brw.is_periodo
        fecha_desde = wizard_brw.fecha_desde
        fecha_hasta = wizard_brw.fecha_hasta
        type = wizard_brw.type
        all_sale = wizard_brw.all_sale
        all_product = wizard_brw.all_product
        user_id = wizard_brw.user_id.id
        user = wizard_brw.user_id.name 
        product_id = wizard_brw.product_id.id
        partner_id = wizard_brw.partner_id.id
        partner = wizard_brw.partner_id.name 
        producto = wizard_brw.product_id.name
        formato = wizard_brw.formato
        is_refund = wizard_brw.is_refund
        journal_access = [i[0] for i in self._select_types(cr, uid, context) if i[0] != '']
        is_dadiva = wizard_brw.is_dadiva
        
        if not is_periodo:
			fecha_desde_date = datetime.datetime.strptime(fecha_desde, "%Y-%m-%d")
			fecha_hasta_date = datetime.datetime.strptime(fecha_hasta, "%Y-%m-%d")
			dif = fecha_desde_date - fecha_hasta_date
			dif = abs(dif)
		   
			if dif.days > 31:
				raise osv.except_osv('ERROR', "Las fechas no pueden exceder de 31 dias") 
        
        agrupador = {'period_id':period_id, 
                     'periodo':periodo, 
                     'journal_id':journal_id, 
                      'journal':journal, 
                      'journal_access':journal_access, 
                      'type':type, 
                      'all_sale':all_sale, 
                      'all_product':all_product, 
                      'user_id':user_id, 
                      'user':user, 
                      'partner_id':partner_id, 
                      'partner':partner, 
                      'product_id':product_id, 
                      'producto':producto, 
                      'is_periodo':is_periodo,  
                      'is_refund':is_refund, 
                      'formato':formato, 
                      'fecha_desde':fecha_desde,
                      'fecha_hasta':fecha_hasta,
                      'is_dadiva':is_dadiva}
        
        datas = {'form':agrupador}
        dict_ruta = {
        'diario':'sisb_reportes_especiales/report/analisis_factura_report.rml',
        'vendedor':'sisb_reportes_especiales/report/analisis_factura_report_salesmen.rml',
        'cliente':'sisb_reportes_especiales/report/analisis_factura_report_client.rml',
        'product':'sisb_reportes_especiales/report/analisis_factura_report_product.rml',
        }
        
        ir_obj = self.pool.get('ir.actions.report.xml')
        ir_id = ir_obj.search(cr, 1, [('model','=',self._name)]) 
        ir_write= ir_obj.write(cr, 1, ir_id, {'report_rml':dict_ruta[type]})
        
        return {
            'type': 'ir.actions.report.xml',
            'report_name': 'analisis.facturas',
            'datas': datas,
       }
    
   
  
        
wizard_analisis_factura()
