#-*- coding:utf-8 -*-
##############################################################################
#
#    SISB, Sistema Integral Simón Bolivar
#    Copyright (C) 2012-2013 Industrias Diana, C.A. All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
from datetime import date
from datetime import datetime
from datetime import timedelta

class res_company(osv.osv):
    """ Clase encargada de incluir el campo 'company_ssn' 
    (Codigo de Seguro Social Obligatorio) de la compañia"""
    
    _inherit = "res.company"
    
    _columns = {
        'company_ssn': fields.char("Company's Mandatory Social Security Number", size=9)
    }    
    
res_company()

class hr_process(osv.osv):

    _inherit = "hr.process"
    
    def _cambiar_estado_reposos(self, cr, uid, ids, estado, context=None):
        if context is None:
            context = {}
        context.update({'_cambiar_estado_reposos': True})
        
        reposo_medico_obj =  self.pool.get('sisb.reposos.medicos')
        variable_concept = self.pool.get('hr.variable.concepts')
        variable_concept_line =  self.pool.get('hr.variable.concept.line')
        historico_conceptos_variables_reposos = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
        procesos = self.pool.get('hr.process').browse(cr, uid, ids, context=context)
        historico_ids = []
        for proceso in procesos:
            if proceso.type == 'nomina':
                variable_concept_ids = variable_concept.search(cr, uid, [('hr_process_id','=',proceso.id),
                                                                         ('para_reposos_medicos','=',True)],
                                                                         context=None)
                lineas_reposos_ids = variable_concept_line.search(cr, uid, [('variable_concept_id','in',variable_concept_ids),('reposo_medico_ids','!=',False)], context=None)
                lineas_reposos = variable_concept_line.browse(cr, uid, lineas_reposos_ids, context=None)
                reposos_medicos_ids = []
                for linea in lineas_reposos:
                    for reposo in linea.reposo_medico_ids:
                        reposos_medicos_ids.append(reposo.id)
                
                reposos_medicos_ids = list(set(reposos_medicos_ids))
                historico_ids.extend(historico_conceptos_variables_reposos.search(cr, uid, 
                                                                 [('reposo_medico_id','in',reposos_medicos_ids)],
                                                                 context=None))
            if historico_ids:
                historico = historico_conceptos_variables_reposos.browse(cr, uid, historico_ids, context=None)
                reposos_ids = [linea.reposo_medico_id.id for linea in historico
                                                         if linea.dias_reposo_acumulados >= linea.dias_reposo
                                                             or linea.reposo_solapado_completado]
                if reposos_ids:
                    reposo_medico_obj.write(cr, uid, reposos_ids, {'state': estado}, context)
                
        context.pop('_cambiar_estado_reposos')
        return True
    
    def aprobar_proceso(self, cr, uid, ids, context=None):
        """
        Se incluye funcion que pasa de estado 'approved' (aprobado) a estado 'done' (terminado) los reposos
        que hayan sido pagados en su totalidad en la aprobacion de este proceso de nomina.
        """
        self._cambiar_estado_reposos(cr, uid, ids, 'done', context)
        return super(hr_process,self).aprobar_proceso(cr, uid, ids, context)
        
    def devolver_proceso(self, cr, uid, ids, context=None):
        """
        Se incluye funcion que pasa de estado 'done' (terminado) a estado 'approved' (aprobado) los reposos
        que hayan sido pagados en su totalidad en la aprobacion de este proceso de nomina.
        """
        self._cambiar_estado_reposos(cr, uid, ids, 'approved', context)
        return super(hr_process,self).devolver_proceso(cr, uid, ids, context)
        
    def cancel(self, cr, uid, ids, context=None):
        """
        Se sobreescribe el metodo para eliminar el historico de los reposos
        que se hayan cargado en los conceptos variables del proceso de nomina.
        """
        if context is None:
            context = {}
        
        variable_concept_obj = self.pool.get('hr.variable.concepts')
        historico_obj = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
        variable_concept_ids = variable_concept_obj.search(cr, uid, [('hr_process_id','in',ids),
                                                         ('para_reposos_medicos','=',True)],
                                                          context=None)
        if variable_concept_ids:
            context.update({'unlink': True}) #Para eliminar historico
            historico_ids = historico_obj.search(cr, uid, [('variable_concepts_id','in',variable_concept_ids)], context=None)
            historico_obj.unlink(cr, uid, historico_ids, context)
            context.pop('unlink')
        return super(hr_process,self).cancel(cr, uid, ids, context)
        
hr_process()


class sisb_reposos_medicos_centro_salud_ivss(osv.osv):
    
    _name = 'sisb.reposos.medicos.centro.salud.ivss'
    _description = 'Centros de salud del IVSS'
    
    _columns = {
        'name': fields.char('Nombre', size=256, required=True)
        
    }
    
    _sql_constraints = [('unique_name','unique(name)', _('Ya existe un centro de salud con el mismo nombre'))]
    _order = "name asc"
sisb_reposos_medicos_centro_salud_ivss()


class sisb_reposos_medicos_centro_salud_privado(osv.osv):
    
    _name = 'sisb.reposos.medicos.centro.salud.privado'
    _description = 'Centros de salud privados'
    
    _columns = {
        'name': fields.char('Nombre', size=256, required=True)
        
    }
    
    _sql_constraints = [('unique_name','unique(name)', _('Ya existe un centro de salud con el mismo nombre'))]

sisb_reposos_medicos_centro_salud_privado()


class sisb_reposos_medicos_tipo_reposo_ivss(osv.osv):

    _name = "sisb.reposos.medicos.tipo.reposo.ivss"
    _description = "Tipos de reposos medicos del IVSS"
    
    _columns = {
        'name': fields.char('Tipo de reposo del IVSS', size=64),
    }

    _sql_constraints = [('unique_name','unique(name)', _('Ya existe el mismo tipo de reposo'))]
    _order = "name asc"

sisb_reposos_medicos_tipo_reposo_ivss()


class sisb_reposos_medicos_tipo_reposo_servicio_medico(osv.osv):
    
    _name = "sisb.reposos.medicos.tipo.reposo.servicio.medico"
    _description = 'Tipos de reposos medicos del servicio medico.'
    
    _columns = {
        'name': fields.char('Tipo de reposo del servicio medico', size=64),
    }
    _sql_constraints = [('unique_name','unique(name)', _('Ya existe este tipo de reposo medico'))]
    
sisb_reposos_medicos_tipo_reposo_servicio_medico()


class sisb_reposos_medicos_codigo_diagnostico(osv.osv):
    
    _name = "sisb.reposos.medicos.codigo.diagnostico"
    _description = 'Diagnostico codificado por servicio medico para los reposos'
    
    _columns = {
        'name': fields.char('Codigo de diagnostico', size=6),
    }
    
    _sql_constraints = [('unique_name','unique(name)', _('Ya existe este codigo de diagnostico'))]
    _order = "name asc"
    
sisb_reposos_medicos_codigo_diagnostico()


class sisb_reposos_medicos_configuracion(osv.osv):
    _name = "sisb.reposos.medicos.configuracion"
    _description = "Configuracion de grupo de medicos"
    
    _columns = {
        'grupo_medicos_reposos': fields.many2one('res.groups','Grupo medicos', required=True,
                                help="Indica el grupo en el sistema al cual pertenecen los medicos que podran\
                                cargar y convalidar los reposos medicos"),
        'grupo_asistente_medico': fields.many2one('res.groups','Grupo asistente medico',
                                help="Indica el grupo en el sistema el cual podra cargar los reposos"),
    }
    _rec_name = 'grupo_medicos_reposos'
    
    def create(self, cr, uid, values, context):
        configuracion = self.search(cr,uid,[])
        if configuracion:
            raise osv.except_osv(_('Accion no permitida!'), _('Solo pude existir un registro para esta configuracion'))
            
        return super(sisb_reposos_medicos_configuracion, self).create(cr, uid, values, context)
        
sisb_reposos_medicos_configuracion()


class sisb_reposos_medicos(osv.osv):

    _name = 'sisb.reposos.medicos'
    _description = "Certificados de incapacidad (reposos medicos)"
    
    def _get_configuracion(self, cr, uid, context=None):
        """ Funcion para extraer la configuracion de sisb.reposos.medicos.configuracion """
        
        if context is None:
            context = {}

        configuracion_obj = self.pool.get('sisb.reposos.medicos.configuracion')
        configuracion_id = configuracion_obj.search(cr, uid,[])
        if configuracion_id:
            return configuracion_obj.browse(cr, uid, configuracion_id[0], context=None)

        raise osv.except_osv(_('Error'), _('No estan configurado un grupo de medicos y de asistentes para los reposos. Por favor contacte al administrador.'))

    def _get_grupo_id(self, cr, uid, group_name, context=None):
        """Devuelve el id del grupo 'group_name'. Devuelve False en caso de no existir """
        model_data_obj = self.pool.get('ir.model.data')
        grupo_search =  model_data_obj.search(cr, uid, [('name','=',group_name),('model','=','res.groups')], context=None)
        grupo = model_data_obj.read(cr, uid, grupo_search, ['res_id'], context)[0]['res_id']
        if grupo:
            return grupo
        return False
    
    def _get_medico_id(self, cr, uid, context=None):
        grupos_usuario = self._get_grupos_usuario(cr, uid, context)
        grupo_medicos = self._get_grupo_id(cr, uid,'group_sisb_reposos_medicos_user')
        if grupo_medicos in grupos_usuario:
            return self.pool.get('res.users').browse(cr, uid, uid, context).id
        return False

    def _get_grupos_usuario(self, cr, uid, context=None):
        return [group.id for group in self.pool.get('res.users').browse(cr, uid, uid, context=None).groups_id]
        
    def _nota_nomina_editable(self, cr, uid, ids, name, args, context=None):
        """
        Funcion para determinar si el campo 'nota_nomina' es editable. El campo sera editable
        cuando el reposo este convalidado (state = review) y el usuario pertenezca al grupo de nomina
        (group = group_hr_user).
        """
        res = {}
        grupo = self._get_grupo_id(cr, uid,'group_hr_user')
        if grupo:
            grupos_usuario = self._get_grupos_usuario(cr, uid, context)
            reposos = self.browse(cr, uid, ids)
            for reposo in reposos:
                if grupo in grupos_usuario and reposo.state == 'review':
                    res[reposo.id] = True
                else:
                    res[reposo.id] = False
            
        return res
        
    def _debe_reintegrarse_trabajador(self, cr, uid, ids, name, args, context=None):
        """
        Devuelve True si finalizo el reposo del trabajador y este debe reintegrarse
        """
        res = {}
        reposos = self.browse(cr, uid, ids)
        hoy = datetime.now()
        for reposo in reposos:
            fecha_fin_reposo = datetime.strptime(reposo.fecha_fin,'%Y-%m-%d')
            es_reposo_padre = self.search(cr, uid, [('parent_id','=',reposo.id)], context=None)
            if hoy > fecha_fin_reposo and not es_reposo_padre and not reposo.trabajador_reintegrado:
                res[reposo.id] = True
            else:
                res[reposo.id] = False
        return res
        
    def _puede_devolverse_aprobacion(self, cr, uid, ids, name, args, context=None):
        """
        Devuelve True si:
         1) El reposo no tienen conceptos variables en un proceso 
            de nomina que este en estado 'Aprobado' o 'En ejecucion'
         2) Si el reposo no es padre de un reposo en estado aprobado.
        """
        res = {}
        reposos = self.browse(cr, uid, ids)
        historico_obj = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
        linea_obj = self.pool.get('hr.variable.concept.line')
        variable_concept_obj = self.pool.get('hr.variable.concepts')
        
        for reposo in reposos:
            res[reposo.id] = True
            reposo_continuo_id = self.search(cr, uid, [('parent_id','=',reposo.id),('state','=','approved')], context=None)
            if reposo_continuo_id:
                reposo_continuo = self.browse(cr, uid, reposo_continuo_id, context=None)[0]
                if reposo_continuo.state == 'approved':
                    res[reposo.id] = False
            
            elif reposo.state == 'approved':
                historico_ids = historico_obj.search(cr, uid, [('reposo_medico_id','=',reposo.id)],
                                                    order="variable_concepts_id desc", context=None)
                if len(historico_ids) == 1:
                    linea_historico = historico_obj.browse(cr, uid, historico_ids[0], context=None)
                    
                    if linea_historico.variable_concepts_id.state in ['execute','done']:
                        res[reposo.id] = False
                
                elif len(historico_ids) > 1:
                    res[reposo.id] = False
    
            else:
                res[reposo.id] = False
                
        return res
        
    def _puede_calcular_conceptos_variables(self, cr, uid, ids, name, args, context=None):
        """
        Devuelve True si el reposo esta en estado 'approved'y:
        1) No tiene reposo padre (reposo continuo) o 
        2) Si su reposo padre :
            *) Esta en estado 'done' o
            *) Tiene acumulado en conceptos variables el numero de dias de reposo
        """
        if not isinstance(ids, list):
            ids = [ids]
            
        historico_obj = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
        reposos = self.browse(cr, uid, ids)
        
        res = {}
        for reposo in reposos:
            if reposo.state == 'approved':
                if not reposo.parent_id or reposo.parent_id.state == 'done':
                    res[reposo.id] = True
                else:
                    historico_ids = historico_obj.search(cr, uid, [('reposo_medico_id','=',reposo.parent_id.id)], order="id desc", context=None)
                    if historico_ids:
                        historico = historico_obj.browse(cr, uid, historico_ids[0], context=None)
                        if historico.dias_reposo_acumulados >= historico.dias_reposo:
                            res[reposo.id] = True
            else:
                res[reposo.id] = False
        return res
        

    def name_get(self, cr, uid, ids, context=None):
        if not isinstance(ids, list):
            ids = [ids]
        res = []
        reposos = self.browse(cr, uid, ids, context)
        for reposo in reposos:
            fecha_inicio= datetime.strptime(reposo.fecha_inicio,"%Y-%m-%d").strftime("%d/%m/%Y")
            fecha_fin= datetime.strptime(reposo.fecha_fin,"%Y-%m-%d").strftime("%d/%m/%Y")
            res.append((reposo.id, reposo.trabajador_id.name.title() + ' - ' +reposo.tipo_reposo_ivss_id.name + ' desde ' + fecha_inicio + ' hasta ' + fecha_fin))
        return res
    
    def _tiene_conceptos_variables_historico(self, cr, uid, ids, field_name, arg, context):
        res = {}
        historico = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
        reposos_historico_ids = historico.search(cr, uid, [('reposo_medico_id','in',ids)], context=None)
        reposos_historico = historico.browse(cr, uid, reposos_historico_ids, context=None)
        reposos_con_historico = [historico.reposo_medico_id.id for historico in reposos_historico]
        for id in ids:
            if id in reposos_con_historico:
                res[id] = True
            else:
                res[id] = False
        return res
        
    def _validar_reposo_arrastre(self, cr, uid, ids, context=None):
        reposo = self.browse(cr, uid, ids, context=context)[0]
        if reposo.parent_id:
            if reposo.parent_id.state == 'rejected':
                raise osv.except_osv(_('Error'), _('El reposo padre se encuentra en estado "rechazado".'))
            
            if reposo.parent_id.state == 'draft':
                raise osv.except_osv(_('Error'), _('El reposo padre se encuentra en estado "borrador".'))
                
            if reposo.fecha_inicio > reposo.parent_id.fecha_reintegro:
                raise osv.except_osv(_('Error'), _('Reposo de arrastre invalido. Revisar fechas de los reposos.'))
    
    def _validar_rechazo_reposo(self, cr, uid, ids, context=None):
        reposo = self.browse(cr, uid, ids, context=context)[0]
        reposo_hijo_id = self.search(cr, uid, [('parent_id','=',reposo.id)], context=context)
        if reposo_hijo_id:
            reposo_hijo = self.browse(cr, uid, reposo_hijo_id, context=context)[0]
            if reposo_hijo.state != 'rejected':
                raise osv.except_osv(_('Error'), _('No se puede rechazar reposo. Existe un reposo de arrastre en estado distinto a "rechazado"'))        
    
    def _semanas_reposo_acumuladas(self, cr, uid, ids, name, args, context=None):
        """
        Calculo del numero de semanas acumuladas de los reposos 
        continuos.
        """
        res = {}
        reposos = self.browse(cr, uid, ids)
        for reposo in reposos:
            dias_acumulados = reposo.dias_reposo
            reposo_padre = reposo.parent_id
            while reposo_padre:
                dias_acumulados += reposo_padre.dias_reposo
                reposo_padre = reposo_padre.parent_id
            res[reposo.id] = dias_acumulados / 7
        return res

    _columns = {
        'forma_reposo_ivss': fields.selection([('',''),('forma_14-73','14-73'),('forma_14-76','14-76'),('forma_15-289','15-289'),
          ('forma_15-447','15-447'),], 'Forma'),
        'centro_salud_ivss_id': fields.many2one('sisb.reposos.medicos.centro.salud.ivss', 'Centro de salud del IVSS', help='Corresponde al nombre del hospital \
                                                o centro de salud que emite el reposo'),
        'centro_salud_privado_id': fields.many2one('sisb.reposos.medicos.centro.salud.privado', 'Centro de salud', help='Corresponde al centro de salud \
                                                que emite el reposo'),
        'tipo_reposo_medico': fields.selection([('ivss','IVSS'),('privado','Privado'), ], 'Tipo de reposo medico'),  
        'tipo_reposo_ivss_id':fields.many2one('sisb.reposos.medicos.tipo.reposo.ivss','Tipo de reposo del IVSS'),
        'tipo_reposo_servicio_medico_id':fields.many2one('sisb.reposos.medicos.tipo.reposo.servicio.medico','Tipo de reposo servicio medico'),
        'state': fields.selection([('draft','Borrador'), ('review','Convalidado'),('not_review','No Convalidado'),('approved','Aprobado'), ('rejected','Rechazado'),('done','Terminado')], 'Estado'),
        'company_id': fields.many2one('res.company','Empresa'),
        'codigo_sso_compania': fields.related('company_id', 'company_ssn', type="char", size=9, relation="res.company", string="Codigo SSO de la compañía", help='Corresponde al codigo SSO de la empresa'),
        'trabajador_id': fields.many2one('hr.employee','Trabajador'),
        'payroll_id': fields.related('trabajador_id', 'hr_payroll_id', type='many2one', 
          relation='hr.payroll', string="Nomina"),
        'sede_id': fields.related('trabajador_id', 'hr_sede_id', type='many2one', 
          relation='hr.sede', string="Sede"),
        'trabajador_cedula': fields.related('trabajador_id','identification_id', type='char', size=256, relation="hr.employee", string='Cédula'),
        'trabajador_codigo': fields.related('trabajador_id','cod_number', type='char', size=256, relation="hr.employee", string='Codigo'),
        'medico_id': fields.many2one('res.users','Convalidado por'),
        'medico_tratante': fields.char('Medico tratante', size=64, help='Medico que trato al paciente en el centro de salud.'),
        'numero_historia_sso': fields.char('Numero de historia SSO', size=10),
        'codigo_diagnostico_id': fields.many2one('sisb.reposos.medicos.codigo.diagnostico','Codigo de diagnostico'),
        'fecha_inicio': fields.date('Inicio de reposo'),
        'fecha_fin': fields.date('Fin de reposo'),
        'dias_reposo': fields.integer('Días de reposo'),
        'semanas_reposo_acumuladas': fields.function(_semanas_reposo_acumuladas, method=True, string='Semanas de arrastre acumuladas', type='integer', store=True, help="Indica el numero de semanas acumuladas de los reposos continuos incluyendo el actual."),
        'fecha_reintegro': fields.date('Fecha de reintegro'),
        'fecha_emision': fields.date('Fecha de emisión de reposo'),
        'fecha_validacion_sso': fields.date('Fecha de validación SSO', help="Fecha en la que fue convalidado el reposo en el Seguro Social"),
        'fecha_validacion_servicio_medico': fields.date('Fecha de convalidacion'),
        'configuracion_id': fields.many2one('sisb.reposos.medicos.configuracion','Configuracion'),
        'grupo_medicos_reposos': fields.related('configuracion_id','grupo_medicos_reposos',type='many2one',relation='res.groups', string='Grupo de medicos'),
        'nota_medico': fields.text('Nota del medico', help="Observaciones generales sobre la carga del reposo en el sistema"),
        'nota_nomina': fields.text('Nota de nomina'),
        'nota_nomina_editable': fields.function(_nota_nomina_editable, method=True, string='¿Es editable?', type='boolean'),
        'parent_id': fields.many2one('sisb.reposos.medicos','Arrastre', help="Se indica el reposo anterior en caso de que sea un reposo continuo"),
        'conceptos_variables_historico': fields.function(_tiene_conceptos_variables_historico, type='boolean', method=True),
        'debe_reintegrarse_trabajador': fields.function(_debe_reintegrarse_trabajador, method=True, string='¿Debe reintegrarse?', type='boolean'),
        'trabajador_reintegrado': fields.boolean('¿Trabajador reintegrado?'),
        'puede_devolverse_aprobacion': fields.function(_puede_devolverse_aprobacion, method=True, string='¿Puede devolverse aprobacion?', type='boolean'),
        'puede_calcular_conceptos_variables': fields.function(_puede_calcular_conceptos_variables, method=True, string='¿Se pueden calcular conceptos variables del reposo?', type='boolean'),
        'variable_concepts_id': fields.many2many('hr.variable.concepts', 'sisb_reposos_medicos_conceptos_variables_rel',\
                            'reposo_medico_id','variable_concepts_id','Conceptos Variables'),
        }
    
    _defaults = {
        'forma_reposo_ivss': None,
        'tipo_reposo_ivss_id': None,
        'tipo_reposo_servicio_medico_id': None,
        'state': 'draft',
        'company_id': lambda self, cr, uid, c: self.pool.get('res.users').browse(cr, uid, uid, c).company_id.id,
        'grupo_medicos_reposos': lambda self, cr, uid, c: self._get_configuracion(cr, uid, c).grupo_medicos_reposos.id,
        'parent_id': None,
        'trabajador_reintegrado': False,
    }
        
    _rec_name = "trabajador_id"
    _order = "fecha_inicio desc, dias_reposo asc, state"

    def validar_fechas_reposo(self, cr, uid, ids, values, context=None):
        
        if not isinstance(ids, list):
            ids = [ids]
        
        DIAS_REQUIERE_SSO = self.pool.get('sisb.reposos.medicos.conceptos.variables')._get_parameter_id(cr, uid, ids, 'cantidad_dias_reposo_medico_paga_empresa')
        LIMITE_DIAS_REPOSO= 365
        tipo_reposo_medico = values.get('tipo_reposo_medico')
        reposo = self.browse(cr, uid, ids, context)

        if reposo:
            if 'fecha_inicio' not in values:
                values['fecha_inicio'] = reposo[0].fecha_inicio
            if 'fecha_fin' not in values:
                values['fecha_fin'] = reposo[0].fecha_fin
            if 'fecha_reintegro' not in values:
                values['fecha_reintegro'] = reposo[0].fecha_reintegro
            tipo_reposo_medico = reposo[0].tipo_reposo_medico
            if 'fecha_validacion_sso' not in values:
                values['fecha_validacion_sso'] = reposo[0].fecha_validacion_sso
            if 'dias_reposo' not in values:
                values['dias_reposo'] = reposo[0].dias_reposo
            if 'parent_id' not in values:
                values['parent_id'] = reposo[0].parent_id.id

        fecha_fin = datetime.strptime(values['fecha_fin'],"%Y-%m-%d")
        values['fecha_reintegro'] = str(fecha_fin + timedelta(days=1))
        
        if values['fecha_inicio'] > values['fecha_fin']:
            raise osv.except_osv(_('Error'), _('La fecha de fin del reposo debe ser mayor que la fecha de inicio'))
        
        if values['dias_reposo'] > DIAS_REQUIERE_SSO:
            if tipo_reposo_medico == 'ivss':
                if values['fecha_validacion_sso'] < values['fecha_inicio'] and not values['parent_id']:
                    raise osv.except_osv(_('Error'), _('La fecha de validacion del reposo en el Seguro Social debe ser mayor a la fecha de inicio del reposo'))
            elif tipo_reposo_medico == 'privado':
                raise osv.except_osv(_('Error'), _('Los reposos de centros de salud distintos al IVSS no pueden ser de mas de %s dias.') %DIAS_REQUIERE_SSO )
        
        if values['dias_reposo'] > LIMITE_DIAS_REPOSO:
            raise osv.except_osv(_('Error'), _('Los reposos medicos no pueden exceder los 365 dias'))
            
    def rechazar_reposo_solapado(self, cr, uid, ids, values, context=None):
        """
        No se permiten fechas de reposo solapadas a menos que sea un reposo continuo.
        """
        if not isinstance(ids, list):
            ids = [ids]
            
        reposo_id = False
        reposo_padre_id = False
        reposo_hijo_id = False
        
        if 'write' in context:
            reposo =  self.browse(cr, uid, ids, context)[0]
            if reposo:
                if 'fecha_inicio' not in values:
                    values['fecha_inicio'] = reposo.fecha_inicio
                if 'fecha_fin' not in values:
                    values['fecha_fin'] = reposo.fecha_fin
                    values['fecha_fin'] = reposo.fecha_fin
                if 'trabajador_id' not in values:
                    values['trabajador_id'] =  reposo.trabajador_id.id
                if 'parent_id' not in values and reposo.parent_id:
                    values['parent_id'] =  reposo.parent_id.id
                    
                reposo_id = reposo.id
                
                reposo_hijo_id = self.search(cr, uid, [('parent_id','=',reposo.id)], context=None)
                if reposo_hijo_id:
                    reposo_hijo_id = reposo_hijo_id[0]
        
        reposo_padre_id = values.get('parent_id')
        reposo_solapado_ids = self.search(cr, uid, ['|','|',
                                                  '&',('fecha_inicio','<=',values['fecha_inicio']),('fecha_fin','>=',values['fecha_inicio']),
                                                  '&',('fecha_inicio','<=',values['fecha_fin']),('fecha_fin','>=',values['fecha_fin']),
                                                  '&',('fecha_inicio','>=',values['fecha_inicio']),('fecha_inicio','<=',values['fecha_fin']),
                                                  ('trabajador_id','=',values.get('trabajador_id')),
                                                  ('id','!=',reposo_id)])

        if reposo_solapado_ids:
            if len(reposo_solapado_ids) == 1:
                if reposo_padre_id == reposo_solapado_ids[0] or reposo_hijo_id == reposo_solapado_ids[0]:
                    return True
            reposo_solapado = self.browse(cr, uid, reposo_solapado_ids, context)[0]
            fecha_inicio = datetime.strptime(reposo_solapado.fecha_inicio,"%Y-%m-%d")
            fecha_inicio = datetime.strftime(fecha_inicio, "%d-%m-%Y")
            fecha_fin = datetime.strptime(reposo_solapado.fecha_fin,"%Y-%m-%d")
            fecha_fin = datetime.strftime(fecha_fin, "%d-%m-%Y")

            raise osv.except_osv(_('Error'), _("El reposo que intenta crear entra en conflicto con el reposo\
                                               de tipo '%s' con fecha de inicio '%s' y fecha de fin '%s'")
                                               %(reposo_solapado.tipo_reposo_ivss_id.name, fecha_inicio, fecha_fin))

    def validar_dias_forma_reposo_ivss(self, cr, uid, ids, values, context=None):
        """ Valida que sea correcta la cantidad de dias dependiendo del numero
        de forma del reposo del IVSS"""
        if not isinstance(ids, list):
            ids = [ids]
        
        DIAS_REQUIERE_SSO = self.pool.get('sisb.reposos.medicos.conceptos.variables')._get_parameter_id(cr, uid, ids, 'cantidad_dias_reposo_medico_paga_empresa')
        forma_reposo_ivss = values.get('forma_reposo_ivss', '')
        dias_reposo = values.get('dias_reposo')
        reposo = self.browse(cr, uid, ids, context)
        if reposo:
            if 'dias_reposo' not in values:
                dias_reposo = reposo[0].dias_reposo
                
            if 'forma_reposo_ivss' not in values:
                forma_reposo_ivss = reposo[0].forma_reposo_ivss
                
        
        if forma_reposo_ivss == 'forma_15-289' and dias_reposo > DIAS_REQUIERE_SSO:
            raise osv.except_osv(_('Error'), _("Los reposos de la Forma '15-289' no pueden ser mayor a %d dias.") %DIAS_REQUIERE_SSO)
        
    def create(self, cr, uid, values, context=None):
        """
        Funcion para hacer validaciones al momento de crear un reposo:
            *- Se garantiza que el numero de dias de reposo sea correcto
            *- Se evita que se almacene un reposo sin que se haya asignado
               el codigo SSO de la compañia.
            *- Se evita que se creen reposos solapados.
        """
        try:
            fecha_inicio = datetime.strptime(values['fecha_inicio'],"%Y-%m-%d")
            fecha_fin = datetime.strptime(values['fecha_fin'],"%Y-%m-%d")
        except:
            raise osv.except_osv(_('Error'), _('Fechas invalidas'))

        values['dias_reposo'] = (fecha_fin - fecha_inicio).days + 1
        values['tipo_reposo_medico'] = context.get('tipo_reposo_medico')
        
        self.validar_dias_forma_reposo_ivss(cr, uid, [], values, context)
        self.validar_fechas_reposo(cr, uid, [], values, context)
        self.rechazar_reposo_solapado(cr, uid, [], values, context)

        forma_reposo_ivss = values.get('forma_reposo_ivss')
        
        company = self.pool.get('res.users').browse(cr,uid,uid,context).company_id.id
        codigo_sso_compania = self.pool.get('res.company').browse(cr, uid, company, context).company_ssn

        if not codigo_sso_compania and forma_reposo_ivss:
            raise osv.except_osv(_('Error'), _('El codigo SSO de la compañia es invalido. Por favor contacte al administrador'))
            
        else:
            values['codigo_sso_compania'] = codigo_sso_compania
            return super(sisb_reposos_medicos,self).create(cr, uid, values, context)
        
    def write(self, cr, uid, ids, values, context=None):
        """
        Funcion para garantizar que: 
            *- Se escriba una nota al momento de rechazar un reposo.
            *- No se guarde un reposo solapado.
        """
        if context is None:
            context = {}
        
        context.update({'write': True})
        if not isinstance(ids, list):
            ids = [ids]
        
        if '_cambiar_estado_reposos' in context:
            return super(sisb_reposos_medicos, self).write(cr, uid, ids, values, context)

        self.validar_dias_forma_reposo_ivss(cr, uid, ids, values, context)
        self.validar_fechas_reposo(cr, uid, ids, values, context)
        self.rechazar_reposo_solapado(cr, uid, ids, values, context)
        
        reposo_brw = self.browse(cr, uid, ids)
        if reposo_brw:
            reposo = reposo_brw[0]
            if 'state' in values and values['state'] == 'rejected':
                if (reposo.nota_nomina is False or reposo.nota_nomina.strip() == '') and 'nota_nomina' not in values:
                    raise osv.except_osv(_('Accion no permitida!'), _('Debe escribir una nota para rechazar el reposo'))
            return super(sisb_reposos_medicos, self).write(cr, uid, ids, values, context)
        return True

    def copy(self, cr, uid, id, default=None, context=None):
        """ Funcion que evita que se dupliquen los reposos """
        raise osv.except_osv(_('Accion no permitida!'), _('No se pueden duplicar los reposos'))
        return False
        
    def unlink(self, cr, uid, ids, context=None):
        """Funcion que evita que se eliminen reposos que no esten en estado borrador"""
        reposo_brw = self.browse(cr, uid, ids)
        for reposo in reposo_brw:
            if reposo.state != 'draft':
                raise osv.except_osv(_('Accion no permitida!'), _('No se permite eliminar reposos que se encuentren en estado \
                                                                  distinto a borrador'))
        return super(sisb_reposos_medicos, self).unlink(cr, uid, ids, context)
        
    def onchange_trabajador(self, cr, uid, ids, trabajador_id=False):
        """ On change de trabajador (trabajador_id). Actualiza el campo
        de cedula del trabajador cuando se establece el nombre del
        trabajador. """
        result = {}
        if trabajador_id:
            trabajador = self.pool.get('hr.employee').browse(cr, uid, trabajador_id)
            if trabajador:
                result = {
                        'trabajador_cedula': trabajador.identification_id,
                        'parent_id': False,
                }
        
        return {'value': result}
    
    def onchange_company(self, cr, uid, ids, company_id=False):
        """ On change de compañia (company_id). Actualiza el campo
        de codigo_sso_compania cuando se establece el nombre de la
        compañia."""
        result = {}
        if company_id:
            company = self.pool.get('res.company').browse(cr,uid,company_id)
            if company:
                result = {'codigo_sso_compania': company.company_ssn}

        return {'value': result}
        
    def onchange_fecha(self, cr, uid, ids, fecha_inicio=False, fecha_fin=False):
        """ On change de fechas (fecha_inicio, fecha_fin). 
        #~ Verifica que fecha inicio > fecha fin > fecha_reintegro"""
        result = {}
        value = {}      
        if fecha_inicio and fecha_fin:
            if fecha_inicio <= fecha_fin:
                fecha_inicio = datetime.strptime(fecha_inicio,"%Y-%m-%d")
                fecha_fin = datetime.strptime(fecha_fin,"%Y-%m-%d")
                fecha_reintegro = fecha_fin + timedelta(days=1)
                
                value = {
                    'fecha_reintegro': str(fecha_reintegro),
                    'dias_reposo': (fecha_fin - fecha_inicio).days + 1,
                    'parent_id': False,
                }
                result = {'value': value}
        return result
        
    def onchange_tipo_reposo(self, cr, uid, ids, tipo_reposo=False):

        result = {}
        value = {}      
        if tipo_reposo:
            value = {
                'parent_id': False,
            }
            result = {'value': value}
        return result
    
    def onchange_dias_reposo(self, cr, uid, ids, fecha_inicio=False, fecha_fin=False, dias_reposo=False):
        result= {}
        if dias_reposo < 1:
            dias_reposo = 1
        if fecha_inicio:
            fecha_inicio = datetime.strptime(fecha_inicio,"%Y-%m-%d")
            fecha_fin = fecha_inicio + timedelta(days=dias_reposo - 1)
            fecha_reintegro = fecha_inicio + timedelta(days=dias_reposo)
            value= {
                'fecha_fin': str(fecha_fin),
                'fecha_reintegro': str(fecha_reintegro),
                'dias_reposo': dias_reposo,
            }
            result = {'value': value}
        return result

    def onchange_parent_id(self, cr, uid, ids, parent_id, context=None):
        """
        Si un reposo es continuo:
            1) El reposo debe ser del mismo tipo que el anterior.
        """
        result = {}
        if parent_id:
            parent = self.browse(cr, uid, parent_id, context)
            if parent:
                fecha_inicio = datetime.strptime(parent.fecha_fin,"%Y-%m-%d") + timedelta(days=1)
                result = {
                    'tipo_reposo_ivss_id': parent.tipo_reposo_ivss_id.id,
                    'tipo_reposo_servicio_medico_id': parent.tipo_reposo_servicio_medico_id.id,
                    'trabajador_id': parent.trabajador_id.id,
                    'trabajador_cedula': parent.trabajador_cedula,
                    #'fecha_inicio': fecha_inicio,
                }
        return {'value': result}

    def get_tipo_ausencia_reposos_medicos(self, cr, uid, context=None):
        hr_parameter = self.pool.get('hr.parameters')
        parametro_id = hr_parameter.search(cr, uid, [('type','=','tipo_ausencia_reposos_medicos')], context=None)
        if not parametro_id:
            raise osv.except_osv(_('ERROR'), _("No existe un parametro definido para 'Tipo de ausencia para reposos medicos' : \
                                                Ir a la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                registrar este parametro."))
        
        tipo_ausencia = hr_parameter.browse(cr, uid, parametro_id, context=None)[0].tipo_ausencia_reposos_medicos_id.id
        return tipo_ausencia
    
    def _get_dias_ausencia(self, cr, uid, fecha_inicio, fecha_fin):
        hr_holidays = self.pool.get('hr.holidays')
        fecha_inicio += " 00:00:00"
        fecha_fin += " 00:00:00"
        result = hr_holidays.onchange_date_from(cr, uid, [], fecha_inicio, fecha_fin)
        dias_ausencia =  result['value']['number_of_days_temp']
        
        return dias_ausencia
    
    def _validar_aprobacion(self, cr, uid, ids, context):
        """
        Muestra un mensaje de error en caso de que:
            *- Se intente aprobar un reposo a una persona que ya tenga 
               un reposo en estado aprobado. Este mensaje no se muestra 
               si el reposo es un reposo continuo (tiene parent_id)
            *- Se intenta aprobar un reposo cuando existe otro 
               en borrador o convalidado de fecha menor.
        """
        reposo = self.browse(cr, uid, ids, context)[0]
        trabajador_id = reposo.trabajador_id
        
        reposo_aprobado = self.search(cr, uid, [('trabajador_id','=',trabajador_id.id),('state','=','approved')], context=None)
        if reposo_aprobado and not reposo.parent_id:
            raise osv.except_osv(_('Error'), _('Ya existe un reposo del trabajador %s en estado aprobado. \
                                               Solo se permiten dos reposos en estado aprobado si son \
                                               reposos continuos. ') % trabajador_id.name)
        
        reposo_anterior = self.search(cr, uid, [('trabajador_id','=',trabajador_id.id),
                                     ('state','in',['draft','review']),
                                     ('fecha_inicio','<',reposo.fecha_inicio)],
                                     order='fecha_inicio asc', context=None)
        if reposo_anterior:
            raise osv.except_osv(_('Error'), _("El reposo '%s' tiene fecha de inicio \
                                menor al reposo que esta intentando aprobar.")
                                % (self.name_get(cr, uid, reposo_anterior)[0][1]))
        return True

    def reposo_crear_lineas_conceptos_variables(self, cr, uid, ids, context=None):
        """
        Para crear conceptos variables de un solo reposo.
        """
        if context is None:
            context = {}
        reposo = self.browse(cr, uid, ids, context=None)
        reposos_conceptos_obj = self.pool.get('sisb.reposos.medicos.conceptos.variables')
        return reposos_conceptos_obj.crear_lineas_conceptos_variables(cr, uid, ids, context, reposo[0])
    
    def marcar_trabajador_reintegrado(self, cr, uid, ids, context=None):
        self.cambiar_estado_trabajador(cr, uid, ids, context=context, estado='activo')
        return self.write(cr, uid, ids, {'trabajador_reintegrado': True}, context=context)


    def cambiar_estado_trabajador(self, cr, uid, ids, context=None, estado='activo'):
        """
        Cambia estado de trabajador.
        """
        hr_employee = self.pool.get('hr.employee')
        reposo =  self.browse(cr, uid, ids[0], context)
        employee_id = reposo.trabajador_id.id

        return hr_employee.write(cr, uid, employee_id, {'state': estado}, context)

    def crear_ausencia_por_reposo(self, cr, uid, ids, context=None):
        """
        -)Metodo para crear una ausencia en hr.holidays al momento de aprobar un reposo.
        -)Si el reposo se solapa con su reposo padre se ajusta la fecha de la ausencia
        y se restan los dias solapados.
        """
        hr_holidays = self.pool.get('hr.holidays')
        tipo_ausencia_reposos_medicos = self.get_tipo_ausencia_reposos_medicos(cr, uid, context)
        
        
        reposo =  self.browse(cr, uid, ids[0], context)
        if reposo.fecha_inicio == reposo.fecha_fin:
            dias_ausencia = 1
        else:
            dias_ausencia = self._get_dias_ausencia(cr, uid, reposo.fecha_inicio, reposo.fecha_fin)

        values = {
                    'name': "Reposo de tipo " + reposo.tipo_reposo_ivss_id.name,
                    'employee_id': reposo.trabajador_id.id,
                    'holiday_status_id': tipo_ausencia_reposos_medicos,
                    'date_from': reposo.fecha_inicio,
                    'date_to': reposo.fecha_fin,
                    'number_of_days_temp': dias_ausencia,
                    'state': 'validate',
        }
        
        if reposo.fecha_inicio < reposo.parent_id.fecha_reintegro:
            values['date_from'] = reposo.parent_id.fecha_reintegro
            dias_solapados = self._get_dias_ausencia(cr, uid, reposo.fecha_inicio, reposo.parent_id.fecha_fin)
            values['number_of_days_temp'] -= dias_solapados
            values['notes'] = "Ausencia de un reposo de arrastre de fechas solapadas con su reposo padre"

        return hr_holidays.create(cr, uid, values, context=None)

    def eliminar_ausencia_de_reposo(self, cr, uid, ids, context=None):
        """
        Metodo para eliminar una ausencia en hr.holidays
        """
        hr_holidays = self.pool.get('hr.holidays')
        reposo =  self.browse(cr, uid, ids[0], context)
        
        if reposo.parent_id and reposo.fecha_inicio < reposo.parent_id.fecha_reintegro:
            #Reposo solapado
            ausencia_id = hr_holidays.search(cr, uid, [('employee_id','=',reposo.trabajador_id.id),
                                            ('date_from','=',reposo.parent_id.fecha_reintegro)], context=None)
        else:
            ausencia_id = hr_holidays.search(cr, uid, [('employee_id','=',reposo.trabajador_id.id),
                                            ('date_from','=',reposo.fecha_inicio)], context=None)
        
        hr_holidays.write(cr, uid, ausencia_id, {'state': 'draft'}, context=None)
        return hr_holidays.unlink(cr, uid, ausencia_id, context=None)

    def reposo_review(self, cr, uid, ids, context=False):
        self._validar_reposo_arrastre(cr, uid, ids, context)        
        fecha_validacion = datetime.now()
        self.crear_ausencia_por_reposo(cr, uid, ids, context)
        self.cambiar_estado_trabajador(cr, uid, ids, context=context, estado='reposo')
        values = {
             'medico_id': self._get_medico_id(cr, uid, context),
             'fecha_validacion_servicio_medico': fecha_validacion,
             'state':'review',
        }
        return self.write(cr, uid, ids, values, context)

    def reposo_not_review(self, cr, uid, ids, context=False):
        values = {
             'state':'not_review',
        }
        return self.write(cr, uid, ids, values, context) 
    
    def reposo_approved(self, cr, uid, ids, context=False):
        self._validar_aprobacion(cr, uid, ids, context)
        self._validar_reposo_arrastre(cr, uid, ids, context)
        return self.write(cr, uid, ids, {'state':'approved'}, context)
        
    def reposo_devolver_aprobacion(self, cr, uid, ids, context=None):
        """
        Funcion para devolver la aprobacion de un reposo. Se eliminan
        las lineas de historico de conceptos variables y se cambiara a 
        estado 'Convalidado'
        """
        reposo_id = ids[0]
        result = self._puede_devolverse_aprobacion(cr, uid, ids, None, None, context=context)
        if result[reposo_id]:
            reposos_conceptos_variables = self.pool.get('sisb.reposos.medicos.conceptos.variables')
            historico_obj = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
            reposo = self.browse(cr, uid, ids, context=None)
            
            historico_ids =  historico_obj.search(cr, uid, [('reposo_medico_id','=',reposo[0].id)], context=None)
            historico = historico_obj.browse(cr, uid, historico_ids, context=None)
            if historico:
                reposos_conceptos_variables._eliminar_historico_y_lineas_conceptos_variables(cr, uid, reposo, context)
            
            return self.write(cr, uid, ids, {'state': 'review'}, context)
        else:
            raise osv.except_osv(_('Error'), _('No puede devolverse aprobacion. EL reposo continuo "%s" se encuentra \
                                              en estado "aprobado" ') %self.name_get(cr, uid, reposo_id)[0][1])
        return False

    def reposo_rejected(self, cr, uid, ids, context=False):
        self._validar_rechazo_reposo(cr, uid, ids, context)
        return self.write(cr, uid, ids, {'state':'rejected'}, context)
        
    def reposo_cancel(self, cr, uid, ids, context=False):
        self.eliminar_ausencia_de_reposo(cr, uid, ids, context)
        reposo = self.browse(cr, uid, ids, context=context)[0]
        reposo_hijo_id = self.search(cr, uid, [('parent_id','=',reposo.id),('state','!=','draft')], context=None)
        if reposo_hijo_id:
            reposo_hijo = self.browse(cr, uid, reposo_hijo_id, context=None)
            raise osv.except_osv(_('Error'), _('No se puede cambiar a borrador. El reposo es padre de "%s" el cual se encuentra en un estado distinto a borrador.')%self.name_get(cr, uid, reposo_hijo_id)[0][1])
        hoy = datetime.strftime(date.today(),"%Y-%m-%d")
        if not reposo.parent_id or reposo.parent_id.fecha_fin < hoy:
            self.cambiar_estado_trabajador(cr, uid, ids, context=context, estado='activo')
        values = {
            'medico_id': False,
            'fecha_validacion_servicio_medico': False, 
            'state':'draft',
            'trabajador_reintegrado': False,
        }
        return self.write(cr, uid, ids, values, context)

sisb_reposos_medicos()
