#-*- coding:utf-8 -*-
##############################################################################
#
#    SISB, Sistema Integral Simón Bolivar
#    Copyright (C) 2012-2013 Industrias Diana, C.A. All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
from datetime import datetime
import logging

ELIMINAR_LINEAS_REPOSO = 'eliminar_linea_reposo'
ES_LINEA_REPOSO_MEDICO = 'es_linea_reposo_medico'

class hr_variable_concepts(osv.osv):
    """
    Herencia para:
    1) Incluir funcion para obtener el nombre de los conceptos
    variables.
    2)Para permitir la eliminacion de lineas de conceptos variables
    asociadas a un reposo cuando se eliminan los conceptos variables.
    """
    _inherit = 'hr.variable.concepts'
    _description = 'Variable Concepts'
    _rec_name = 'hr_process_id'
    
    _columns = {
        'para_reposos_medicos': fields.boolean('¿Para cargar reposos medicos?', readonly=True,
                                help="En estos conceptos variables solo se cargan \
                                lineas asociadas a reposos creadas por \
                                el modulo 'SISB Reposos Medicos' "),
    }

    _defaults = {
        'para_reposos_medicos': False,
    }

    def name_get(self, cr, uid, ids, context=None):
        res = []
        conceptos = self.browse(cr, uid, ids, context)
        res = [(concepto.id,concepto.hr_process_id.name) for concepto in conceptos]
        return res
    
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        
        variable_concepts = self.browse(cr, uid, ids, context=context)[0]
        reposos_conceptos_variables = self.pool.get('sisb.reposos.medicos.conceptos.variables')
        
        if variable_concepts.state in ['draft','review','cancel']:
            if variable_concepts.para_reposos_medicos:
                reposos = set()
                context['wizard'] = True  #Para que se elimine todos los registros de la creados a partir del wizard de conceptos variables de los reposos 
                for linea in variable_concepts.variable_concept_line_ids:
                    for reposo in linea.reposo_medico_ids:
                        reposos.add(reposo)
                reposos = list(reposos)
                reposos_conceptos_variables._eliminar_historico_y_lineas_conceptos_variables (cr, uid, reposos, context=context)
                context.pop('wizard')
        super(hr_variable_concepts,self).unlink(cr, uid, ids, context=context)
        return True

hr_variable_concepts()

class sisb_reposos_medicos_conceptos_variables_rel(osv.osv):
    """
    Se sobreescribe la relacion many2many (entre sisb_reposos_medicos y hr_variable_concepts )
    para incluir los campos de codigo de concepto y valor de linea de concepto variable.
    Estos valores permiten realizar correctamente los calculos para el pago continuo de reposos de arrastre.
    """
    _name = 'sisb.reposos.medicos.conceptos.variables.rel'
    _description = "Relacion Reposos medicos - Conceptos Variables"
    _log_access = False
    
    _columns = {
        'variable_concepts_id': fields.many2one('hr.variable.concepts', 'Conceptos variables'),
        'reposo_medico_id': fields.many2one('sisb.reposos.medicos', 'Reposo',),
        'code_concept_id': fields.many2one('hr.concept', 'Codigo Concepto'),
        'value': fields.integer('Valor de la linea (dias)'),
    }
    
    def set_value(self, cr, reposo_id, variable_concepts_id, code_concept_ids, value):
        if not isinstance(code_concept_ids, list):
            code_concept_ids = [code_concept_ids]
        
        query = " SELECT * \
                  FROM sisb_reposos_medicos_conceptos_variables_rel \
                  WHERE reposo_medico_id = %s and variable_concepts_id = %s \
                         and code_concept_id in %s"
        cr.execute(query, (reposo_id, variable_concepts_id, tuple(code_concept_ids)))
        result = cr.fetchall()
        if not result:
            query = " INSERT INTO  sisb_reposos_medicos_conceptos_variables_rel\
                      VALUES %s"
            for concept_id in code_concept_ids:
                values = (reposo_id, variable_concepts_id, concept_id, value)
                cr.execute(query, (values,))
        else:
            query = " UPDATE sisb_reposos_medicos_conceptos_variables_rel \
                      SET value = %s \
                      WHERE reposo_medico_id = %s and variable_concepts_id = %s \
                             and code_concept_id in %s"
            cr.execute(query, (value,reposo_id, variable_concepts_id, tuple(code_concept_ids)))
        
    def get_value(self, cr, reposo_id, variable_concepts_id, code_concept_id):
        query = " SELECT value \
                  FROM sisb_reposos_medicos_conceptos_variables_rel\
                  WHERE reposo_medico_id = %s and variable_concepts_id = %s \
                         and code_concept_id = %s"
        cr.execute(query, (reposo_id, variable_concepts_id,code_concept_id))
        value = cr.fetchall()
        if value:
            return value[0][0]
        return False
        
    def delete(self, cr, reposo_id, variable_concepts_id, code_concept_ids=None):
        if not isinstance(reposo_id,list):
            reposo_id = [reposo_id]
        if not isinstance(variable_concepts_id,list):
            variable_concepts_id = [variable_concepts_id]
        
        if not code_concept_ids:
            query = " DELETE \
                      FROM sisb_reposos_medicos_conceptos_variables_rel \
                      WHERE reposo_medico_id in %s and variable_concepts_id in %s"
            cr.execute(query, (tuple(reposo_id),tuple(variable_concepts_id)))
        else:
            if not isinstance(code_concept_ids,list):
                code_concept_ids = [code_concept_ids]
            query = " DELETE \
                      FROM sisb_reposos_medicos_conceptos_variables_rel \
                      WHERE reposo_medico_id in %s and variable_concepts_id in %s \
                                                    and code_concept_id in %s"
            cr.execute(query, (tuple(reposo_id),tuple(variable_concepts_id),tuple(code_concept_ids)))
        return True

    def _auto_init(self, cr, context=None):
        """
        Se elimina la clave primaria por defecto.
        """
        drop_default_primary_key =  """
                                    ALTER TABLE sisb_reposos_medicos_conceptos_variables_rel
                                    DROP CONSTRAINT IF EXISTS sisb_reposos_medicos_concepto_reposo_medico_id_variable_con_key
                                    """
        result = super(sisb_reposos_medicos_conceptos_variables_rel, self)._auto_init(cr, context)
        cr.execute(drop_default_primary_key)
        return result
        
    #Se crea una nueva clave primaria que incluye el codigo de concepto variable. 
    _sql_constraints = [('composite_pk','unique(reposo_medico_id,variable_concepts_id,code_concept_id)', 'Error: Registro duplicado')]

sisb_reposos_medicos_conceptos_variables_rel()


class hr_variable_concept_line(osv.osv):
    """
    Herencia para evitar que se eliminen directamente lineas de conceptos variables
    asociadas a un reposo 
    """

    _inherit = "hr.variable.concept.line"
    _description = 'Variable Concept Lines'
    
    _columns = {
        'reposo_medico_ids': fields.many2many('sisb.reposos.medicos', 'sisb_reposos_medicos_linea_concepto_variable_rel',\
                            'variable_concept_line_id', 'reposo_medico_id', 'Reposos'),
    }
    
    def create(self, cr, uid, values, context=None):
        if context is None:
            context = {}
        conceptos_variables_obj =  self.pool.get('hr.variable.concepts')
        conceptos_variables =  conceptos_variables_obj.browse(cr, uid, values['variable_concept_id'], context=None)
        
        if conceptos_variables.para_reposos_medicos and not ES_LINEA_REPOSO_MEDICO in context:
            raise osv.except_osv(_('Error'), _('No se pueden crear lineas de concepto\
                                 variable que no esten asociadas a un reposo medico'))
        
        return super(hr_variable_concept_line,self).create(cr, uid, values, context=None)
        
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
        if not isinstance(ids, list):
            ids = [ids]
        lineas_reposo = self.search(cr, uid, [('id','in',ids),('reposo_medico_ids','!=',False)])
        if lineas_reposo and ELIMINAR_LINEAS_REPOSO not in context:
            raise osv.except_osv(_('ERROR'), _('No se pueden eliminar lineas de conceptos variables asociadas a un reposo'))
        
        return super(hr_variable_concept_line,self).unlink(cr, uid, ids, context)

hr_variable_concept_line()

class sisb_reposos_medicos_conceptos_variables_historico(osv.osv):

    _name = "sisb.reposos.medicos.conceptos.variables.historico"
    _description = ""

    _columns = {
        'reposo_medico_id': fields.many2one('sisb.reposos.medicos','Reposo', required=True, readonly=True),
        'trabajador_id': fields.related('reposo_medico_id', 'trabajador_id', type="many2one", relation="hr.employee", string="Trabajador"),
        'variable_concepts_id': fields.many2one('hr.variable.concepts','Conceptos variables', required=True, readonly=True),
        'variable_concepts_state': fields.related('variable_concepts_id', 'state', type="char", size=9, relation="", string="Estado de los conceptos variables"),
        'dias_reposo': fields.integer('Dias reposo', help="Dias totales del reposo", required=True, readonly=True),
        'dias_reposo_cargados': fields.integer('Dias  cargados', help="Cantidad de dias del reposo que se cargan en los conceptos variables actuales", required=True, readonly=True),
        'dias_reposo_acumulados': fields.integer('Dias acumulados', required=True, readonly=True, help="Cantidad total de dias del reposo que se han \
                                                 cargado en conceptos variables anteriores incluyendo los actuales."),
        'reposo_solapado_completado': fields.boolean('Completado', help="Indica cuando se ha descontado en su totalidad el reposo solapado ya que en este \
                                                caso los dias_reposo_acumulados no seran mayor que los dias_reposo. Por lo tanto el reposo se pasara a estado 'done' \
                                                en la aprobacion del proceso de nomina"),
    }
    
    def create(self, cr, uid, values, context=None):
        if context is None:
            context = {}
 
        if 'no_create' in context:
           raise osv.except_osv(_('Accion no permitida!'), _('No se permite la creacion manual.'))
        return super(sisb_reposos_medicos_conceptos_variables_historico, self).create(cr, uid, values, context)
        
    def copy(self, cr, uid, ids, values,context=False):
        raise osv.except_osv(_('Accion no permitida!'), _('No se permite duplicar este documento'))
    
    def unlink(self, cr, uid, ids, context=None):
        if context is None:
            context = {}
 
        if 'unlink' in context:
            return super(sisb_reposos_medicos_conceptos_variables_historico, self).unlink(cr, uid, ids, context)
        else:
            raise osv.except_osv(_('Accion no permitida!'), _('No se permite eliminar este registro'))

    _defaults = {
        'reposo_solapado_completado': False,
    }

sisb_reposos_medicos_conceptos_variables_historico()

class sisb_reposos_medicos_conceptos_variables(osv.osv_memory):
    """
    Wizard para crear en un proceso de nomina los conceptos variables de
    reposos remunerados.
    """
    _name = "sisb.reposos.medicos.conceptos.variables"
    _description = ""
    _logger = logging.getLogger('sisb.reposos.medicos')

    _columns = {
        'hr_payroll_id': fields.many2one('hr.payroll','Nomina'),
        'hr_sede_ids': fields.many2many('hr.sede','sisb_reposos_medicos_conceptos_variables_sede_rel','sisb_reposos_medicos_conceptos_variables_id','hr_sede_id'),
        'ejecutado':fields.boolean('Ejecutado'),
    }
    
    _defaults = {
        'ejecutado': False,
    }
    
    def get_lineas_concepto_variable(self, cr, uid, employee_id, variable_concept_id, conceptos, context=None):
        if not isinstance(conceptos, list):
            conceptos = [conceptos]
        linea_concepto_variable =  self.pool.get('hr.variable.concept.line')
        linea_concepto_variable_id = linea_concepto_variable.search(cr, uid, 
                                                [('employee_id','=',employee_id),
                                                ('variable_concept_id','=',variable_concept_id),
                                                ('concept_id','in',conceptos),
                                                ], context=None)
        if linea_concepto_variable_id:
            return linea_concepto_variable.browse(cr, uid, linea_concepto_variable_id, context=None)
        
        return None
        
    def _get_parameter_id(self, cr, uid, ids, tipo_parametro, context=None):
        """
        Metodo para obtener el valor de un parametro de nomina a partir de su tipo.
        Ocurre una excepcion en caso de que no este definido el parametro.
        """
        hr_parameter = self.pool.get('hr.parameters')
        parametro_id = hr_parameter.search(cr, uid, [('type','=',tipo_parametro)], context=None)
        if not parametro_id:
            raise osv.except_osv(_('Accion no permitida!'), _("No existe un parametro definido para uno o varios de los siguientes conceptos: \
                                                              'Reposo Remunerado', '1/3 Sueldo', '2/3 Sueldo', \
                                                              'Cantidad de dias de reposo medico que paga la empresa', \
                                                              'Correo para envio de notificaciones sobre reposos medicos'. \
                                                              Ir a la pestaña Configuracion -> Configuraciones de nomina -> Parametros de nomina y \
                                                              registrar los parametros no definidos."))
        
        parametro = hr_parameter.browse(cr, uid, parametro_id, context=None)[0]
        if tipo_parametro == 'concepto_reposo_remunerado':
            return parametro.concept_reposo_remunerado_id
            
        if tipo_parametro == 'concepto_un_tercio_sueldo':
            return parametro.concept_un_tercio_sueldo_id
            
        if tipo_parametro == 'concepto_dos_tercios_sueldo':
            return parametro.concept_dos_tercios_sueldo_id
        
        if tipo_parametro == 'cantidad_dias_reposo_medico_paga_empresa':
            cantidad_dias = parametro.cantidad_dias_reposo_medico_paga_empresa
            if cantidad_dias <= 0:
                raise osv.except_osv(_('Error'), _('El parametro de cantidad de dias de reposo medico que paga la empresa debe ser mayor o igual a 1 dia. Contacte al administrador del sistema'))
            return cantidad_dias
        
        if tipo_parametro == 'correos_notificaciones_reposos':
            return parametro.correos_notificaciones_reposos

    def _enviar_correo(self, cr, uid, asunto, mensaje, destinatarios, context):
        """
        Envia un mensaje de correo para notificar de los reposos para los
        cuales no habia un proceso de nomina en el cual cargarlos.
        """

        smtp_client = self.pool.get('email.smtpclient')
        smtp_server = smtp_client.search(cr, uid, [('active','=',True),
                                        ('state','=','confirm')], context=False)
        if smtp_server:
            correos = destinatarios.split(',')

            if correos:
                smtp_server_id = smtp_server[0]
                server_name = smtp_client.browse(cr, uid, smtp_server_id).server
                email = {
                        'server_id': smtp_server_id,
                        'name': (asunto),
                        'server': server_name,
                        'body': (mensaje),
                        'to': correos,
                        'from': uid,
                        }
                smtp_client.send_email(cr, uid, smtp_server_id, email['to'], 
                                       email['name'], email['body'])
                return True
        return False

    def _eliminar_historico_y_lineas_conceptos_variables (self, cr, uid, reposos, context=None):
        """
        1)Eliminar lineas de conceptos variables (hr.variable.concept.line) que tengan referencia
        a un solo reposo
        2) Si la linea tiene referencia a mas de un reposo se elimina la referencia a dicho reposo
        y se actualiza el valor de la linea. 
        3)Eliminar historico de conceptos variables (sisb.reposos.medicos.conceptos.variables.historico)
        de los reposos.
        """
        if context is None:
            context={}
        context.update({ELIMINAR_LINEAS_REPOSO: True})
        context.update({ES_LINEA_REPOSO_MEDICO: True})
        context.update({'unlink': True}) #Para eliminar historico
        
        linea_obj = self.pool.get('hr.variable.concept.line')
        reposo_conceptos_rel = self.pool.get('sisb.reposos.medicos.conceptos.variables.rel')
        variable_concepts_obj = self.pool.get('hr.variable.concepts')
        historico_obj = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')
        
        reposos_ids = [reposo.id for reposo in reposos]
        conceptos_variables_ids = variable_concepts_obj.search(cr, uid, [('state','in',['draft','load','review']),
                                                                ('para_reposos_medicos','=',True)], context=None)
        historico_ids = historico_obj.search(cr, uid, [('reposo_medico_id','in',reposos_ids),
                                                      ('variable_concepts_id','in',conceptos_variables_ids)], 
                                                      order='id desc')
        lineas_ids = linea_obj.search(cr, uid, [('variable_concept_id','in',conceptos_variables_ids)])
        lineas_brw = [linea for linea in linea_obj.browse(cr, uid, lineas_ids, context=None)
                            if set(linea.reposo_medico_ids).intersection(reposos)]
        
        if context.get('wizard'):
            #Si viene del wizard se eliminan todos los registros relacionados
            #con los reposos.
            lineas_eliminar_ids = [linea.id for linea in lineas_brw]
            conceptos_variables_ids = list(set([linea.variable_concept_id.id for linea in lineas_brw]))
            if conceptos_variables_ids:
                reposo_conceptos_rel.delete(cr, reposos_ids, conceptos_variables_ids)
            linea_obj.unlink(cr, uid, lineas_eliminar_ids, context)
        else:
            reposo_lineas = {}
            for linea in lineas_brw:
                for reposo in linea.reposo_medico_ids:
                    if reposo.id in reposo_lineas:
                        reposo_lineas[reposo.id].extend([linea])
                    else:
                        reposo_lineas[reposo.id] = [linea]

            lineas_eliminar_ids = []
            reposo_id = reposos_ids[0]
            if reposo_lineas:
                for linea in reposo_lineas[reposo_id]:
                    if len(linea.reposo_medico_ids) == 1:
                        #Si la linea esta asociada a un solo reposo se elimina
                        lineas_eliminar_ids.append(linea.id)
                    else:
                        #Si la linea esta asociada a mas de un reposo se actualiza
                        #el valor de la linea restando el valor del reposo correspondiente
                        reposo_medico_ids = [reposo.id for reposo in linea.reposo_medico_ids]
                        reposo_medico_ids.remove(reposo_id)
                        values = {
                            'variable_concept_id': linea.variable_concept_id.id,
                            'employee_id': reposo.trabajador_id.id,
                            'concept_id': linea.concept_id.id,
                            'code_concept': linea.code_concept,
                            'type': 'qty',
                            'value': linea.value - reposo_conceptos_rel.get_value(cr, reposo_id, linea.variable_concept_id.id, linea.concept_id.id),
                            'reposo_medico_ids': [(6,0,reposo_medico_ids)],
                        }
                        linea_obj.write(cr, uid, [linea.id], values, context)
                    reposo_conceptos_rel.delete(cr, reposo_id, linea.variable_concept_id.id, linea.concept_id.id)
            
                linea_obj.unlink(cr, uid, lineas_eliminar_ids, context)
        
        historico_obj.unlink(cr, uid, historico_ids, context)
        context.pop('unlink')
        return True
    
    def _get_procesos_reposos(self, cr, uid, hr_payroll, reposos, context=None):
        """
        Devuelve un diccionario con clave el id del reposo y valor
        el id de proceso en el que se cargaran los conceptos variables del
        reposo. Si no existe un proceso en el cual cargar el reposo, este
        no se incluira en el resultado.
        """
        proceso_obj = self.pool.get('hr.process')
        proceso_ids = proceso_obj.search(cr, uid, [('type','=','nomina'),('state','in',['load', 'review'])], context=None)
        procesos = proceso_obj.browse(cr, uid, proceso_ids, context)
        procesos_reposos = {}
        
        for proceso in procesos:
            if len(procesos_reposos) == len(reposos):
                break;
            if hr_payroll in proceso.hr_payroll_ids:
                for reposo in reposos:
                    if (not proceso.hr_sede_ids or reposo.trabajador_id.hr_sede_id in \
                      proceso.hr_sede_ids) and (not proceso.hr_employee_ids \
                      or reposo.trabajador_id in proceso.hr_employee_ids):
                        procesos_reposos[reposo.id] = proceso.id
        
        if not procesos_reposos:
            raise osv.except_osv(_('Error'), _('No existe un proceso en el cual cargar los reposos medicos para los trabajadores de la nomina %s') % hr_payroll.name)
        return procesos_reposos 
        
    def _get_conceptos_variables_procesos(self, cr, uid, procesos_ids, context=None):
        """
        Devuelve un diccionario con clave el id del proceso y valor el id de los
        conceptos variables en los cuales se cargaran las lineas asociadas a los reposos.
        """
        conceptos_variables_obj =  self.pool.get('hr.variable.concepts')
        conceptos_variables_reposos = {}
        
        for proceso_id in procesos_ids:
            conceptos_variables_id = conceptos_variables_obj.search(cr, uid, 
                                 [('hr_process_id','=',proceso_id),
                                 ('para_reposos_medicos','=','True'),
                                 ('state','in',['draft','load','review'])], context=None)
            
            if not conceptos_variables_id:
                values = {
                    'para_reposos_medicos': True,
                    'hr_process_id': proceso_id,
                }
                self._logger.info("Creando conceptos variables de reposos medicos para el proceso %s",proceso_id )
                conceptos_variables_id = conceptos_variables_obj.create(cr, uid, values, context)
                conceptos_variables_reposos[proceso_id] = conceptos_variables_id
            else:
                conceptos_variables_reposos[proceso_id] = conceptos_variables_id[0]
            
        return conceptos_variables_reposos
    
    def crear_lineas_conceptos_variables(self, cr, uid, ids, context=None, reposo_medico=None):
        """
        Crear lineas de concepto variable para 'reposo_medico'. En caso de que 
        sea 'None' se crean las lineas para todos los reposos que esten en estado
        'approved' y pertenezcan a la nomina indicada en el wizard.
        """
        if context is None:
            context={}
        context.update({ES_LINEA_REPOSO_MEDICO: True})
        
        context['wizard'] = False;
        reposo_medico_obj =  self.pool.get('sisb.reposos.medicos')
        reposo_conceptos_variables_historico = self.pool.get('sisb.reposos.medicos.conceptos.variables.historico')

        
        if reposo_medico:
            """Si proviene del boton de creacion de conceptos variables en el reposo"""
            reposo_hijo_id = reposo_medico_obj.search(cr, uid, [('parent_id','=',reposo_medico.id)], context=context)
            if reposo_hijo_id:
                historico_hijo = reposo_conceptos_variables_historico.search(cr, uid, 
                                                            [('reposo_medico_id','=', reposo_hijo_id),], 
                                                            context=context)
                if historico_hijo:
                    raise osv.except_osv(_('ERROR'), _('No se pueden crear los conceptos variables para \
                                este reposo. Ya se crearon los conceptos variables para reposo "%s".') 
                                % reposo_medico_obj.name_get(cr, uid, reposo_hijo_id, context=context)[0][1])
            
            if reposo_medico.parent_id:
                reposo_padre = reposo_medico_obj.browse(cr, uid, reposo_medico.parent_id.id, context=context)
                if reposo_padre.state == 'approved':
                    historico_id = reposo_conceptos_variables_historico.search(cr, uid, 
                                                            [('reposo_medico_id','=',reposo_padre.id),
                                                            '|',('dias_reposo_acumulados','>=',reposo_padre.dias_reposo),
                                                            ('reposo_solapado_completado','=',True)],
                                                            context=context)
                    if not historico_id:
                        raise osv.except_osv(_('ERROR'), _('No se pueden crear los conceptos variables para \
                                este reposo. Primero debe crear los conceptos variables del reposo padre.'))
            nomina = reposo_medico.trabajador_id.hr_payroll_id
            reposos_medicos = [reposo_medico]
        else:
            """Si proviene del wizard de creacion de conceptos variables para reposos"""
            context['wizard'] = True
            nomina = self.browse(cr, uid, ids[0], context).hr_payroll_id
            sedes = self.browse(cr, uid, ids[0], context).hr_sede_ids
            if not sedes:
                raise osv.except_osv(_('Error'), _('Debe seleccionar al menos una sede'))
            
            sede_ids = [sede.id for sede in sedes]
            reposos_medicos_ids = reposo_medico_obj.search(cr, uid, [('payroll_id','=',nomina.id),('sede_id','in',sede_ids),('state','=','approved')], order="fecha_inicio asc")
            reposos_medicos = [reposo for reposo in reposo_medico_obj.browse(cr, uid, reposos_medicos_ids, context)]

            if not reposos_medicos:
                raise osv.except_osv(_('Error'), _('No existen trabajadores con reposos aprobados en la nomina y sede(s) seleccionadas'))
        
        linea_concepto_variable = self.pool.get('hr.variable.concept.line')
        reposo_conceptos_rel = self.pool.get('sisb.reposos.medicos.conceptos.variables.rel')
        
        procesos_reposos = self._get_procesos_reposos(cr, uid, nomina, reposos_medicos, context)
        procesos_ids =  list(set(procesos_reposos.values()))
        
        conceptos_variables_procesos = self._get_conceptos_variables_procesos(cr, uid, procesos_ids, context)
        conceptos_variables_ids = list(set(conceptos_variables_procesos.values()))
        
        dias_periodo = nomina.period_days
        if not dias_periodo:
            raise osv.except_osv(_('Error'), _("La nomina no tiene un valor establecido para 'Dias de periodo'"))
        
        concepto_reposo_remunerado = self._get_parameter_id(cr, uid, ids, 'concepto_reposo_remunerado')
        concepto_un_tercio_sueldo = self._get_parameter_id(cr, uid, ids, 'concepto_un_tercio_sueldo')
        concepto_dos_tercios_sueldo = self._get_parameter_id(cr, uid, ids, 'concepto_dos_tercios_sueldo')
        cantidad_dias_reposo_medico_paga_empresa = self._get_parameter_id(cr, uid, ids, 'cantidad_dias_reposo_medico_paga_empresa')
        correos_notificaciones_reposos = self._get_parameter_id(cr, uid, ids, 'correos_notificaciones_reposos')
        
        self._logger.info("Eliminando el historico y las lineas de conceptos variables para los reposos medicos" )
        self._eliminar_historico_y_lineas_conceptos_variables(cr, uid, reposos_medicos, context)
        
        reposos_no_cargados_por_sede = {}
        self._logger.info("Inicia la creacion de las lineas de concepto variable para los reposo medicos de '%s'" % (nomina.name,))
        for reposo in reposos_medicos:
            proceso_id = procesos_reposos.get(reposo.id)
            nombre_reposo = reposo.name_get(reposo.id)[0][1]
            if not proceso_id:
                sede = reposo.trabajador_id.hr_sede_id.name

                if not sede in reposos_no_cargados_por_sede:
                    reposos_no_cargados_por_sede[sede] = [nombre_reposo]
                else:
                    reposos_no_cargados_por_sede[sede] = reposos_no_cargados_por_sede[sede] + [nombre_reposo]
                
                self._logger.warning("%s: No se crearon lineas de concepto variable. No existe un proceso en estado 'Cargado' o 'Procesado' donde cargar el reposo." % (nombre_reposo,))
            else:
                conceptos_variables_id = conceptos_variables_procesos[proceso_id]
                values = {
                    'variable_concept_id': conceptos_variables_id,
                    'employee_id': reposo.trabajador_id.id,
                    'concept_id': None,
                    'code_concept':None,
                    'type': 'qty',
                    'value': None,
                    'reposo_medico_ids': [(4,reposo.id)],
                }
                dias_a_cargar = 0
                dias_acumulados = 0
                dias_cargados = 0
                reposo_valido = True
                reposo_solapado = False
                dias_solapados = 0
                self._logger.info("%s: Inicia la creacion de las lineas de concepto variable" % (nombre_reposo,))
                if not reposo.parent_id:  #Reposos no continuos
                    if reposo.dias_reposo <= cantidad_dias_reposo_medico_paga_empresa:
                        values['value'] = reposo.dias_reposo
                        values['concept_id'] = concepto_reposo_remunerado.id
                        values['code_concept'] = concepto_reposo_remunerado.code
                        self._logger.debug("%s: Creando linea de concepto variable '%s %s'" % (nombre_reposo, concepto_reposo_remunerado.code, concepto_reposo_remunerado.name))
                        linea_concepto_variable.create(cr, uid, values, context)
                        reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, concepto_reposo_remunerado.id, values['value'])
                        dias_cargados = reposo.dias_reposo
                        
                        
                    else:
                        historico_ids = reposo_conceptos_variables_historico.search(cr, uid, [('reposo_medico_id','=',reposo.id)], order='variable_concepts_id desc' )
                        if historico_ids:
                            dias_acumulados = reposo_conceptos_variables_historico.browse(cr, uid, historico_ids[0], context=None).dias_reposo_acumulados
                            dias_a_cargar = reposo.dias_reposo - dias_acumulados
                            if dias_a_cargar > dias_periodo:
                                dias_a_cargar = dias_periodo
                        
                        else:
                            values['value'] = cantidad_dias_reposo_medico_paga_empresa
                            values['concept_id'] = concepto_reposo_remunerado.id
                            values['code_concept'] = concepto_reposo_remunerado.code
                            self._logger.debug("%s: Creando linea de concepto variable '%s %s'" % (nombre_reposo, concepto_reposo_remunerado.code, concepto_reposo_remunerado.name))
                            linea_concepto_variable.create(cr, uid, values, context)
                            reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, concepto_reposo_remunerado.id, values['value'])
                            
                            dias_cargados = cantidad_dias_reposo_medico_paga_empresa
                            dias_a_cargar = reposo.dias_reposo - dias_cargados
                            
                            if dias_a_cargar + dias_cargados > dias_periodo:
                                dias_a_cargar = dias_periodo - dias_cargados
                        
                        if dias_a_cargar > 0:
                            values['value'] = dias_a_cargar
                            for concepto in [concepto_un_tercio_sueldo, concepto_dos_tercios_sueldo]:
                                values['concept_id'] = concepto.id
                                values['code_concept'] = concepto.code
                                self._logger.debug("%s: Creando linea de concepto variable '%s %s'" % (nombre_reposo, concepto.code, concepto.name))
                                linea_concepto_variable.create(cr, uid, values, context)
                            reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, [concepto_un_tercio_sueldo.id, concepto_dos_tercios_sueldo.id], values['value'])
                        dias_cargados += dias_a_cargar
                   
                else:  #Reposos continuos
                    if reposo.fecha_inicio < reposo.parent_id.fecha_reintegro:
                        reposo_solapado = True
                        inicio_solapa = datetime.strptime(reposo.fecha_inicio,"%Y-%m-%d")
                        fin_solapa = datetime.strptime(reposo.parent_id.fecha_reintegro,"%Y-%m-%d")
                        dias_solapados = (fin_solapa - inicio_solapa).days
                        dias_a_cargar = reposo.dias_reposo - dias_solapados
                    else:
                        dias_a_cargar = reposo.dias_reposo

                    historico_ids = reposo_conceptos_variables_historico.search(cr, uid, 
                                                                        [('reposo_medico_id','=',reposo.id)], 
                                                                        order='dias_reposo_acumulados desc' )
                    
                    if historico_ids:
                        dias_acumulados = reposo_conceptos_variables_historico.browse(cr, uid, historico_ids[0], context=None).dias_reposo_acumulados
                        dias_a_cargar -= dias_acumulados
                    
                    if reposo.parent_id.state == 'approved': #Si no se ha terminado de pagar el reposo padre
                        historico_padres_ids = reposo_conceptos_variables_historico.search(cr, uid, 
                                                                        [('trabajador_id','=',reposo.trabajador_id.id),
                                                                        ('variable_concepts_id','=',conceptos_variables_id)], 
                                                                        order='variable_concepts_id desc')
                        if historico_padres_ids:
                            historico_padres = reposo_conceptos_variables_historico.browse(cr, uid, historico_padres_ids, 
                                                                                    context=None)
                                                                                    
                            total_dias_cargados = sum([linea.dias_reposo_cargados for linea in historico_padres])
                            if total_dias_cargados >= dias_periodo or historico_padres[0].dias_reposo_acumulados < historico_padres[0].dias_reposo:
                                reposo_valido = False
                            else:
                                tope = dias_periodo - total_dias_cargados
                                if reposo.dias_reposo > tope:
                                    dias_a_cargar = tope
                                else:
                                    if reposo_solapado:
                                        dias_a_cargar = reposo.dias_reposo - dias_solapados
                                    else:
                                        dias_a_cargar = reposo.dias_reposo
                                
                                values['value'] = dias_a_cargar
                                linea_concepto_reposo_remunerado_brw = self.get_lineas_concepto_variable(cr, uid, 
                                            reposo.trabajador_id.id, conceptos_variables_id,
                                            concepto_reposo_remunerado.id,context)
                                if linea_concepto_reposo_remunerado_brw:
                                    linea_concepto_reposo_remunerado = linea_concepto_reposo_remunerado_brw[0]

  
                                    if linea_concepto_reposo_remunerado.value < cantidad_dias_reposo_medico_paga_empresa:
                                        reposo_medico_ids = [reposo_medico.id for reposo_medico in linea_concepto_reposo_remunerado.reposo_medico_ids]
                                        reposo_medico_ids.append(reposo.id)
                                        dias_reposo_remunerado_faltante = cantidad_dias_reposo_medico_paga_empresa - linea_concepto_reposo_remunerado.value
                                        
                                        values['value'] = linea_concepto_reposo_remunerado.value
                                        if dias_reposo_remunerado_faltante <= dias_a_cargar:
                                            values['value'] += dias_reposo_remunerado_faltante
                                            dias_linea = dias_reposo_remunerado_faltante
                                        else:
                                            values['value'] += dias_a_cargar
                                            dias_linea = dias_a_cargar
                                        
                                        values['concept_id'] = concepto_reposo_remunerado.id
                                        values['code_concept'] = concepto_reposo_remunerado.code
                                        values['reposo_medico_ids'] = [(6,0,reposo_medico_ids)]
                                        self._logger.debug("%s: Creando linea de concepto variable (Arrastre)'%s %s'" % (nombre_reposo, concepto_reposo_remunerado.code, concepto_reposo_remunerado.name))
                                        linea_concepto_variable.write(cr, uid, [linea_concepto_reposo_remunerado.id], values, context)
                                        
                                        reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, concepto_reposo_remunerado.id, dias_linea)
                                       
                                        values['value'] = dias_a_cargar - dias_reposo_remunerado_faltante

                                if values['value'] > 0:  # Si faltan dias para cargar.
                                    lineas_padre = self.get_lineas_concepto_variable(cr, uid, 
                                                reposo.trabajador_id.id, conceptos_variables_id,
                                                [concepto_un_tercio_sueldo.id, concepto_dos_tercios_sueldo.id],
                                                context)

                                    if lineas_padre:
                                        values['value'] += lineas_padre[0].value
                                        reposo_medico_ids = [reposo_medico.id for reposo_medico in lineas_padre[0].reposo_medico_ids]
                                        reposo_medico_ids.append(reposo.id)
                                        for linea_padre in lineas_padre:
                                            values['concept_id'] = linea_padre.concept_id.id
                                            values['code_concept'] = linea_padre.code_concept
                                            values['reposo_medico_ids'] = [(6,0,reposo_medico_ids)]
                                            self._logger.debug("%s: Creando linea de concepto variable (Arrastre)'%s %s'" % (nombre_reposo, linea_padre.code_concept, linea_padre.concept_id.name))
                                            linea_concepto_variable.write(cr, uid, [linea_padre.id], values, context)
                                            reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, linea_padre.concept_id.id, dias_a_cargar)

                                    else:
                                        values['reposo_medico_ids'] = [(6,0,[reposo.id])]
                                        for concepto in [concepto_un_tercio_sueldo, concepto_dos_tercios_sueldo]:
                                            values['concept_id'] = concepto.id
                                            values['code_concept'] = concepto.code
                                            self._logger.debug("%s: Creando linea de concepto variable '%s %s'" % (nombre_reposo, concepto.code, concepto.name))
                                            linea_concepto_variable.create(cr, uid, values, context)
                                            reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, concepto.id, values['value'])

                                
                    elif reposo.parent_id.state == 'done':
                        if dias_a_cargar > dias_periodo:
                            dias_a_cargar = dias_periodo
                        values['value'] = dias_a_cargar
                        
                        reposo_padre = reposo.parent_id
                        reposos_padre_ids = [reposo_padre.id]
                        while reposo_padre:
                            reposo_padre = reposo_padre.parent_id
                            if reposo_padre:
                                reposos_padre_ids.append(reposo_padre.id)
                            
                            
                        historico_padres_ids = reposo_conceptos_variables_historico.search(cr, uid, 
                                                                        [('trabajador_id','=',reposo.trabajador_id.id),
                                                                        ('reposo_medico_id','in',reposos_padre_ids)],)
                        if historico_padres_ids:
                            historico_padres = reposo_conceptos_variables_historico.browse(cr, uid, historico_padres_ids, 
                                                                                    context=None)
                                                                                    
                            total_dias_cargados = sum([linea.dias_reposo_cargados for linea in historico_padres])
                            if total_dias_cargados < cantidad_dias_reposo_medico_paga_empresa:
                                values['value'] = cantidad_dias_reposo_medico_paga_empresa - total_dias_cargados
                                values['concept_id'] = concepto_reposo_remunerado.id
                                values['code_concept'] = concepto_reposo_remunerado.code
                                self._logger.debug("%s: Creando linea de concepto variable (Arrastre)'%s %s'" % (nombre_reposo, concepto_reposo_remunerado.code, concepto_reposo_remunerado.name))
                                linea_concepto_variable.create(cr, uid, values, context)
                                reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, concepto_reposo_remunerado.id, values['value'])

                                values['value'] = dias_a_cargar - values['value']
                        
                        if values['value'] > 0:
                            for concepto in [concepto_un_tercio_sueldo, concepto_dos_tercios_sueldo]:
                                values['concept_id'] = concepto.id
                                values['code_concept'] = concepto.code
                                self._logger.debug("%s: Creando linea de concepto variable '%s %s'" % (nombre_reposo, concepto.code, concepto.name))
                                linea_concepto_variable.create(cr, uid, values, context)
                                reposo_conceptos_rel.set_value(cr, reposo.id, conceptos_variables_id, concepto.id, values['value'])
                    
                    dias_cargados = dias_a_cargar
                
                if reposo_valido:
                    self._logger.info("%s: Creando historico de conceptos variables" % (nombre_reposo,))
                    values =  {
                        'reposo_medico_id': reposo.id,
                        'variable_concepts_id': conceptos_variables_id, 
                        'dias_reposo': reposo.dias_reposo, 
                        'dias_reposo_cargados': dias_cargados, 
                        'dias_reposo_acumulados': dias_acumulados + dias_cargados
                    }
                    if reposo_solapado:
                        #Se marca el reposo para pasar a estado 'done' en la aprobacion del proceso
                        
                        if values['dias_reposo_acumulados'] >= reposo.dias_reposo - dias_solapados:
                            values['reposo_solapado_completado'] = True
                    
                    reposo_conceptos_variables_historico.create(cr, uid, values)
                    self._logger.info("%s: Se crearon lineas de concepto variable para el reposo." % (nombre_reposo,))

        if reposos_no_cargados_por_sede:
            asunto = "Reposos Medicos " + nomina.name + " - Conceptos Variables de  no cargados"
            mensaje = "<b>No existe un proceso de nomina en estado 'Cargado' o 'Procesado' donde cargar "
            mensaje += "los conceptos variables para los siguientes reposos.</b><br><br>"
            for sede, reposos in reposos_no_cargados_por_sede.iteritems():
                mensaje += "<b>" + sede + ":" + "</b><br>"
                for reposo in reposos:
                    mensaje += "&nbsp;~&nbsp;" + reposo + ".<br>"
                mensaje += "<br>"

            self._enviar_correo(cr, uid, asunto, mensaje, correos_notificaciones_reposos, context)

        self._logger.info("Culmino la creacion de las lineas de concepto variable para los reposo medicos de '%s'" % (nomina.name,))
        
        if context['wizard']:
            context.pop('wizard')
            return self.write(cr, uid, ids, {'ejecutado': True}, context)
        return True

sisb_reposos_medicos_conceptos_variables()
