# -*- coding: utf-8 -*-
from osv import osv, fields
import decimal_precision as dp
from datetime import datetime, date, time, timedelta
import calendar
import netsvc
import pooler
from tools.translate import _
import time

(MON, TUE, WED, THU, FRI, SAT, SUN) = range(7)

class account_transf_int(osv.osv):
    _name = 'account.transf.int'
    
    def _get_amount(self, cr, uid, ids, name, args, context=None):
        #~ Esta funcion calcula el saldo de las lineas de transferencias
        precision_obj = self.pool.get('decimal.precision')
        result = {}
        amount = 0.00
        transf_brw = self.browse(cr, uid, ids, context=context)
        for header in transf_brw:
            for move in header.line_ids:
                    amount += move.amount
            result[header.id] = round(amount, precision_obj.precision_get(cr, uid, 'Account'))
        return result
        
    
    _columns = {
        'name': fields.char('Concepto',size=128),
        'date_reg': fields.date('Fecha Transaccion'),
        'date': fields.date('Fecha Registro'),
        'period_id': fields.many2one('account.period','Periodo'),
        'ref': fields.char('Referencia',size=64),
        'user_id': fields.many2one('res.users','Responsable'),
        'amount_check': fields.float('Monto'),
        'amount': fields.function(_get_amount, method=True, type='float', string='Total'),
        'line_ids': fields.one2many('account.transf.int.line', 'transf_id', 'Lineas', ondelete='cascade',readonly=True, states={'draft':[('readonly',False)]}),
        'note': fields.text('Nota'),
        'state':fields.selection([('draft','Borrador'),('confirm','Confirmada'),('done','Realizado')]),
    }
    
    _defaults = {
        'state':'draft',
        'user_id': lambda self, cr, uid, ctx: uid,
        'date': lambda *a: time.strftime('%Y-%m-%d'),
        'ref':'/',
        }
    #~ Suma los dias habiles a una fecha, dependiendo de la cantidad de dias que se le envie 
    def addworkdays(self, start, days, holidays=(), workdays=(MON,TUE,WED,THU,FRI)):
        weeks, days = divmod(days, len(workdays))
        result = start + timedelta(weeks=weeks)
        lo, hi = min(start, result), max(start, result)
        count = len([h for h in holidays if h >= lo and h <= hi])
        days += count * (-1 if days < 0 else 1)
        for _ in range(days):
            result += timedelta(days=1)
            while result in holidays or result.weekday() not in workdays:
                result += timedelta(days=1)
        return result
    
    
    def validate_date(self, cr, uid, values, context=None):
        #~ Busca la fecha fin del mes anterior
        today = date.today()
        if today.month == 1:
            dateMonthEnd = date(today.year-1, 12, 31)
        else:
            dateMonthEnd=date(today.year, today.month-1, calendar.monthrange(today.year, today.month-1)[1])
        #~ Se le suman 5 días habiles
        dateExtendEnd = self.addworkdays(dateMonthEnd, 5)
        #~ Busca los periodos de las fechas
        ap_obj = self.pool.get("account.period")
        period_id = ap_obj.search(cr, uid, ['|','&',('date_start', '<=', values['date_reg']), ('date_stop', '>=', values['date_reg']),'&',('date_start', '<=', str(today.strftime('%Y-%m-%d'))),('date_stop', '>=', str(today.strftime('%Y-%m-%d')))])
        #~ print "Period Id: ",period_id
        #~ TODO: Descomentar cuando se necesite la validacion
        #~ if today > dateExtendEnd:
            #~ if len(period_id) > 1:
                #~ raise osv.except_osv(_('Error !'), _('No puede crear una transferencia del mes anterior despues de los primeros 5 dias habiles del mes'))
        if values['date_reg'] > time.strftime('%Y-%m-%d'):
            raise osv.except_osv(_('Error !'), _('No puede crear un documento con una fecha mayor a la actual'))
        return True
        
    def unlink(self, cr, uid, ids, context=None):
        state = self.read(cr, uid, ids, ['state'])
        if state[0]['state'] == 'done':
            raise osv.except_osv(_('Error !'), _('No puede eliminar un documento aprobado'))
        return super(account_transf_int, self).unlink(cr, uid, ids, context=context)
        
    def create(self, cr, uid, values, context=None):
        self.validate_date(cr, uid, values, context=context)
        return super(account_transf_int, self).create(cr, uid, values, context=context)
        
    def write(self, cr, uid, ids, values, context=None):
        if values is not None and values.get('date_reg',False):
            self.validate_date(cr, uid, {'date_reg': values['date_reg']}, context=context)
        return super(account_transf_int, self).write(cr, uid, ids, values, context=context)
    
    def dummy(self, cr, uid, ids, context=None):
        return True 
    
    def copy(self, cr, uid, ids, defaults, context=None):
        raise osv.except_osv(_('Error !'), _('No esta permitido duplicar el documento'))
        return True
    
    
    def draft_to_confirm(self, cr, uid, ids, context=None):
        trans = self.browse(cr, uid, ids, context=context)
        for h in trans:
            if not h.line_ids:
                raise osv.except_osv(_('Error !'), _('No se puede confirmar un documento sin lineas'))
        self.write(cr, uid, ids, {'state':'confirm'})
        return True
    
    def confirm_to_draft(self, cr, uid, ids, context=None):
        self.write(cr, uid, ids, {'state':'draft'})
        return True
    
    def done(self, cr, uid, ids, context=None):
        trans = self.browse(cr, uid, ids, context=context)
        for h in trans:
            if h.amount_check != h.amount:
                raise osv.except_osv(_('Error !'), _('El monto %s no es igual al monto total %s del documento'%(h.amount_check,h.amount)))
            if not h.line_ids:
                raise osv.except_osv(_('Error !'), _('El documento no contiene lineas para ser procesadas'))
            for l in h.line_ids:
                if l.date_reg > h.date_reg:
                    raise osv.except_osv(_('Error !'), _('La fecha no puede ser mayor a la fecha de registro'))

        self.create_moves(cr,uid,ids,context=context)
        number = self.pool.get('ir.sequence').get(cr, uid, 'account.transf.int.code')
        self.write(cr, uid, ids, {'state':'done','ref':number})
        return True
    
    def create_moves(self, cr, uid, ids, context=None):
        abs_obj = self.pool.get("account.bank.statement")
        absl_obj = self.pool.get("account.bank.statement.line")
        #~ Busca la cuenta puente configurada
        config_obj = self.pool.get('account.bank.statement.config')
        config_id = config_obj.search(cr, uid, [('type','in',['account']),('type_reg','in',['transferencias'])])
        if config_id:
            account_ids = config_obj.browse(cr, uid, config_id, context=context)[0].account_id.id
        else:
            raise osv.except_osv(_('Error !'), _('No ha configurado la cuenta puente para este modulo'))
        #~ Cargo el registro de la transferencia
        trans = self.browse(cr, uid, ids, context=context)
        for h in trans:
            for l in h.line_ids:
                #~ Crea el extracto bancario del banco fuente
                abs_id = abs_obj.create(cr, uid, 
                {'date':l.date_reg,
                'journal_id':l.journal_src_id.id,
                'period_id':h.period_id.id,
                'user_id':h.user_id.id,
                'balance_start':0.00,
                'balance_end_real':l.amount*-1,
                })
                #~ Crea las lineas del extracto bancario del banco fuente
                absl_id = absl_obj.create(cr, uid, 
                {'date': l.date_reg,
                'date_ref':l.date_reg,
                'name': l.type+"-"+l.ref_src,
                'ref': l.type+"-"+l.ref_src,
                'partner_id': 1,
                'type': 'general',
                'account_id': account_ids,
                'analytic_account_id': l.analytic_account_src.id,
                'amount':l.amount*-1,
                'statement_id':abs_id,
                })
                #~ Se aprueba el extracto Fuente
                abs_obj.button_confirm_bank(cr, uid, [abs_id], context=context)
                #~ Crea el extracto bancario del banco destino
                abs_id = abs_obj.create(cr, uid, 
                {'date':l.date_reg,
                'journal_id':l.journal_dst_id.id,
                'period_id':h.period_id.id,
                'user_id':h.user_id.id,
                'balance_start':0.00,
                'balance_end_real':l.amount,
                })
                #~ Crea las lineas del extracto bancario del banco destino
                absl_id = absl_obj.create(cr, uid, 
                {'date': l.date_reg,
                'date_ref':l.date_reg,
                'name': l.type+"-"+l.ref_dst,
                'ref': l.type+"-"+l.ref_dst,
                'partner_id': 1,
                'type': 'general',
                'account_id': account_ids,
                'analytic_account_id': l.analytic_account_dst.id,
                'amount':l.amount,
                'statement_id':abs_id,
                })
                #~ Se aprueba el extracto destino
                abs_obj.button_confirm_bank(cr, uid, [abs_id], context=context)
        return True
    
account_transf_int()

class account_transf_int_line(osv.osv):
    _name = "account.transf.int.line"
    
    _columns = {
        'date_reg': fields.date('Fecha'),
        'name': fields.char('Descripcion',size=64),
        'type': fields.selection([('TR','TR')]),
        'ref_src': fields.char('Ref Origen',size=64),
        'ref_dst': fields.char('Ref Destino',size=64),
        'journal_src_id': fields.many2one('account.journal','Banco Origen'),
        'journal_dst_id': fields.many2one('account.journal','Banco Destino'),
        'analytic_account_src': fields.many2one('account.analytic.account','Cta Analitica Origen'),
        'analytic_account_dst': fields.many2one('account.analytic.account','Cta Analitica Destino'),
        'amount': fields.float('Monto'),
        'transf_id': fields.many2one('account.transf.int','ID Documento'),
    }
    
    _defaults = {
    'date_reg':lambda self, cr, uid, context: context.get('date_reg', None),
    'name':lambda self, cr, uid, context: context.get('name', None),
    }
    

account_transf_int_line()
