#!/usr/bin/python
# -*- encoding: utf-8 -*-
###########################################################################
#    Copyright (C) Industrias Diana, C.A. (www.industriasdiana.gob.ve)
#    All Rights Reserved
###############Credits######################################################
#    Developed by: Industrias Diana, C.A.
##############################################################################

from osv import osv
from osv import fields
from tools.translate import _
from tools import config
import time
import datetime
from xml.etree.ElementTree import Element, SubElement, ElementTree, tostring
import sys
import base64
import decimal_precision as dp

class islr_xml_wh_doc(osv.osv):
    _inherit = "islr.xml.wh.doc"
    
    _order = "id desc"
    
    _defaults = {
        'name':lambda self,cr,uid,context : _('Withholding Income ')+time.strftime('%m/%Y')
    }
    
    def _get_amount_total_payroll(self,cr,uid,ids,name,args,context=None):
        res = {}
        for xml in self.browse(cr,uid,ids,context):
            res[xml.id]= 0.0
            for line in xml.xml_nomina_ids:
                res[xml.id] += line.wh
        return res
    
    def _get_amount_total_base_payroll(self,cr,uid,ids,name,args,context=None):
        res = {}
        for xml in self.browse(cr,uid,ids,context):
            res[xml.id]= 0.0
            for line in xml.xml_nomina_ids:
                res[xml.id] += line.base
        return res
    
    _columns = {
        'xml_ids':fields.one2many('islr.xml.wh.line','islr_xml_wh_doc','XML Document Lines', readonly=True),
        'xml_nomina_ids': fields.one2many('islr.xml.wh.line.payroll', 'islr_xml_wh_doc_id', 'XML Document Payroll Lines', readonly=True),
        'amount_total_ret_payroll':fields.function(_get_amount_total_payroll,method=True, digits=(16, 2), readonly=True, string='Withholding Income Amount Total for payroll', help="Amount Total of withholding - Payroll"),
        'amount_total_base_payroll':fields.function(_get_amount_total_base_payroll,method=True, digits=(16, 2), readonly=True, string='Without Tax Amount Total for payroll', help="Total without taxes - Payroll"),
    }
    
    #Validaciones antes de crear/actualizar la tabla islr_xml_wh_doc
    #Se asume que son campos relacionales
    def validar_campo(self, cr, user, doc_xml_id, vals, campo, context=None):
        if context is None:
            context = {}

        value = ''
        result = False
        
        if doc_xml_id: #si esta editando
            value = vals.get(campo, 'no')
            if value == 'no':
                result = getattr(self.browse(cr, user, doc_xml_id, context=context),campo).id
            else:
                result = vals.get(campo)
        else: #esta creando
            result = vals.get(campo, False)
            
        return result
        
    def validar_islr_xml_wh_doc(self, cr, user, doc_xml_id, vals, context=None):
        doc_period_id = self.validar_campo(cr, user, doc_xml_id, vals, 'period_id', context=context)
        doc_company_id = self.validar_campo(cr, user, doc_xml_id, vals, 'company_id', context=context)
        
        res = self.search(cr, user, [('company_id', '=', doc_company_id), ('period_id', '=', doc_period_id), ('id','!=',doc_xml_id)])
        if res:
            texto = ''
            for doc in self.browse(cr, user, res, context=context):
                texto += doc.name + '  '
            raise osv.except_osv(_('Error!'), _("There is already a document of that company for that period: %s ") % (texto))
        
        return True
        
    def create(self, cr, user, vals, context=None):
        if context is None:
            context = {}
        
        self.validar_islr_xml_wh_doc(cr, user, 0, vals, context)
        
        return super(islr_xml_wh_doc, self).create(cr, user, vals, context)
    
    def write(self, cr, user, ids, vals, context=None):
        if context is None:
            context = {}
            
        if not type(ids) == type([]):
            up_ids = [ids]
        else:
            up_ids = ids
        
        for doc_xml_id in self.browse(cr, user, up_ids, context=context):
            self.validar_islr_xml_wh_doc(cr, user, doc_xml_id.id, vals, context)

        return super(islr_xml_wh_doc, self).write(cr, user, ids, vals, context)
    
    def load_withholding(self, cr, uid, ids, context={}):
        """
            Buscar los parametros a consultar
            concepto base
            periodo contable
            concepto de pago de nomina
            
            Luego registrar el resultado en la tabla islr_xml_wh_line_payroll y ejecutar el super del metodo
        """
        parametros_obj = self.pool.get('hr.parameters')
        param_s_id = parametros_obj.search(cr,uid,[('type','=','concepto_sislr')])
        param_id = parametros_obj.search(cr,uid,[('type','=','concepto_islr')])
        xml_doc_brw = self.browse(cr, uid, ids, context=context)
        if not param_s_id or not param_id:
            raise osv.except_osv(_('Error!'),_('Parameters configuration incomplete!'))
            
        concepto_base = parametros_obj.browse(cr,uid, param_s_id[0]).concepto_sislr_id.code 
        concepto_pago = parametros_obj.browse(cr,uid, param_id[0]).concept_islr_id.code
        period_id = xml_doc_brw[0].period_id.id
        xml_doc_id = xml_doc_brw[0].id
        
        #Proveedores/Nomina
        #Desasociar del documento
        self.desasociar_doc(cr, uid, ids, context=context)
        
        #Asociar al documento lista de retenciones de Proveedores
        list_xml_wh_line = self.pool.get('islr.xml.wh.line').search(cr,uid,[('period_id','=',period_id)],context=context)
        self.pool.get('islr.xml.wh.line').write(cr, uid, list_xml_wh_line, {'islr_xml_wh_doc':xml_doc_id})
        
        #Asociar al documento lista de Nomina        
        sql = '''
            select name_employee as name, period_id, employee_vat, 
                sum(base_islr) as base,
                round(sum(monto_retenido_islr)*100/sum(base_islr),2) as porcent_rete,
                sum(monto_retenido_islr) as wh,
                %s as islr_xml_wh_doc_id
            from (
                select 
                he.name as name_employee,
                hpe.period_account_id as period_id,
                --se rellena con 0 hasta completar 9 digitos
                --CASE WHEN rc.code = 'VE' THEN 'V' ELSE 'E' END || lpad(he.identification_id,9,'0') as employee_vat, 
                --Se toma el valor de campo rif del empleado
                CASE WHEN he.rif is NULL THEN 'V000000000' ELSE he.rif END as employee_vat,
                hpl.total as monto_retenido_islr, 
                hpl1.total as base_islr
                from hr_payslip_line hpl
                join hr_employee he on he.id = hpl.employee_id
                join hr_payslip_line hpl1 on hpl1.slip_id = hpl.slip_id
                join hr_concept hc1 on hc1.id = hpl1.concept_id and hc1.code = '%s' -- concepto base
                join hr_department hd on hd.id = he.department_id
                join hr_concept hc on hc.id = hpl.concept_id
                join hr_process hp on hp.id = hpl.process_id
                join hr_period_process_rel hppr on hppr.process_id = hp.id
                join hr_period hpe on hpe.id = hppr.period_id
                join hr_month hm on hm.id = hpe.hr_month_id
                join res_country rc on rc.id = he.country_id
                where hpe.period_account_id = %s --periodo contable
                and hc.code in ('%s') -- concepto de pago de nomina
                order by hd.name, hc.name, hpe.name
                ) tlb_islr
            group by name_employee, period_id, employee_vat 
        '''%(xml_doc_id,concepto_base,period_id,concepto_pago)
        cr.execute(sql)
        data_xml_payroll = cr.dictfetchall()

        #Llenar tabla islr_xml_wh_line_payroll
        for item in data_xml_payroll:
            self.pool.get('islr.xml.wh.line.payroll').create(cr, uid, item, context)
            
        return True
    
    def desasociar_doc(self, cr, uid, ids, context={}):
        if context is None:
            context = {}
        
        if ids:
            xml_doc_brw = self.browse(cr, uid, ids, context=context)
            xml_doc_id = xml_doc_brw[0].id
            
            #Proveedores
            #Desasociar del documento
            list_xml_wh_line = self.pool.get('islr.xml.wh.line').search(cr,uid,[('islr_xml_wh_doc','=',xml_doc_id)],context=context)
            self.pool.get('islr.xml.wh.line').write(cr, uid, list_xml_wh_line, {'islr_xml_wh_doc':False}, context=context)
            
            #Nomina
            #Eliminar los registros de islr_xml_wh_line_payroll asociados al documento
            islr_xml_wh_line_payroll_ids = [p.id for p in xml_doc_brw[0].xml_nomina_ids]
            self.pool.get('islr.xml.wh.line.payroll').unlink(cr, uid, islr_xml_wh_line_payroll_ids, context)
        
        return True
        
    def unload_withholding(self, cr, uid, ids, context={}):
        self.desasociar_doc(cr, uid, ids)
        return True
        
    def onchange_period_id(self, cr, uid, ids, period_id=False, context={}):
        if context is None:
            context = {}
        
        if not context.get('lang', False):
            context.update({'lang':self.pool.get('res.company').browse(cr, uid, 1, context=context).partner_id.lang})
        val = {}
        update_reg = False
        
        if ids:
            up_ids = ids
            if isinstance(ids, list):
                up_ids = ids[0]
            
            doc_brw = self.browse(cr, uid, up_ids, context=context)
            if doc_brw.xml_ids or doc_brw.xml_nomina_ids:
                doc_period_id = doc_brw.period_id.id
                val = {'value': {'period_id': doc_period_id}, 'warning': {'message': _('Must press the button unlink withholding before to change the period.')}}
            else:
                update_reg = True
        else:
            if period_id:
                update_reg = True
            
        if update_reg:
            period = self.pool.get('account.period').browse(cr,uid,period_id,context=context).code
            val = {'value': {'name':_('Withholding Income ')+period}}
        
        return val
        
    def onchange_company_id(self, cr, uid, ids, company_id=False, context={}):
        if context is None:
            context = {}
        
        if not context.get('lang', False):
            context.update({'lang':self.pool.get('res.users').browse(cr, uid, 1, context= context).context_lang})
            
        val = {}
        
        if ids:
            up_ids = ids
            if isinstance(ids, list):
                up_ids = ids[0]
            
            doc_brw = self.browse(cr, uid, up_ids, context=context)
            if doc_brw.xml_ids or doc_brw.xml_nomina_ids:
                doc_company_id = doc_brw.company_id.id
                if doc_company_id != company_id:
                    val = {'value': {'company_id': doc_company_id}, 'warning': {'message': _('Must press the button unlink withholding  before to change the company.')}}

        return val
    
    #validar el proceso de confirmar
    def action_confirm1(self, cr, uid, ids, context={}):
        for xml_doc_brw in self.browse(cr, uid, ids, context=context):
            if not xml_doc_brw.xml_ids and not xml_doc_brw.xml_nomina_ids:
                raise osv.except_osv(_('Error!'), _("Retentions must be loaded before confirming"))
                
        return super(islr_xml_wh_doc, self).action_confirm1(cr, uid, ids, context)
    
    #Sobreescribir el metodo que genera el archivo xml
    def _xml(self, cr,uid,ids):
        #Modificaciones para extraer la fecha contable de la factura
        obj_ai = self.pool.get('account.invoice')
        ############################################################
        root = ''
        for id in ids:
            wh_brw = self.browse(cr,uid,id)
            
            period = wh_brw.period_id.name.split('/')
            period2 = period[1]+period[0]
            
            #Modificaciones para extraer la fecha final del periodo
            fecha_fin = datetime.datetime.strptime(wh_brw.period_id.date_stop,'%Y-%m-%d')
            #######################################################

            sql= '''SELECT partner_vat,control_number,porcent_rete,concept_code,invoice_number, SUM(COALESCE(base,0)) as base,account_invoice_id
            FROM islr_xml_wh_line 
            WHERE period_id= %s 
            GROUP BY partner_vat,control_number,porcent_rete,concept_code,invoice_number,account_invoice_id'''%(wh_brw.period_id.id)
            cr.execute(sql)
            xml_lines=cr.fetchall()

            root = Element("RelacionRetencionesISLR")
            root.attrib['RifAgente'] = wh_brw.company_id.partner_id.vat[2:]
            root.attrib['Periodo'] = period2.strip()
            for line in xml_lines:
                partner_vat,control_number,porcent_rete,concept_code,invoice_number,base,inv_id=line
                fecha_contable_ai = datetime.datetime.strptime(obj_ai.browse(cr, uid, inv_id).date_invoice,'%Y-%m-%d').strftime('%d/%m/%Y')
                detalle = SubElement(root,"DetalleRetencion")
                SubElement(detalle, "RifRetenido").text = partner_vat
                SubElement(detalle, "NumeroFactura").text =invoice_number
                SubElement(detalle, "NumeroControl").text = control_number
                SubElement(detalle, "FechaOperacion").text = fecha_contable_ai
                SubElement(detalle, "CodigoConcepto").text = concept_code
                SubElement(detalle, "MontoOperacion").text = str(base)
                SubElement(detalle, "PorcentajeRetencion").text = str(porcent_rete)
                
                
            #XML payroll
            for line_payroll in wh_brw.xml_nomina_ids:
                detalle = SubElement(root,"DetalleRetencion")
                SubElement(detalle, "RifRetenido").text = line_payroll.employee_vat
                SubElement(detalle, "NumeroFactura").text = line_payroll.invoice_number
                SubElement(detalle, "NumeroControl").text = line_payroll.control_number
                SubElement(detalle, "FechaOperacion").text = fecha_fin.strftime('%d/%m/%Y')
                SubElement(detalle, "CodigoConcepto").text = line_payroll.concept_code
                SubElement(detalle, "MontoOperacion").text = str(line_payroll.base)
                SubElement(detalle, "PorcentajeRetencion").text = str(line_payroll.porcent_rete)
            

        #~ ElementTree(root).write("/home/gabriela/openerp/Gabriela/5.0/Helados Gilda/islr_withholding/xml.xml")
        self.indent(root)
        return tostring(root,encoding="ISO-8859-1")
        
islr_xml_wh_doc()


class islr_xml_wh_line_payroll(osv.osv):
    _name = 'islr.xml.wh.line.payroll'
    _descripcion = 'Generate XML Lines of Payroll'
    
    _columns = {
        'name':fields.char('Name',128, required=True, select=True, help = "Employee Name"),
        'period_id':fields.many2one('account.period','Period', select=True, required=False, help="Period when the accounts entries were done"),
        'employee_vat': fields.char('VAT', size=10, select=True, required=True, help="Employee VAT"),
        'invoice_number': fields.char('Invoice Number',size=10,required=True, help="Number of invoice"),
        'control_number': fields.char('Control Number',size=8,required=True, help="Reference"),
        'concept_code': fields.char('Concept Code', size=10, required=True, help="Concept code"),
        'base': fields.float('Without Tax Amount', required=True, help="Taxable", digits_compute= dp.get_precision('Withhold ISLR')),
        'porcent_rete': fields.float('% Withhold', required=True, help="Percentage of Withhold", digits_compute= dp.get_precision('Withhold ISLR')),
        'wh':fields.float('Withheld Amount',required=True, help="Withheld amount to employee", digits_compute= dp.get_precision('Withhold ISLR')),
        'islr_xml_wh_doc_id': fields.many2one('islr.xml.wh.doc', 'ISLR XML Document – Payroll', ondelete='cascade', help="Income tax XML Doc - Payroll"),
    }
    
    _defaults = {
        'invoice_number': lambda *a: '0',
        'control_number': lambda *a: '0',
        'concept_code': lambda *a: '001',
    }
    
    
islr_xml_wh_line_payroll()
